@@ -1,15 +1,21 @@
 #!/bin/bash
 
 id | tee ~/lastent
-echo ######                                                                     
+echo ######
 
-IP=`ifconfig -a | grep "inet" | grep -v 127.0.0.1 | grep -v "inet6" | awk '{print $2}'`
+set -a
+source ./envs
+echo rm ./envs
+set +a
 
+sed -i "s/^LOGGER_LEVEL =.*$/LOGGER_LEVEL = $LOGGER_LEVEL/g" ~/.config/pnslocal.py
 
-sed -i "s/^EXTHOST =.*$/EXTHOST = \'$IP\'/g" ~/.config/pnslocal.py
+sed -i "s/^EXTHOST =.*$/EXTHOST = \'$HOST_IP\'/g" ~/.config/pnslocal.py
 sed -i "s/^EXTPORT =.*$/EXTPORT = $HOST_PORT/g" ~/.config/pnslocal.py
 sed -i "s/^EXTUSER =.*$/EXTUSER = \'$HOST_USER\'/g" ~/.config/pnslocal.py
 sed -i "s/^EXTPASS =.*$/EXTPASS = \'$HOST_PASS\'/g" ~/.config/pnslocal.py
+sed -i "s/^SELF_HOST =.*$/SELF_HOST = \'$SELF_HOST\'/g" ~/.config/pnslocal.py
+sed -i "s/^SELF_PORT =.*$/SELF_PORT = $SELF_PORT/g" ~/.config/pnslocal.py
 sed -i "s/^EXTRO_USER =.*$/EXTRO_USER = \'$RO_USER\'/g" ~/.config/pnslocal.py
 sed -i "s/^EXTRO_PASS =.*$/EXTRO_PASS = \'$RO_PASS\'/g" ~/.config/pnslocal.py
 
@@ -25,8 +31,14 @@ echo =====  .config/pnslocal.py >> ~/lastent
 grep ^conf  ~/.config/pnslocal.py >> ~/lastent
 grep ^EXTHOST  ~/.config/pnslocal.py >> ~/lastent
 grep ^EXTPORT  ~/.config/pnslocal.py >> ~/lastent
+grep ^EXTUSER  ~/.config/pnslocal.py >> ~/lastent
+grep ^SELF_HOST  ~/.config/pnslocal.py >> ~/lastent
+grep ^SELF_PORT  ~/.config/pnslocal.py >> ~/lastent
+grep ^SELF_USER  ~/.config/pnslocal.py >> ~/lastent
+grep ^API_BASE  ~/.config/pnslocal.py >> ~/last_entry.log
 grep ^BASE_POOLPATH  ~/.config/pnslocal.py >> ~/lastent
 grep ^SERVER_POOLPATH  ~/.config/pnslocal.py >> ~/lastent
+grep ^LOGGER_LEVEL  ~/.config/pnslocal.py >> ~/lastent
 
 rm -rf /tmp/fditest* /tmp/data
 
@@ -38,5 +50,4 @@ for i in $@; do
 if [ $i = no-run ]; then exit 0; fi;
 done
 
-/bin/sleep 99999999999
-
+exec "$@"
