@@ -25,6 +25,21 @@ logger = logging.getLogger(__name__)
 
 MdpInfo = Model['metadata']
 
+# how many columns and rows to display and where to add '...'
+COLUMN_WIDTH = 20
+COLUMN_W_BREAK = 16
+ROW_HEIGHT = 20
+ROW_H_BREAK = 16
+
+
+def mkline(line):
+    """ make a display row from a given sequence.
+
+    :line: the array or sequence line to display.
+    """
+    return list(line[:COLUMN_W_BREAK]) + ['...'] + \
+        list(line[COLUMN_W_BREAK-COLUMN_WIDTH:])
+
 
 class ArrayDataset(GenericDataset, Iterable, Shaped):
     """  Special dataset that contains a single Array Data object.
@@ -52,7 +67,7 @@ class ArrayDataset(GenericDataset, Iterable, Shaped):
         """ Initializes an ArrayDataset.
 
         Default ```None``` will initialize MetaData Parameters to their default values.
-        If ``data`` is not None and has shape (``len`` applies), ``shape`` MDP is set to the actual ``data`` shape. 
+        If ``data`` is not None and has shape (``len`` applies), ``shape`` MDP is set to the actual ``data`` shape.
         """
 
         # collect MDPs from args-turned-local-variables.
@@ -71,6 +86,7 @@ class ArrayDataset(GenericDataset, Iterable, Shaped):
         #      self.zInfo['metadata']['version'], list(metasToBeInstalled.keys()))
         # must be the first line to initiate meta
         super().__init__(zInfo=zInfo, **metasToBeInstalled, **kwds)
+        self.updateShape()
 
     # def getData(self):
     #     """ Optimized """
@@ -79,6 +95,9 @@ class ArrayDataset(GenericDataset, Iterable, Shaped):
     def setData(self, data):
         """
         """
+
+        # if issubclass(data.__class__, memoryview):
+        #    d = data
         isitr = hasattr(data, '__iter__')  # and hasattr(data, '__next__')
         if not isitr and data is not None:
             # dataWrapper initializes data as None
@@ -163,7 +182,7 @@ class ArrayDataset(GenericDataset, Iterable, Shaped):
         cn = self.__class__.__name__
         if level > 1:
 
-            s = cn + '(' + \
+            s = cn + '(' +\
                 self.meta.toString(
                     level=level, heavy=heavy,
                     tablefmt=tablefmt, tablefmt1=tablefmt1, tablefmt2=tablefmt2,
@@ -176,21 +195,51 @@ class ArrayDataset(GenericDataset, Iterable, Shaped):
             # (vs, us, ts, ds, fs, cs)
             return '%s data= %s)' % (s, vs)
 
+        html = 'html' in tablefmt.lower() or 'html' in tablefmt2.lower()
+        br = '<br>' if html else '\n'
+        if html:
+            tablefmt = tablefmt2 = 'unsafehtml'
+
         s, last = make_title_meta_l0(self, level=level, width=width, heavy=heavy,
                                      tablefmt=tablefmt, tablefmt1=tablefmt1,
                                      tablefmt2=tablefmt2, center=center,
-                                     excpt=['description'])
+                                     param_widths=param_widths,
+                                     html=html, excpt=['description'])
         width = len(last)-1
         if level == 0:
-            d = 'DATA'.center(width) + '\n' + '----'.center(width) + '\n'
+            if html:
+                d = '<center><u>%s</u></center>\n' % 'DATA' + '<hr>'
+            else:
+                d = 'DATA'.center(width) + '\n' + '----'.center(width) + '\n'
         else:
             d = ''
 
-        ds = bstr(self.data, level=level, **kwds) if matprint is None else \
-            matprint(self.data, trans=False, headers=[], tablefmt2='plain',
+        # limit max rows and columns of display
+        ls = len(self.shape)
+        if ls == 0 or \
+           ls == 1 and len(self.data) <= COLUMN_WIDTH or \
+           ls > 1 and len(self.data[0]) <= COLUMN_WIDTH and \
+           len(self.data) <= ROW_HEIGHT:
+            tdata = self.data
+        elif ls == 1:
+            tdata = mkline(self.data)
+        else:
+            # ls > 1
+            if len(self.data) <= ROW_HEIGHT:
+                tdata = [mkline(line) for line in self.data]
+            else:
+                tdata = [mkline(line) for line in self.data[:ROW_H_BREAK]]
+                tdata += [['...'] * len(tdata[0])]
+                tdata += [mkline(line)
+                          for line in self.data[ROW_H_BREAK-ROW_HEIGHT:]]
+
+        ds = bstr(tdata, level=level, **kwds) if matprint is None else \
+            matprint(tdata, trans=False, headers=[],
+                     tablefmt2='html' if html else 'plain',
                      **kwds)
-        d += lls(ds, 1000)
-        return f'{s}\n{d}\n{last}\n'
+        #d += lls(ds, 9000 if html else 2000)
+        d += ds
+        return '%s\n%s%s%s%s' % (s, d, br, last, br)
 
     string = toString
 
