@@ -1,13 +1,15 @@
 # -*- coding: utf-8 -*-
 
-from collections import ChainMap
-import os
-from copy import copy
-
 from .metadataholder import MetaDataHolder
 from .metadata import AbstractParameter, Parameter
 from .datatypes import DataTypes
 from .deserialize import Class_Look_Up
+from .finetime import FineTime
+
+from collections import ChainMap
+import os
+import datetime
+from copy import copy
 
 import logging
 # create logger
@@ -19,7 +21,8 @@ MdpInfo = {}
 
 """ Names not for mormal properties. """
 Reserved_Property_Names = ['history', 'meta', 'refs', 'dataset',
-                           'zInfo', '_MDP', 'extraMdp', 'alwaysMeta']
+                           'zInfo', '_MDP', 'extraMdp', 'alwaysMeta',
+                           'toString', 'string', 'yaml', 'tree']
 
 """ These MetaData Parameters (MDPs) and vital attrbutes are Set By Parent classes:
 | Special MDPs and attrbutes | set by parent Classes | attribute holder |
@@ -29,7 +32,7 @@ Reserved_Property_Names = ['history', 'meta', 'refs', 'dataset',
 | 'listeners' | `EventSender` | `_listener` |
 """
 MDP_Set_by_Parents = {'meta': '_meta',
-                      'description': 'description',
+                      # 'description': 'description',
                       'data': '_data',
                       'listeners': '_listeners'}
 
@@ -78,12 +81,14 @@ class Attributable(MetaDataHolder):
         # merge extra MDP there may be and Model MDPs.
         self._MDP = ChainMap(self.extraMdp, zm)
         # This will set MDP_Set_by_Parents args.
-
-        super().__init__(meta=meta, **kwds)
-        # do not set MDPs that have been set by super classes
-        for p in copy(mdps):
-            if p in self._meta:
-                mdps.pop(p)
+        # Do not use super() as it traverses MRO and calls annotatable that sets self.description
+        MetaDataHolder.__init__(self, meta=meta, **kwds)
+        # # do not set MDPs that have been set by prev classes on mro
+        # for p in copy(mdps):
+        #     if p in self._meta:
+        #         print('@@@@@@ ', p)
+        #         # mdps.pop(p)
+        # print(self.data)
         self.setParameters(mdps)
 
     def setParameters(self, params):
@@ -185,10 +190,16 @@ class Attributable(MetaDataHolder):
                 if value is None or issubclass(v_type, p_type):
                     p.setValue(value)
                     return
+                elif issubclass(v_type, (int, datetime.datetime, str)) and issubclass(p_type, FineTime):
+                    # The parameter is of type `finetime`
+                    #fmt = p['typecode']
+                    # p(value, format=None if fmt in ('Q', '') else fmt)
+                    p.setValue(value)
+                    return
                 else:
                     vs = value
                     raise TypeError(
-                        "Value %s type is %s, not %s's %s." % (vs, v_type.__name__, name, p_type.__name__))
+                        "Value %s type is %s, not %s needed by %s." % (vs, v_type.__name__, p_type.__name__, name))
         else:
             # named parameter is not in meta
 
@@ -213,8 +224,6 @@ class Attributable(MetaDataHolder):
 
         If ```self.alwaysMeta is True``` all properties of :class:`AbstractParameter` are taken as MDPs except those named in ```Reserved_Property_Names```
         """
-        # if self.__class__.__name__ == 'CompositeDataset' and name == '_STID':
-        #    __import__('pdb').set_trace()
 
         #print('setattr ' + name, value)
         if name in Reserved_Property_Names:
@@ -308,7 +317,12 @@ def value2parameter(name, value, descriptor):
                               )
     elif im['data_type'] == 'finetime':
         from .dateparameter import DateParameter
-        ret = DateParameter(value=value,
+        if issubclass(value.__class__, str):
+            fmt = ext.pop('typecode')
+            dt = FineTime(value, format=None if fmt in ('Q', '') else fmt)
+        else:
+            dt = value
+        ret = DateParameter(value=dt,
                             description=im['description'],
                             typ_=im['data_type'],
                             default=fs,
