@@ -41,7 +41,7 @@ class Classes_meta(type):
         'fdi.dataset.product': ['Product'],
         'fdi.dataset.browseproduct': ['BrowseProduct'],
         'fdi.dataset.testproducts': ['TP', 'TC', 'TM'],
-        'fdi.dataset.datatypes': ['Vector', 'Vector2D', 'Quaternion'],
+        'fdi.dataset.datatypes': ['Vector', 'Vector2D', 'Vector3D', 'Quaternion'],
         'fdi.dataset.metadata': ['AbstractParameter', 'Parameter', 'MetaData'],
         'fdi.dataset.numericparameter': ['NumericParameter', 'BooleanParameter'],
         'fdi.dataset.dateparameter': ['DateParameter'],
@@ -50,7 +50,7 @@ class Classes_meta(type):
         'fdi.dataset.mediawrapper': ['MediaWrapper'],
         'fdi.dataset.dataset': ['GenericDataset', 'CompositeDataset'],
         'fdi.dataset.tabledataset': ['TableDataset', 'IndexedTableDataset'],
-        'fdi.dataset.unstructureddataset': ['UnstrcturedDataset'],
+        'fdi.dataset.unstructureddataset': ['UnstructuredDataset'],
         'fdi.dataset.readonlydict': ['ReadOnlyDict'],
         'fdi.pal.context': ['AbstractContext', 'Context',
                             'MapContext',
@@ -184,20 +184,11 @@ class Classes_meta(type):
 
     # https://stackoverflow.com/a/1800999
     @property
-    def mapping(cls, ignore_error=False):
+    def mapping(cls):
         """ Returns the dictionary of classes allowed for deserialization, including the fdi built-ins and user added classes.
 
-        Will update the classes if the list is empty
-        Parameters
-        ----------
-
-        Returns
-        -------
         """
-        if len(cls._classes) == 0:
-            return cls.updateMapping(c=None, rerun=False, exclude=None,
-                                     verbose=False, ignore_error=ignore_error)
-        return cls._classes
+        return cls.getMapping()
 
     @mapping.setter
     def mapping(cls, c):
@@ -209,9 +200,23 @@ class Classes_meta(type):
         Returns
         -------
         """
-        raise NotImplementedError('Use Classes.updateMapping(c).')
+        raise NotImplementedError('Use Classes.updateMapping(c, **kwds).')
         cls.updateMapping(c)
 
+    def getMapping(cls, **kwds):
+        """         Will update the classes if the class list is empty
+        Parameters
+        ----------
+        passed to `updateMapping`.
+
+        Returns
+        -------
+
+        """
+        if len(cls._classes) == 0:
+            return cls.updateMapping(c=None, **kwds)
+        return cls._classes
+
 
 class Classes(metaclass=Classes_meta):
     """ A dictionary of class names and their class objects that are allowed to be deserialized.
