@@ -49,7 +49,7 @@ class Dataset(Attributable, DataContainer, Serializable, MetaDataListener):
     directly. Instead, they should inherit from one of the generic
     datasets that this package provides:
 
-    mh: GenericDataset, UnstrcturedDataset
+    mh: GenericDataset, UnstructuredDataset
     ArrayDataset.
     TableDataset or
     CompositeDataset.
@@ -97,17 +97,26 @@ class Dataset(Attributable, DataContainer, Serializable, MetaDataListener):
                     tablefmt=tablefmt, tablefmt1=tablefmt1, tablefmt2=tablefmt2,
                     level=level, width=width, param_widths=param_widths,
                     matprint=matprint, trans=trans, heavy=heavy, **kwds))
+        html = 'html' in tablefmt.lower() or 'html' in tablefmt2.lower()
+        br = '<br>' if html else '\n'
 
         s, last = make_title_meta_l0(self, level=level, width=width, heavy=heavy,
                                      tablefmt=tablefmt, tablefmt1=tablefmt1,
                                      tablefmt2=tablefmt2, center=center,
-                                     excpt=['description'])
+                                     param_widths=param_widths,
+                                     html=html, excpt=['description'])
         width = len(last) - 1
-        d = 'DATA'.center(width) + '\n' + '----'.center(width) + '\n'
+        if html:
+            d = '<center><u>%s</u></center>\n' % 'DATA'
+        else:
+            d = 'DATA'.center(width) + '\n' + '----'.center(width) + '\n'
+
         d += bstr(self.data, level=level, heavy=heavy, center=center,
                   tablefmt=tablefmt, tablefmt1=tablefmt1, tablefmt2=tablefmt2,
-                  yaml=True, **kwds) if matprint is None else \
-            matprint(self.data, level=level, trans=False, headers=[], tablefmt2='rst', heavy=heavy,
+                  yaml=True, param_widths=param_widths, html=html,
+                  **kwds) if matprint is None else \
+            matprint(self.data, level=level, trans=False, headers=[],
+                     tablefmt2='rst', heavy=heavy,
                      **kwds)
         return f'{s}\n{d}\n{last}\n'
 
@@ -130,21 +139,23 @@ class Dataset(Attributable, DataContainer, Serializable, MetaDataListener):
         return s
 
 
-def make_title_meta_l0(self, level=0, heavy=True, center=0, **kwds):
+def make_title_meta_l0(self, level=0,
+                       heavy=True, center=0, html=False, **kwds):
     """ make toString title and metadata.
 
     :heavy: use bold symbols for separaters.
     :center: 0 for no centering;  -1 for centering with metadata table; other  for ``str.center(<center>``.
     """
-
     # title
     cn = self.__class__.__name__
-    t = '*** %s (%s) ***' % (cn, self.description if hasattr(
-        self, 'description') else '')
+    desc = self.meta.get('description', '')
+    if desc:
+        desc = desc.value
+    t = ('*** <b>%s (%s)</b> ***' if html else '*** %s (%s) ***') % (cn, desc)
     tw = len(t)
     # make the table and find out the width first
-    table = mstr(self._meta, level=level, **kwds)
-    if center:
+    table = mstr(self._meta, level=level, html=html, **kwds)
+    if center and not html:
         # max separation between consequitive '\n' s
         if center == -1:
             width = max(len(x) for x in table[:600].split('\n'))
@@ -159,11 +170,19 @@ def make_title_meta_l0(self, level=0, heavy=True, center=0, **kwds):
     else:
         # beginning of a dataset that is a component of a another
         l = '_' * tw
-    m = 'META\n----\n'
+
+    br = '<br>' if html else '\n'
     if center:
-        t = t.center(width)
-        l = l.center(width)
-        m = 'META'.center(width)
+        if html:
+            t = '<center>%s</center>' % t
+            l = '<center>%s</center>' % l
+            m = '<center><u>%s</u></center>' % 'META'
+        else:
+            t = t.center(width)
+            l = l.center(width)
+            m = 'META'.center(width)
+    else:
+        m = 'META%s----%s' % (br, br)
     t += '\n'
     l += '\n'
 
@@ -172,8 +191,11 @@ def make_title_meta_l0(self, level=0, heavy=True, center=0, **kwds):
     else:
         s = l + t + ('' if level else l + m)
     s += table
-    last = "="*width if heavy else "~"*width
-    last += '\n'
+    if html:
+        last = '<hr size="3">' if heavy else '<hr>'
+    else:
+        last = "=" * width if heavy else "~" * width
+    last += br
 
     return s, last
 
@@ -185,8 +207,7 @@ class GenericDataset(Dataset, Typed, DataWrapper):
     def __init__(self, **kwds):
         """
         """
-        super(GenericDataset, self).__init__(
-            **kwds)  # initialize data, meta, unit
+        super().__init__(**kwds)  # initialize data, meta, unit
 
     def __iter__(self):
         for x in self.getData():
@@ -215,7 +236,7 @@ class CompositeDataset(MetaDataListener, AbstractComposite):
             **kwds)  # initialize _sets, meta, unit
 
     def __getstate__(self):
-        """ Can be encoded with serializableEncoder 
+        """ Can be encoded with serializableEncoder
 
         Parameter
         ---------
