@@ -1,21 +1,21 @@
 # -*- coding: utf-8 -*-
 
 from .serializable import serialize, ATTR, LEN_ATTR
-from .odict import ODict
 from .classes import Classes
-from ..utils.common import lls, trbk, guess_value
+from ..utils.common import lls
+from ..dataset.metadata import guess_value
 
 import logging
 import json
 import codecs
+import gzip
 import binascii
 import array
-import mmap
 from collections import ChainMap
 import builtins
 import urllib
-from collections import UserDict
-from collections.abc import MutableMapping as MM, MutableSequence as MS, MutableSet as MSe
+from .odict import ODict
+from collections.abc import MutableMapping as MM
 import sys
 if sys.version_info[0] >= 3:  # + 0.1 * sys.version_info[1] >= 3.3:
     PY3 = True
@@ -34,7 +34,7 @@ Serializable.
 '''
 
 
-def constructSerializable(obj, lookup=None, debug=False):
+def constructSerializable(obj, lookup=None, int_key=False, debug=False):
     """ mh: reconstruct object from the output of jason.loads().
     Recursively goes into nested class instances that are not
     encoded by default by JSONEncoder, instantiate and fill in
@@ -45,7 +45,7 @@ def constructSerializable(obj, lookup=None, debug=False):
 
     Parameters
     ----------
-
+    :useint: is a key in key-value pair has only digits characters the key will be substituted by `int(key)`.
     Returns
     -------
     """
@@ -75,7 +75,8 @@ def constructSerializable(obj, lookup=None, debug=False):
                 print(spaces + 'looping through list %d <%s>' %
                       (i, xc.__name__))
             if issubclass(xc, (list, dict)):
-                des = constructSerializable(x, lookup=lookup, debug=debug)
+                des = constructSerializable(
+                    x, lookup=lookup, int_key=int_key, debug=debug)
             else:
                 des = x
             inst.append(des)
@@ -114,13 +115,28 @@ def constructSerializable(obj, lookup=None, debug=False):
                     print(spaces + 'Instanciate hex')
                 indent -= 1
                 return inst
-            elif classname.startswith('array.array'):
-                tcode = classname.rsplit('_', 1)[1]
-                inst = array.array(tcode, binascii.a2b_hex(obj['code']))
+            elif classname == 'bytes_gz':
+                inst = gzip.decompress(binascii.a2b_base64(obj['code']))
                 if debug:
-                    print(spaces + 'Instanciate array.array')
+                    print(spaces + 'Instanciate hex_gz')
                 indent -= 1
                 return inst
+            elif classname.startswith('a.array'):
+                array_t, tcode = tuple(classname.rsplit('_', 1))
+                if array_t == 'a.array':
+                    inst = array.array(tcode, binascii.a2b_hex(obj['code']))
+                    if debug:
+                        print(spaces + 'Instanciate array.array')
+                    indent -= 1
+                    return inst
+                elif array_t == 'a.array,gz,b64':
+                    # gzip-base64
+                    inst = array.array(tcode, gzip.decompress(
+                        binascii.a2b_base64(obj['code'])))
+                    if debug:
+                        print(spaces + 'Instanciate array.array gzip base64')
+                    indent -= 1
+                    return inst
         if classname in lookup:
             # Now we have a blank instance.
             inst = lookup[classname]()
@@ -132,7 +148,8 @@ def constructSerializable(obj, lookup=None, debug=False):
             indent -= 1
             return Ellipsis
         elif classname in lookup and 'obj' in obj:
-            o = constructSerializable(obj['obj'], lookup=lookup, debug=debug)
+            o = constructSerializable(
+                obj['obj'], lookup=lookup, int_key=int_key, debug=debug)
             inst = lookup[classname](o)
             if debug:
                 print(spaces + 'Instanciate defined %s' % obj['obj'])
@@ -148,8 +165,10 @@ def constructSerializable(obj, lookup=None, debug=False):
             raise ValueError('Class %s is not known.' % classname)
     if debug:
         print(spaces + 'Go through properties of instance')
-    for (k, v) in obj.items():
+    # we might change key during iteration so save the original keys
+    for k in list(obj.keys()):
         """ loop through all key-value pairs. """
+        v = obj[k]
         if k == '_STID':
             continue
         # deserialize v
@@ -159,11 +178,12 @@ def constructSerializable(obj, lookup=None, debug=False):
                 print(spaces + '[%s]value(dict/list) <%s>: %s' %
                       (k, v.__class__.__qualname__,
                        lls(list(iter(v)), 70)))
-            desv = constructSerializable(v, lookup=lookup, debug=debug)
+            desv = constructSerializable(
+                v, lookup=lookup, int_key=int_key, debug=debug)
         else:
             if debug:
                 print(spaces + '[%s]value(simple) <%s>: %s' %
-                      (k, v.__class__.__name__, lls(v, 70)))
+                      (str(k), v.__class__.__name__, lls(v, 70)))
             if 1:
                 desv = v
             else:
@@ -177,14 +197,20 @@ def constructSerializable(obj, lookup=None, debug=False):
         icn = inst.__class__.__name__
         dcn = desv.__class__.__name__
         if issubclass(inst.__class__, (MM)):    # should be object_pairs_hook
-            if k.startswith(ATTR):
+            if issubclass(k.__class__, str) and k.startswith(ATTR):
                 k2 = k[LEN_ATTR:]
                 setattr(inst, k2, desv)
                 if debug:
                     print(spaces + 'Set attrbute to dict/usrd <%s>.%s = %s <%s>' %
                           (icn, str(k2), lls(desv, 70), dcn))
             else:
-                inst[k] = desv
+                # convert key to int if required
+                if int_key and k.isdigit():
+                    # obj == inst so delete old entry if to change key
+                    inst.pop(k)
+                    inst[int(k)] = desv
+                else:
+                    inst[k] = desv
                 if debug:
                     print(spaces + 'Set member to dict/usrd <%s>[%s] = %s <%s>' %
                           (icn, str(k), lls(desv, 70), dcn))
@@ -264,7 +290,7 @@ class IntDecoderOD(IntDecoder):
             return o
 
 
-def deserialize(js, lookup=None, debug=False, usedict=True):
+def deserialize(js, lookup=None, debug=False, usedict=True, int_key=False, verbose=False):
     """ Loads classes with _STID from the results of serialize.
 
     if usedict is True dict insted of ODict will be used.
@@ -287,7 +313,7 @@ def deserialize(js, lookup=None, debug=False, usedict=True):
             # , cls=IntDecoderOD)
             obj = json.loads(js, object_pairs_hook=ODict)
     except json.decoder.JSONDecodeError as e:
-        msg = '\nBad string to decode as JSON=====>\n%s\n<======\n%s' % (
+        msg = 'Bad JSON====>\n%s\n<====\n%s' % (
             lls(js, 500), str(e))
         logging.error(msg)
         obj = msg
@@ -297,7 +323,7 @@ def deserialize(js, lookup=None, debug=False, usedict=True):
 
     global indent
     indent = -1
-    return constructSerializable(obj, lookup=lookup, debug=debug)
+    return constructSerializable(obj, lookup=lookup, int_key=int_key, debug=debug)
 
 
 Class_Look_Up = ChainMap(Classes._classes, globals(), vars(builtins))
@@ -427,4 +453,5 @@ def deserialize_args(all_args, not_quoted=False, first_string=True, serialize_ou
                 del dese['apiargs']
             kwds = dese
             break
+    logger.debug('args %s KWDS %s' % (str(args), str(kwds)))
     return 200, args, kwds
