@@ -14,13 +14,16 @@ from .invalid import INVALID
 from ..utils.masked import masked
 from ..utils.common import grouper
 from ..utils.common import exprstrs, wls, bstr, t2l
-
+from fdi.dataset.listener import ListnerSet
 
 import cwcwidth as wcwidth
 import tabulate
 
 from itertools import zip_longest, filterfalse
 import builtins
+import array
+import datetime
+import copy
 from collections import OrderedDict, UserList
 from numbers import Number
 import logging
@@ -31,7 +34,7 @@ logger = logging.getLogger(__name__)
 tabulate.wcwidth = wcwidth
 tabulate.WIDE_CHARS_MODE = True
 tabulate.MIN_PADDING = 0
-#tabulate.PRESERVE_WHITESPACE = True
+# tabulate.PRESERVE_WHITESPACE = True
 Default_Extra_Param_Width = 10
 
 """
@@ -79,6 +82,7 @@ Parameter_Attr_Defaults = {
         typ_='',
         default=0,
         valid=None,
+        typecode=None
     ),
 
     'StringParameter': dict(
@@ -350,8 +354,8 @@ class AbstractParameter(Annotatable, Copyable, DeepEqual, DatasetEventSender, Se
         -------
 
         """
-        vs = str(self._value)
-        ds = str(self.description)
+        vs = str(self._value if hasattr(self, '_value') else '')
+        ds = str(self.description if hasattr(self, 'description') else '')
         ss = '%s' % (vs) if level else \
             '%s, "%s"' % (vs, ds)
         if alist:
@@ -375,6 +379,71 @@ class AbstractParameter(Annotatable, Copyable, DeepEqual, DatasetEventSender, Se
                            )
 
 
+def guess_value(data, parameter=False, last=str):
+    """ Returns guessed value from a string.
+
+    | input | output |
+    | ```'None'```,```'null'```,```'nul'``` any case | `None` |
+    | integer | `int()` |
+    | float | `float()` |
+    | ```'True'```, ```'False```` | `True`, `False` |
+    | string starting with ```'0x'``` | `hex()` |
+    | else | run `last`(data) |
+
+    """
+    from .numericparameter import NumericParameter, BooleanParameter
+    from .dateparameter import DateParameter
+    from .stringparameter import StringParameter
+    from .datatypes import Vector
+    from .metadata import Parameter
+    from .finetime import FineTime
+    if data is None:
+        return Parameter(value=data) if parameter else data
+    else:
+        if issubclass(data.__class__, (list, tuple, set, array.array)):
+            res = data
+            return NumericParameter(value=res) if parameter else res
+        try:
+            if issubclass(data.__class__, int):
+                res = data
+            else:
+                res = int(data)
+            return NumericParameter(value=res) if parameter else res
+        except (ValueError, TypeError):
+            try:
+                if issubclass(data.__class__, float):
+                    res = data
+                else:
+                    res = float(data)
+                return NumericParameter(value=res) if parameter else res
+            except (ValueError, TypeError):
+                # string, bytes, bool
+                if issubclass(data.__class__, bytes):
+                    res = data
+                    return NumericParameter(value=res) if parameter else res
+                if issubclass(data.__class__, bool):
+                    res = data
+                    return BooleanParameter(value=res) if parameter else res
+                if issubclass(data.__class__, (datetime.datetime, FineTime)):
+                    res = data
+                    return DateParameter(value=res) if parameter else res
+                elif data[:4].upper() in ('NONE', 'NULL', 'NUL'):
+                    return Parameter(value=None) if parameter else None
+                elif data.startswith('0x'):
+                    res = bytes.fromhex(data[2:])
+                    return NumericParameter(value=res) if parameter else res
+                elif data.upper() in ['TRUE', 'FALSE']:
+                    res = bool(data)
+                    return BooleanParameter(value=res) if parameter else res
+                elif len(data) > 16 and data[0] in '0987654321' and 'T' in data and ':' in data and '-' in data:
+                    res = FineTime(data)
+                    return DateParameter(value=res) if parameter else res
+                else:
+                    res = last(data)
+                    return Parameter(value=res) if parameter else res
+    return StringParameter('null') if parameter else None
+
+
 def make_jsonable(valid):
 
     return[t2l([k, v]) for k, v in valid.items()] if issubclass(valid.__class__, dict) else t2l(valid)
@@ -400,9 +469,8 @@ class Parameter(AbstractParameter, Typed):
         """ invoked with no argument results in a parameter of
         None value and 'UNKNOWN' description ''. typ_ DataTypes[''], which is None.
         With a signle argument: arg -> value, 'UNKNOWN'-> description. ParameterTypes-> typ_, hex values have integer typ_.
-        Unsuported parameter types will get a NotImplementedError.
 f        With two positional arguments: arg1-> value, arg2-> description. ParameterTypes['']-> typ_.
-        Unsuported parameter types will get a NotImplementedError.
+
         With three positional arguments: arg1 casted to DataTypes[arg3]-> value, arg2-> description. arg3-> typ_.
         Unsuported parameter types will get a NotImplementedError.
         Incompatible value and typ_ will get a TypeError.
@@ -532,7 +600,7 @@ f        With two positional arguments: arg1-> value, arg2-> description. Parame
         """
 
         if value is None:
-            v = self._default if hasattr(self, '_default') else value
+            v = None  # self._default if hasattr(self, '_default') else value
         else:
             v = self.checked(value)
         super().setValue(v)
@@ -794,14 +862,19 @@ f        With two positional arguments: arg1-> value, arg2-> description. Parame
                            )
 
 
+# Headers of MetaData.toString(1)
 MetaHeaders = ['name', 'value', 'unit', 'type', 'valid',
                'default', 'code', 'description']
+
+# Headers of extended MetaData.toString(1)
 ExtraAttributes = ['fits_keyword', 'id_zh_cn',
                    'description_zh_cn', 'valid_zh_cn']
 
 
 class MetaData(ParameterListener, Composite, Copyable, DatasetEventSender):
-    """ A container of named Parameters. A MetaData object can
+    """ A container of named Parameters.
+
+    A MetaData object can
     have one or more parameters, each of them stored against a
     unique name. The order of adding parameters to this container
     is important, that is: the keySet() method will return a set of
@@ -809,10 +882,11 @@ class MetaData(ParameterListener, Composite, Copyable, DatasetEventSender):
     Note that replacing a parameter with the same name,
     will keep the order. """
 
-    Table_Widths = [
-        {'name': 15, 'value': 18, 'unit': 6, 'type': 8,
-         'valid': 17, 'default': 15, 'code': 4, 'description': 17}
-    ]
+    Default_Param_Widths = {'name': 15, 'value': 18, 'unit': 6,
+                            'type': 8, 'valid': 17, 'default': 15,
+                            'code': 10, 'description': 17}
+
+    MaxDefWidth = max(Default_Param_Widths.values())
 
     def __init__(self, copy=None, defaults=None, **kwds):
         """
@@ -918,18 +992,18 @@ class MetaData(ParameterListener, Composite, Copyable, DatasetEventSender):
 
     def toString(self, level=0,
                  tablefmt='grid', tablefmt1='simple', tablefmt2='rst',
-                 extra=False, param_widths=None, width=0, **kwds):
+                 extra=False, param_widths=None, **kwds):
         """ return  string representation of metada.
 
         level: 0 is the most detailed, 2 is the least,
         tablefmt: format string in packae ``tabulate``, for level==0, tablefmt1 for level1, tablefmt2: format of 2D table data.
-        param_widths: controls how the attributes of every parameter are displayed in the table cells. If is set to -1, there is no cell-width limit. For finer control set a dictionary of parameter attitute names and how many characters wide its tsble cell is, 0 for ommiting the attributable. Default is `MetaData.Table_Widths[0]`. e.g.
-``{'name': 8, 'value': 17, 'unit': 7, 'type': 8,
-                      'valid': 20, 'default': 17, 'code': 4, 'description': 15}``
+        param_widths: controls how the attributes of every parameter are displayed in the table cells. If is set to -1, there is no cell-width limit. For finer control set a dictionary of parameter attitute names and how many characters wide its table cell is, 0 for ommiting the attributable. Default is `MetaData.Default_Param_Widths`. e.g.
+``{'name': 15, 'value': 18, 'unit': 6, 'type': 8,
+         'valid': 17, 'default': 15, 'code': 4, 'description': 17}``
         """
 
-        if param_widths is None or param_widths == 0:
-            param_widths = MetaData.Table_Widths[0]
+        html = 'html' in tablefmt.lower() or 'html' in tablefmt2.lower()
+        br = '<br>' if html else '\n'
         tab = []
         # N parameters per row for level 1
         N = 3
@@ -945,11 +1019,13 @@ class MetaData(ParameterListener, Composite, Copyable, DatasetEventSender):
             elif k == '_STID':
                 continue
             att['name'] = k
+
             # get values of line k.
             if issubclass(v.__class__, Parameter):
                 att['value'], att['unit'], att['type'], att['description'],\
                     att['default'], att['valid'], att['code'], ext = v.toString(
                         level=level, width=0 if level > 1 else 1,
+                        param_widths=param_widths,
                         tablefmt=tablefmt, tablefmt1=tablefmt1, tablefmt2=tablefmt2,
                         extra=extra,
                         alist=True)
@@ -960,7 +1036,7 @@ class MetaData(ParameterListener, Composite, Copyable, DatasetEventSender):
                 ext0 = ext
                 ext = {'fits_keyword': fk}
                 ext.update(ext0)
-            else:
+            elif issubclass(v.__class__, ListnerSet):
                 # listeners
                 lstr = '' if v is None else v.toString(level=level, alist=True)
                 if len(lstr) < 3:
@@ -971,18 +1047,26 @@ class MetaData(ParameterListener, Composite, Copyable, DatasetEventSender):
                     '\n'.join(x[2] for x in lstr)
                 att['default'], att['valid'], att['code'] = '', '', ''
                 ext = dict((n, '') for n in ext)
+            else:
+                raise ValueError('Need a `Parameter` or a `ListenerSet`, not a `%s`, to print "%s" in `MetaData`.' % (
+                    type(v).__name__, k))
 
+            # if tablefmt == 'html':
+            #    att['valid'] = att['valid'].replace('\n', '<br>')
             # generate column vallues of the line and ext headers
             # limit cell width for level=0,1.
             if level == 0:
-                if param_widths == -1:
-                    l = tuple(att[n] for n in MetaHeaders)
+                if param_widths == -1 or html:
+                    w = MetaData.MaxDefWidth * 2
+                    l = tuple(wls(att[n], w, linebreak=br)
+                              for n in MetaHeaders)
                     if extra:
                         l += tuple(v for v in ext.values())
                 else:
-                    l = tuple(
-                        wls(att[n], w)
-                        for n, w in param_widths.items() if w != 0)
+                    thewidths = param_widths if param_widths else \
+                        MetaData.Default_Param_Widths
+                    l = tuple(wls(att[n], w)
+                              for n, w in thewidths.items() if w != 0)
                     if extra:
                         l += tuple(
                             wls(v, Default_Extra_Param_Width)
@@ -990,7 +1074,7 @@ class MetaData(ParameterListener, Composite, Copyable, DatasetEventSender):
                         # print(l)
 
                 tab.append(l)
-                exh = [v for v in ext.keys()]
+                ext_hdr = [v for v in ext.keys()]
 
             elif level == 1:
                 ps = '%s= %s' % (att['name'], att['value'])
@@ -1005,8 +1089,10 @@ class MetaData(ParameterListener, Composite, Copyable, DatasetEventSender):
                         tab.append(row)
                         i, row = 0, []
             else:
+                # level > 1
                 n = att['name']
-                if n in self._defaults and self._defaults[n]['default'] == v.value:
+
+                if v is None or n in self._defaults and self._defaults[n]['default'] == v.value:
 
                     has_omission = True
                     pass
@@ -1016,7 +1102,7 @@ class MetaData(ParameterListener, Composite, Copyable, DatasetEventSender):
                     ps = '%s=%s' % (n, v.toString(level)) if level == 2 else n
                     # tab.append(wls(ps, 80//N))
                     tab.append(ps)
-            #nn += 1
+            # nn += 1
             # if nn == 2:
             #    pass  # break
 
@@ -1025,18 +1111,24 @@ class MetaData(ParameterListener, Composite, Copyable, DatasetEventSender):
 
         # write out the table
         if level == 0:
-            allh = (MetaHeaders + exh) if extra else MetaHeaders
-            if param_widths == -1:
+            allh = copy.copy(MetaHeaders)
+            if extra:
+                allh += ext_hdr
+            if param_widths == -1 or html:
                 headers = allh
             else:
                 headers = []
+                thewidths = param_widths if param_widths else \
+                    MetaData.Default_Param_Widths
                 for n in allh:
-                    w = param_widths.get(n, Default_Extra_Param_Width)
+                    w = thewidths.get(n, Default_Extra_Param_Width)
                     #print(n, w)
                     if w != 0:
                         headers.append(wls(n, w))
             fmt = tablefmt
-            s += tabulate.tabulate(tab, headers=headers, tablefmt=fmt, missingval='',
+            maxwidth = MetaData.MaxDefWidth
+            s += tabulate.tabulate(tab, headers=headers, tablefmt=fmt,
+                                   missingval='', maxcolwidths=maxwidth,
                                    disable_numparse=True)
         elif level == 1:
             t = grouper(tab, N)
