@@ -1,9 +1,11 @@
 # -*- coding: utf-8 -*-
 
+
 # from ..utils.common import fullname
 
 import array
 import binascii
+import gzip
 # from .odict import ODict
 import logging
 import json
@@ -150,11 +152,13 @@ class SerializableEncoder(json.JSONEncoder):
                 # print('&&&& %s %s' % (str(obj.__class__), str(obj)))
                 if PY3:
                     if issubclass(obj.__class__, bytes):
-                        return dict(code=str(codecs.encode(obj, 'hex'), encoding='ascii'), _STID='bytes')
+                        # return dict(code=str(codecs.encode(obj, 'hex'), encoding='ascii'), _STID='bytes')
+                        return dict(code=binascii.b2a_base64(gzip.compress(obj, 5)).decode('ascii'), _STID='bytes_gz')
                     elif issubclass(obj.__class__, array.array):
-                        return dict(code=str(binascii.b2a_hex(obj), encoding='ascii'), _STID='array.array_'+obj.typecode)
+                        return dict(code=binascii.b2a_base64(gzip.compress(obj, 5)).decode('ascii'), _STID='a.array,gz,b64_'+obj.typecode)
                 if not PY3 and issubclass(obj.__class__, str):
-                    return dict(code=codec.encode(obj, 'hex'), _STID='bytes')
+                    # return dict(code=codec.encode(obj, 'hex'), _STID='bytes')
+                    return dict(code=gzip.compress(obj, 5), _STID='bytes_gz')
                 if obj is Ellipsis:
                     return {'obj': '...', '_STID': 'ellipsis'}
                 if issubclass(obj.__class__, type):
@@ -174,7 +178,7 @@ class SerializableEncoder(json.JSONEncoder):
 #    obj = json.loads(jstring)
 
 def serialize(o, cls=None, **kwds):
-    """ return JSON using special encoder SerializableEncoder 
+    """ return JSON using special encoder SerializableEncoder
 
     Parameterts
     -----------
@@ -184,7 +188,7 @@ def serialize(o, cls=None, **kwds):
     """
     if not cls:
         cls = SerializableEncoder
-    return json.dumps(o, cls=cls, **kwds)
+    return json.dumps(o, cls=cls, allow_nan=True, **kwds)
 
 
 ATTR = '_ATTR_'
@@ -294,8 +298,44 @@ class Serializable():
         s.update({'_STID': self._STID})
         return s
 
-    def yaml(self):
+    def yaml(self, *args, **kwds):
         """ Get a YAML representation. """
         from ..utils.ydump import ydump, yinit
         yinit()
-        return ydump(self)
+        return ydump(self, *args, **kwds)
+
+    def tree(self, *args, **kwds):
+        """ Get a directory-tree-like representation. """
+        from ..utils.tree import tree
+
+        return '\n'.join(tree(self, *args, **kwds))
+
+    def fits(self, *args, **kwds):
+        """ Get a FITS representation. """
+        from ..utils.tofits import toFits, FITS_INSTALLED
+
+        if not FITS_INSTALLED:
+            raise NotImplemented(
+                'Astropy not installed. Include SCI in extra-dependency when installing FDI.')
+
+        return toFits(self, *args, **kwds)
+
+    def html(self, extra=False, param_widths=-1, **kwds):
+        """ Get a HTML representation. """
+
+        return self.toString(level=0,
+                             tablefmt='unsafehtml',
+                             tablefmt1='unsafehtml',
+                             tablefmt2='unsafehtml',
+                             extra=extra,
+                             param_widths=param_widths, **kwds)
+
+    def jsonPath(self, expr, val='simple', sep='/', indent=None, *args, **kwds):
+        from ..utils.jsonpath import jsonPath
+
+        return jsonPath(self.data, expr=expr, val=val, indent=indent, *args, **kwds)
+
+    def fetch(self, paths, exe=['is'], not_quoted=True):
+        from ..utils.fetch import fetch
+
+        return fetch(paths, self, re='', exe=exe, not_quoted=not_quoted)
