@@ -3,10 +3,10 @@
 from .indexed import Indexed
 from .ndprint import ndprint
 from .odict import ODict
-from ..utils.common import mstr, bstr, lls, wls, exprstrs, findShape
-from .dataset import Dataset, make_title_meta_l0, CompositeDataset
+from ..utils.common import wls
+from .dataset import make_title_meta_l0, CompositeDataset
 from .shaped import Shaped
-from .indexed import Indexed
+from .metadata import MetaData
 
 try:
     from .tabledataset_datamodel import Model
@@ -92,8 +92,15 @@ class TableModel():
         self.getColumn(columnIndex).data[rowIndex] = value
 
 
-def maybe2rows(header_names, units, col_width):
-    """ makes one-row or two-row header """
+def maybe2rows(header_names, units, col_width, one_row=False, linebreak='\n'):
+    """ makes one-row or two-row header
+
+    :one_row: Force one row but add line breaks at '.'
+
+    """
+    if col_width is None:
+        col_width = -1
+
     found_repeat = False
     hd, hd2 = [], []
     last = None
@@ -118,15 +125,21 @@ def maybe2rows(header_names, units, col_width):
                 p0 = ''
                 hd.append(p[0])
             hd2.append(p0)
-            if not found_repeat:
+            # repeat is never found if one row
+            if not one_row and not found_repeat:
                 if p0 != '' and p0 == last:
                     found_repeat = True
                 last = p0
 
     # [(column name, unit), ...]. Widths of column head is limited
-    hdr1 = [wls(x, width=col_width)
+    hdr1 = [wls(x.replace('.', '.\n') if one_row else x,
+                width=col_width, linebreak=linebreak)
             for x in (hd if found_repeat else header_names)]
-    hdr = list('%s\n(%s)' % nu for nu in zip(hdr1, units))
+    hdr = list('%s%s(%s)' % (nu[0], linebreak, nu[1])
+               for nu in zip(hdr1, units))
+
+    if 0:  # one_row:
+        return list(linebreak.join(hd2, hdr))
     if found_repeat:
         # if there is found_repeat. use 2-row header
         return list(zip(hd2, hdr))
@@ -187,7 +200,7 @@ class TableDataset(CompositeDataset, TableModel, Shaped):
 
         super().__init__(zInfo=zInfo, **metasToBeInstalled,
                          **kwds)  # initialize data, meta, unit
-        # self.setData(data)
+        self.updateShape()
 
     # def getData(self):
     #     """ Optimized for _data being an ``ODict/dict`` implemented with ``DataContaier``.
@@ -326,7 +339,7 @@ class TableDataset(CompositeDataset, TableModel, Shaped):
     def addRow(self, row, rows=False):
         """ Adds the specified map as a new row to this table.
 
-        row: mh: row is a dict with names as keys and row data as value.
+        row: row is a dict with names as keys and row data as values.
         rows: append each element in row if the row data is a list.
         """
 
@@ -574,9 +587,9 @@ Default is to return all columns.
         return self.toString(level=2)
 
     def toString(self, level=0,
-                 tablefmt='grid', tablefmt1='simple', tablefmt2='rst',
+                 tablefmt='grid', tablefmt1='simple', tablefmt2='plain',
                  width=0, param_widths=None, matprint=None, trans=True,
-                 heavy=True, center=-1, col_width=8, **kwds):
+                 heavy=True, center=-1, **kwds):
         """
         tablefmt2: format of 2D data, others see `MetaData.toString`.
         """
@@ -600,28 +613,45 @@ Default is to return all columns.
                     for k, v in self.getColumnMap().items()) + \
                 '})'
 
+        html = 'html' in tablefmt.lower() or 'html' in tablefmt2.lower()
+        br = '<br>' if html else '\n'
+        if html:
+            tablefmt = tablefmt2 = 'unsafehtml'
         s, last = make_title_meta_l0(self, level=level, width=width, heavy=heavy,
                                      tablefmt=tablefmt, tablefmt1=tablefmt1,
-                                     tablefmt2=tablefmt2, center=center, excpt=['description'])
+                                     tablefmt2=tablefmt2, center=center,
+                                     param_widths=param_widths,
+                                     html=html, excpt=['description'],
+                                     **kwds)
         width = len(last)-1
         if level == 0:
-            d = 'DATA'.center(width) + '\n' + '----'.center(width) + '\n'
+            if html:
+                d = '<center><u>%s</u></center>\n' % 'DATA'
+            else:
+                d = 'DATA'.center(width) + '\n' + '----'.center(width) + '\n'
         else:
             d = ''
-        stp = 2 if level > 1 else 20 if level == 1 else None
-        cols = self.getData().values()
+        rowlimit = 2 if level > 1 else 20 if level == 1 else None
 
-        coldata = [list(itertools.islice(x.data, stp)) for x in cols]
+        cols = self.getData().values()
+        # widest width in all of default and in param_widths
+        w = MetaData.MaxDefWidth if param_widths == -1 or param_widths is None else \
+            max(MetaData.MaxDefWidth, max(param_widths.values()))
+        if html:
+            w = w
+        coldata = [list(itertools.islice(x.data, rowlimit)) for x in cols]
         hdr = maybe2rows(self.getData().keys(),
                          (str(x.unit) for x in cols),
-                         col_width)
+                         col_width=w, one_row=html,
+                         linebreak=br)
         d += matprint(coldata, trans=trans, headers=hdr,
                       tablefmt=tablefmt, tablefmt1=tablefmt1,
                       tablefmt2=tablefmt2, center=center,
+                      mdim=2, param_widths=param_widths,
                       maxElem=sys.maxsize, **kwds)
         collen = self.getRowCount()
-        if level and stp < collen:
-            d += '(Only display %d rows of %d for level=%d.)' % (stp, collen, level)
+        if level and rowlimit is not None and rowlimit < collen:
+            d += '(Only display %d rows of %d for level=%d.)' % (rowlimit, collen, level)
         return f'{s}\n{d}{last}\n'
 
     string = toString
