@@ -1,25 +1,25 @@
 #!/usr/bin/env python3
 # -*- coding: utf-8 -*-
 
-from ruamel.yaml import YAML
-# import yaml
-from collections import OrderedDict
-import os
-import sys
-from string import Template
-import pkg_resources
-from datetime import datetime
-import importlib
-
 # from ..pal.context import MapContext
 from ..utils.options import opt
 from ..utils.common import pathjoin, lls
-from ..utils.ydump import ydump
+from ..utils.ydump import yinit, ydump
 from ..utils.moduleloader import SelectiveMetaFinder, installSelectiveMetaFinder
 from .attributable import make_class_properties
 # a dictionary that translates metadata 'type' field to classname
 from .datatypes import DataTypes, DataTypeNames
 
+# from ruamel.yaml import YAML
+# import yaml
+from collections import OrderedDict
+import os
+import sys
+from itertools import chain
+from string import Template
+from datetime import datetime
+import importlib
+
 import logging
 
 # create logger
@@ -72,7 +72,7 @@ def sq(s):
     return '%s%s%s' % (qm, s, qm)
 
 
-def getPython(val, indents, demo, onlyInclude, debug=False):
+def get_Python(val, indents, demo, onlyInclude, debug=False):
     """ make Model and init__() code strings from given data.
 
     Parameters
@@ -96,7 +96,7 @@ def getPython(val, indents, demo, onlyInclude, debug=False):
                 # as in k:v
                 # k:
                 #   data_type: string
-                #   description: Description of this dataset
+                #   description: Description of this parent_dataset
                 #   default: UNKNOWN
                 #   valid: ''
                 # v is a dict of parameter attributes
@@ -104,7 +104,7 @@ def getPython(val, indents, demo, onlyInclude, debug=False):
                     v, indents[1:], demo, onlyInclude, debug=debug)
             else:
                 # headers such as name, parents, schema metadata...
-                istr, d_code = getPython(
+                istr, d_code = get_Python(
                     v, indents[1:], demo, onlyInclude, debug=debug)
             infostr += istr
             code[sk] = d_code
@@ -119,7 +119,7 @@ def getPython(val, indents, demo, onlyInclude, debug=False):
                 istr, d_code = params(
                     v, indents[1:], demo, onlyInclude, debug=debug)
             else:
-                istr, d_code = getPython(
+                istr, d_code = get_Python(
                     v, indents[1:], demo, onlyInclude, debug=debug)
             infostr += istr
             code.append(d_code)
@@ -131,10 +131,10 @@ def getPython(val, indents, demo, onlyInclude, debug=False):
     return infostr, code
 
 
-def makeinitcode(dt, pval):
+def make_init_code(dt, pval):
     """ python instanciation source code.
 
-    will be like "default: FineTime1(0)"
+    will be like "default: FineTime1(0)" in 'def __init__(self...'
     Parameters
     ----------
 
@@ -155,17 +155,17 @@ def makeinitcode(dt, pval):
 
 
 def params(val, indents, demo, onlyInclude, debug=False):
-    """ generates python strng for val, a parameter with a set of attribute
+    """ generates python strng for val, a parameter with a set of attributes
 
     val: as in ```name:val```
     ```
     nam:
         data_type: string
-        description: Description of this dataset
+        description: Description of this parent_dataset
         default: UNKNOWN
         valid: ''
     ```
-    see getPython
+    see get_Python
     Parameters
     ----------
 
@@ -224,7 +224,7 @@ def params(val, indents, demo, onlyInclude, debug=False):
             # get string representation
             pval = str(pv).strip() if iss else str(pv)
             if pname == 'default':
-                code = makeinitcode(dt, pval)
+                code = make_init_code(dt, pval)
             if pname in ['example', 'default']:
                 # here data_type instead of input type determines the output type
                 iss = (val['data_type'] == 'string') and (pval != 'None')
@@ -235,26 +235,31 @@ def params(val, indents, demo, onlyInclude, debug=False):
     return modelString, code
 
 
-def getCls(clp, rerun=True, exclude=None, ignore_error=True, verbose=False):
+def get_projectclasses(clp, rerun=True, exclude=None, verbose=False):
     """
+    return a `Classes` object that is going  to give {class-name:class-type} from a file at gieven location.
+
     Parameters
     ----------
+    :clp: path of the mapping file.
+    :rerun, exclude: from `Classes`
 
     Returns
     -------
+    The `classes.Classes` object.
     """
+
     if clp is None or len(clp.strip()) == 0:
-        return {}
+        return None
+
     if exclude is None:
         exclude = []
     if '/' not in clp and '\\' not in clp and not clp.endswith('.py'):
         print('Importing project classes from module '+clp)
         # classes path not given on command line
         pc = importlib.import_module(clp)
-        ret = pc.PC.updateMapping(rerun=rerun, exclude=exclude,
-                                  verbose=verbose, ignore_error=True)
         print(
-            'Imported project classes from svom.products.projectclasses module.')
+            'Imported project classes from %s module.' % clp)
 
     else:
         clpp, clpf = os.path.split(clp)
@@ -263,13 +268,10 @@ def getCls(clp, rerun=True, exclude=None, ignore_error=True, verbose=False):
         print('Importing project classes from file '+clp)
         pc = importlib.import_module(clpf.replace('.py', ''))
         sys.path.pop(0)
-        ret = pc.PC.updateMapping(rerun=rerun, exclude=exclude,
-                                  verbose=verbose, ignore_error=ignore_error)
-
-    return ret
+    return pc
 
 
-def readyaml(ypath, version=None, verbose=False):
+def read_yaml(ypath, version=None, verbose=False):
     """ read YAML files in ypath.
 
     output: nm is  stem of file name. desc is descriptor, key being yaml[name]
@@ -278,7 +280,7 @@ def readyaml(ypath, version=None, verbose=False):
     Returns
     -------
     """
-    yaml = YAML()
+    yaml = yinit()
     desc = OrderedDict()
     fins = {}
     for findir in os.listdir(ypath):
@@ -299,68 +301,39 @@ def readyaml(ypath, version=None, verbose=False):
         print('--- Reading ' + fin + '---')
         with open(fin, 'r', encoding='utf-8') as f:
             # pyYAML d = OrderedDict(yaml.load(f, Loader=yaml.FullLoader))
-            d = OrderedDict(yaml.load(f))
+            ytext = f.read()
+        y = yaml.load(ytext)
+        d = dict(OrderedDict(y))
+
+        if float(d['schema']) < 1.2:
+            raise NotImplemented('Schema %s is too old.' % d['schema'])
         if 'metadata' not in d or d['metadata'] is None:
             d['metadata'] = {}
         if 'datasets' not in d or d['datasets'] is None:
             d['datasets'] = {}
-        if float(d['schema']) >= 1.0:
-            pass
-        if float(d['schema']) > 0.6:
-            attrs = OrderedDict(d['metadata'])
-            datasets = OrderedDict()
-            # move primary level table to datasets
-            if 'TABLE' in attrs:
-                datasets['TABLE'] = {}
-                datasets['TABLE']['TABLE'] = attrs['TABLE']
-                del attrs['TABLE']
-            if 'datasets' in d:
-                datasets.update(d['datasets'])
-            if verbose:
-                print('Pre-emble:\n%s' %
-                      (''.join([k + '=' + str(v) + '\n'
-                                for k, v in d.items() if k not in ['metadata', 'datasets']])))
 
-            logger.debug('Find attributes:\n%s' %
-                         ''.join(('%20s' % (k+'=' + str(v['default'])
-                                            if 'default' in v else 'url' + ', ')
-                                  for k, v in attrs.items()
-                                  )))
-            if float(d['schema']) > 1.1:
-                itr = ('%20s' % (k+'=' + str([c for c in (v['TABLE'] if 'TABLE'
-                                                          in v else [])]))
-                       for k, v in datasets.items())
-                logger.debug('Find datasets:\n%s' % ', '.join(itr))
-            else:
-                # v1.1 1.0 0.7
-                itr = ('%20s' %
-                       (k+'=' + str([c['name'] for c in (v['TABLE'] if 'TABLE'
-                                                         in v else [])]))
-                       for k, v in datasets.items())
-                print('Find datasets:\n%s' % ', '.join(itr))
-            desc[d['name']] = (d, attrs, datasets, fin)
-        else:
-            # float(d['schema']) <= 0.6:
-            d2 = OrderedDict()
-            metadata = OrderedDict()
-            for k, v in d.items():
-                if issubclass(v.__class__, dict):
-                    if v['unit'] == 'None':
-                        dt = v['data_type']
-                        if dt in ['boolean', 'string']:
-                            v['unit'] = None
-                    metadata[k] = v
-                else:
-                    if k == 'definition':
-                        d2['description'] = v
-                    elif k == 'schema':
-                        d2[k] = version
-                    elif k == 'parent':
-                        d2['parents'] = [v]
-                    else:
-                        d2[k] = v
-            d2['metadata'] = metadata
-            desc[d['name']] = d2
+        attrs = dict(d['metadata'])
+        datasets = dict(d['datasets'])
+        # move primary level table to datasets, named 'TABLE_META'
+        if 'TABLE' in attrs:
+            datasets['TABLE_META'] = {}
+            datasets['TABLE_META']['TABLE'] = attrs['TABLE']
+            del attrs['TABLE']
+        if verbose:
+            print('Pre-emble:\n%s' %
+                  (''.join([k + '=' + str(v) + '\n'
+                            for k, v in d.items() if k not in ['metadata', 'datasets']])))
+
+        logger.debug('Find attributes:\n%s' %
+                     ''.join(('%20s' % (k+'=' + str(v['default'])
+                                        if 'default' in v else 'url' + ', ')
+                              for k, v in attrs.items()
+                              )))
+        itr = ('%20s' % (k+'=' + str([c for c in (v['TABLE'] if 'TABLE'
+                                                  in v else [])]))
+               for k, v in datasets.items())
+        logger.debug('Find datasets:\n%s' % ', '.join(itr))
+        desc[d['name']] = (d, attrs, datasets, fin)
     return desc, fins
 
 
@@ -384,7 +357,7 @@ def output(nm, d, fins, version, dry_run=False, verbose=False):
             ydump(d,  yamlfile)
 
 
-def yamlupgrade(descriptors, fins, ypath, version, dry_run=False, verbose=False):
+def yaml_upgrade(descriptors, fins, ypath, version, dry_run=False, verbose=False):
     """
     Parameters
     ----------
@@ -426,7 +399,7 @@ def yamlupgrade(descriptors, fins, ypath, version, dry_run=False, verbose=False)
             md = OrderedDict()
             for pname, w in d['metadata'].items():
                 # dt = w['data_type']
-                # no dataset yet
+                # no parent_dataset yet
                 if pname == 'type':
                     v = w['default']
                     w.clear()
@@ -467,15 +440,16 @@ def dependency_sort(descriptors):
         for i in range(len(working_list)):
             # find parents of i
             nm = working_list[i]
+            # 0 for top level
             p = descriptors[nm][0]['parents']
             nm_found_parent = False
             if len(p) == 0:
                 continue
             found = set(working_list) & set(p)
-            # for x in working_list:
-            #    if x == nm:
-            #        continue
-            #    if x in p:
+            # type_of_nm = glb[nm]
+            # found2 = any(issubclass(type_of_nm, glb[x])
+            #              for x in working_list if x != nm)
+            # assert bool(len(found)) == found2
             if len(found):
                 # parent is in working_list
                 working_list.remove(nm)
@@ -499,13 +473,16 @@ def dependency_sort(descriptors):
     return ret
 
 
-def removeParent(a, b):
+def remove_Parent(a, b):
     """ Returns the one who is the other one's parent.
     Parameters
     ----------
+    :a: a class name
+    :b: a class name
 
     Returns
     -------
+    classname if found or `None`.
     """
     if a == b:
         logger.debug('%s and %s are the same class' % (b, a))
@@ -523,13 +500,17 @@ def removeParent(a, b):
         return None
 
 
-def noParentsParents(pn):
+def no_Parents_Parents(pn):
     """
+    return a subset of class names such that no member is any other's parent.
+
     Parameters
     ----------
+    :pn: list of class names.
 
     Returns
     -------
+    list of non-parents.
     """
 
     removed = []
@@ -537,7 +518,7 @@ def noParentsParents(pn):
         if pn[i] in removed:
             continue
         for j in range(i+1, len(pn)):
-            r = removeParent(pn[i], pn[j])
+            r = remove_Parent(pn[i], pn[j])
             if r:
                 removed.append(r)
             if r == pn[i]:
@@ -547,12 +528,89 @@ def noParentsParents(pn):
     return pn
 
 
+def inherit_from_parents(parentNames, attrs, datasets, schema, seen):
+    """ inherit metadata and datasets from parents.
+
+    :attrs: metadata descriptor of the child
+    :datasets: datasets descriptor of the child
+    :seen: a dict holding class names that the py file is going to import
+ """
+    if parentNames and len(parentNames):
+        parentsAttributes = {}
+        parentsTableColumns = {}
+        for parent in parentNames:
+            if parent is None:
+                continue
+            mod_name = glb[parent].__module__
+            s = 'from %s import %s' % (mod_name, parent)
+            if parent not in seen:
+                seen[parent] = s
+
+            # get parent attributes and tables
+            mod = sys.modules[mod_name]
+            if hasattr(mod, '_Model_Spec'):
+                parentsAttributes.update(
+                    mod._Model_Spec['metadata'])
+
+                parentsTableColumns.update(
+                    mod._Model_Spec['datasets'])
+        # merge to get all attributes including parents' and self's.
+        toremove = []
+        for nam, val in attrs.items():
+            if float(schema) > 1.5 and 'data_type' not in val:
+                # update parent's
+                parentsAttributes[nam].update(attrs[nam])
+                toremove.append(nam)
+            else:
+                # override
+                parentsAttributes[nam] = attrs[nam]
+        for nam in toremove:
+            del attrs[nam]
+        if 0 and parentsAttributes['type']['default'] == 'Oem_Nssc':
+            __import__('pdb').set_trace()
+        # parents are updated but names and orders follow the child's
+        for ds_name, child_dataset in datasets.items():
+            # go through datasets  TODO: ArrayDataset
+            if ds_name not in parentsTableColumns:
+                # child has a name that the parent does not have
+                parentsTableColumns[ds_name] = child_dataset
+                continue
+            p_dset = parentsTableColumns[ds_name]
+            # go through the child's dataset
+            for name, c_val in child_dataset.items():
+                # child has a name that the parent does not have
+                if name not in p_dset:
+                    p_dset[name] = c_val
+                    continue
+                # p and c have dataset name in common
+                if name != 'TABLE':
+                    # parameter in meta
+                    p_dset.update(c_val)
+                    continue
+                p_tab = p_dset['TABLE']
+                _tab = {}
+                # go through the child columns
+                for colname, col in c_val.items():
+                    # child has a name that the parent does not have
+                    if colname not in p_tab:
+                        _tab[colname] = col
+                        continue
+                    p_tab[colname].update(col)
+                    _tab[colname] = p_tab[colname]
+                p_dset['TABLE'] = _tab
+    else:
+        parentsAttributes = attrs
+        parentsTableColumns = datasets
+
+    return parentsAttributes, parentsTableColumns
+
+
 if __name__ == '__main__':
 
     print('product class generatiom')
 
     # schema version
-    version = '1.6'
+    version = '1.8'
 
     # Get input file name etc. from command line. defaut 'Product.yml'
     cwd = os.path.abspath(os.getcwd())
@@ -575,7 +633,7 @@ if __name__ == '__main__':
         {'long': 'userclasses=', 'char': 'c', 'default': '',
          'description': 'Python file name, or a module name,  to import prjcls to update Classes with user-defined classes which YAML file refers to.'},
         {'long': 'upgrade', 'char': 'u', 'default': False,
-         'description': 'Upgrade the file to current schema, by yamlupgrade(), to version + ' + version},
+         'description': 'Upgrade the file to current schema, by yaml_upgrade(), to version + ' + version},
         {'long': 'dry_run', 'char': 'n', 'default': False,
          'description': 'No writing. Dry run.'},
         {'long': 'debug', 'char': 'd', 'default': False,
@@ -599,15 +657,11 @@ if __name__ == '__main__':
     dry_run = out[8]['result']
     debug = out[9]['result']
 
-    if debug:
-        import pdb
-        pdb.set_trace()
-
     # input file
-    descriptors, files_imput = readyaml(ypath, version, verbose)
+    descriptors, files_imput = read_yaml(ypath, version, verbose)
     if upgrade:
-        yamlupgrade(descriptors, files_imput, ypath, version,
-                    dry_run=dry_run, verbose=verbose)
+        yaml_upgrade(descriptors, files_imput, ypath, version,
+                     dry_run=dry_run, verbose=verbose)
         sys.exit()
 
     # Do not import modules that are to be generated. Thier source code
@@ -621,12 +675,16 @@ if __name__ == '__main__':
     # include project classes for every product so that products made just
     # now can be used as parents
     from .classes import Classes
-    pcl = getCls(project_class_path, rerun=True,
-                 exclude=importexclude, verbose=verbose, ignore_error=True)
+    pc = get_projectclasses(project_class_path,  rerun=True,
+                            exclude=importexclude, verbose=verbose)
     glb = Classes.updateMapping(
-        c=pcl, rerun=True, exclude=importexclude, verbose=verbose)
-    # make a list whose members do not depend members behind
+        c=pc.PC.getMapping(exclude=importexclude) if pc else {},
+        rerun=True,
+        exclude=importexclude,
+        verbose=verbose)
+    # make a list whose members do not depend on members behind (to the left)
     sorted_list = dependency_sort(descriptors)
+
     skipped = []
     for nm in sorted_list:
         d, attrs, datasets, fin = descriptors[nm]
@@ -656,50 +714,22 @@ if __name__ == '__main__':
         schema = d['schema']
 
         # the generated source code must import these
-        seen = []
+        seen = {}
         imports = 'from collections import OrderedDict\n'
         # import parent classes
         parentNames = d['parents']
         # remove classes that are other's parent class (MRO problem)
         try:
             if parentNames and len(parentNames):
-                parentNames = noParentsParents(parentNames)
+                parentNames = no_Parents_Parents(parentNames)
         except KeyError as e:
             logger.warning('!!!!!!!!!!! Skipped %s due to %s.' %
                            (nm, type(e).__name__+str(e)))
             skipped.append(nm)
             continue
-        if parentNames and len(parentNames):
-            includingParentsAttributes = OrderedDict()
-            for parent in parentNames:
-                if parent is None:
-                    continue
-                modnm = glb[parent].__module__
-                s = 'from %s import %s\n' % (modnm, parent)
-                if parent not in seen:
-                    seen.append(parent)
-                    imports += s
-
-                # get parent attributes
-                mod = sys.modules[modnm]
-                if hasattr(mod, '_Model_Spec'):
-                    includingParentsAttributes.update(
-                        mod._Model_Spec['metadata'])
-            # merge to get all attributes including parents' and self's.
-            toremove = []
-            for nam, val in attrs.items():
-                if float(schema) > 1.5 and 'data_type' not in val:
-                    # update parent's
-                    includingParentsAttributes[nam].update(attrs[nam])
-                    toremove.append(nam)
-                else:
-                    # override
-                    includingParentsAttributes[nam] = attrs[nam]
-            for nam in toremove:
-                del attrs[nam]
-        else:
-            includingParentsAttributes = attrs
 
+        parentsAttributes, parentsTableColumns = \
+            inherit_from_parents(parentNames, attrs, datasets, schema, seen)
         # make output filename, lowercase modulename + .py
         fout = pathjoin(opath, modulename + '.py')
         print("Output python file is "+fout)
@@ -710,25 +740,30 @@ if __name__ == '__main__':
             fin, datetime.now(), d['description'])))
 
         # parameter classes used in init code may need to be imported, too
-        for met, val in includingParentsAttributes.items():
+        for val in chain(parentsAttributes.values(),
+                         chain(col for ds in
+                               parentsTableColumns.values()
+                               for col in ds.get('TABLE', {}).values()
+                               )
+                         ):
+            print(val)
             a = DataTypes[val['data_type']]
             if a in glb:
                 # this attribute class has module
                 s = 'from %s import %s' % (glb[a].__module__, a)
                 if a not in seen:
-                    seen.append(a)
-                    imports += s+'\n'
+                    seen[a] = s
 
-        # make metadata and dataset dicts
-        d['metadata'] = includingParentsAttributes
-        d['datasets'] = datasets
-        infs, default_code = getPython(d, indents[1:], demo, onlyInclude)
+        # make metadata and parent_dataset dicts
+        d['metadata'] = parentsAttributes
+        d['datasets'] = parentsTableColumns
+        infs, default_code = get_Python(d, indents[1:], demo, onlyInclude)
         # remove the ',' at the end.
         modelString = (ei + '_Model_Spec = ' + infs).strip()[:-1]
 
         # keyword argument for __init__
         ls = []
-        for x in includingParentsAttributes:
+        for x in parentsAttributes:
             arg = 'typ_' if x == 'type' else x
             val = default_code['metadata'][x]
             ls.append(' '*17 + '%s = %s,' % (arg, val))
@@ -744,8 +779,8 @@ if __name__ == '__main__':
         print('product name: %s' % subs['MODELNAME'])
         subs['PARENTS'] = ', '.join(c for c in parentNames if c)
         print('parent class: %s' % subs['PARENTS'])
-        subs['IMPORTS'] = imports
-        print('import class: %s' % seen)
+        subs['IMPORTS'] = imports + '\n'.join(seen.values())
+        print('import class: %s' % ', '.join(seen.keys()))
         subs['CLASSDOC'] = doc
         subs['MODELSPEC'] = modelString
         subs['INITARGS'] = ikwds
@@ -776,9 +811,11 @@ if __name__ == '__main__':
 
         # import the newly made module to test and, for class generatiom, so the following classes could use it
         importexclude.remove(modulename)
-        # importlib.invalidate_caches()
         if cwd not in sys.path:
             sys.path.insert(0, cwd)
+        importlib.invalidate_caches()
+        logger.debug('sys.path is '+str(sys.path))
+
         newp = 'fresh ' + modelName + ' from ' + modulename + \
             '.py of package ' + package_name + ' in ' + opath + '.'
         if modelName.endswith('_DataModel'):
@@ -795,7 +832,7 @@ if __name__ == '__main__':
             glb[modelName] = getattr(_o, modelName)
         except Exception as e:
             print('Unable to import ' + newp)
-            raise(e)
+            raise
         importexclude.extend(exclude_save)
         print('Imported ' + newp)
         # Instantiate and dump metadata in text format
