@@ -1,16 +1,16 @@
 # -*- coding: utf-8 -*-
 
-from ..model.user import getUsers, auth
+from ..model.user import auth
 
-from ...utils.common import lls
-from ...dataset.deserialize import deserialize, deserialize_args
+from ...dataset.deserialize import deserialize
 from ...dataset.serializable import serialize
 from ...dataset.mediawrapper import MediaWrapper
 from ...pal.urn import makeUrn
+from ...pal.poolmanager import PoolManager as PM
 from ...dataset.classes import Classes
 from ...utils.common import trbk, getUidGid
 from ...utils.fetch import fetch
-from ...pal.poolmanager import PoolManager as PM, DEFAULT_MEM_POOL
+from ...pns.fdi_requests import POST_PRODUCT_TAG_NAME
 
 # from .db_utils import check_and_create_fdi_record_table, save_action
 
@@ -19,7 +19,7 @@ from ...pal.poolmanager import PoolManager as PM, DEFAULT_MEM_POOL
 
 from flasgger import swag_from
 
-from flask import request, make_response, jsonify, Blueprint, current_app
+from flask import request, make_response, Blueprint, current_app
 from flask.wrappers import Response
 
 
@@ -30,6 +30,7 @@ import builtins
 import functools
 from pathlib import Path
 import importlib
+import logging
 
 if sys.version_info[0] >= 3:  # + 0.1 * sys.version_info[1] >= 3.3:
     PY3 = True
@@ -92,7 +93,6 @@ def checkpath(path, un):
 
 # @data_api.before_app_first_request
 
-
 def resp(code, result, msg, ts, serialize_out=False, ctype='application/json', length=80, req_auth=False):
     """
     Make response.
@@ -114,7 +114,9 @@ def resp(code, result, msg, ts, serialize_out=False, ctype='application/json', l
     else:
         w = result
 
-    logger.debug(lls(w, length))
+    if logger.getEffectiveLevel() <= logging.DEBUG:
+        from ...utils.common import lls
+        logger.debug(lls(w, length))
     # logger.debug(pprint.pformat(w, depth=3, indent=4))
     resp = make_response(w, code)
     resp.headers['Content-Type'] = ctype
@@ -125,6 +127,8 @@ def resp(code, result, msg, ts, serialize_out=False, ctype='application/json', l
 
 
 def excp(e, code=400, msg='', serialize_out=True):
+    if current_app.config['DEBUG']:
+        raise e
     result = '"FAILED"' if serialize_out else 'FAILED'
     msg = '%s\n%s: %s.\nTrace back: %s' % (
         msg, e.__class__.__name__, str(e), trbk(e))
@@ -147,6 +151,13 @@ def check_readonly(usr, meth, logger):
     return None
 
 
+@data_api.before_request
+def before_request_callback():
+    path = request.path
+    method = request.method
+
+    print("data_api " + path + " >>>>>>[" + method + "]<<<<<<")
+
 ######################################
 ####  /urn{parts} get data ####
 ######################################
@@ -282,8 +293,8 @@ def save_data(pool):
         return resp(code, result, msg, ts, serialize_out=True)
 
     # save product
-    if request.headers.get('tags') is not None:
-        tags = request.headers.get('tags').split(',')
+    if request.headers.get(POST_PRODUCT_TAG_NAME) is not None:
+        tags = deserialize(request.headers.get(POST_PRODUCT_TAG_NAME))
     else:
         tags = None
 
@@ -317,18 +328,18 @@ def save_product(data, paths, tags=None, serialize_in=True, serialize_out=False)
 
     Saving Products to HTTPpool will have data stored on the server side. The server only returns URN strings as a response. ProductRefs will be generated by the associated httpclient pool which is the front-end on the user side.
 
-    :tags: a list off tag strings. default is None meaning no tag.
+    :tags: a list of tag strings. default is None meaning no tag.
     Returns a URN object or a list of URN objects.
     """
     FAILED = '"FAILED"' if serialize_out else 'FAILED'
 
     poolname = paths[0]
-    fullpoolpath = os.path.join(current_app.config['POOLPATH_BASE'], poolname)
+    fullpoolpath = os.path.join(
+        current_app.config['FULL_BASE_LOCAL_POOLPATH'], poolname)
     poolurl = current_app.config['POOLURL_BASE'] + poolname
     # resourcetype = fullname(data)
-    tag = tags[0] if tags else None  # TODO: accept all tags
 
-    if checkpath(fullpoolpath, current_app.config['PC']['serveruser']) is None:
+    if checkpath(fullpoolpath, current_app.config['PC']['self_username']) is None:
         result = FAILED
         msg = 'Pool directory error: ' + fullpoolpath
         return 400, result, msg
@@ -339,7 +350,7 @@ def save_product(data, paths, tags=None, serialize_in=True, serialize_out=False)
     try:
         poolobj = PM.getPool(poolname=poolname, poolurl=poolurl)
         result = poolobj.saveProduct(
-            product=data, tag=tag, geturnobjs=True, serialize_in=serialize_in, serialize_out=serialize_out)
+            product=data, tag=tags, geturnobjs=True, serialize_in=serialize_in, serialize_out=serialize_out)
         msg = 'Save data to ' + poolurl + ' OK.'
         code = 200
     except Exception as e:
@@ -423,6 +434,20 @@ def getProduct_Or_Component(paths, serialize_out=False):
         return 400, '"FAILED"', 'Unknown path %s' % str(paths)
 
 
+HTML_STYLE = """
+<style>
+table,th,td {
+    border: 1px solid black;
+    border-collapse: collapse;
+}
+tr:nth-child(even){
+    background-color: #DFDFDF;
+    color: black;
+}
+</style>
+"""
+
+
 def get_component_or_method(paths, mInfo, serialize_out=False):
     """ Get the component and the associated command and return
 
@@ -463,14 +488,28 @@ def get_component_or_method(paths, mInfo, serialize_out=False):
                compo_meth_name[:6] == 'string':
                 if 'html' in compo_meth_name:
                     ct = 'text/html'
+                    result = HTML_STYLE + compo
+                elif 'rst' in compo_meth_name:
+                    ct = 'text/plain;charset=utf-8'
+                    result = compo
                 elif 'fancy_grid' in compo_meth_name:
                     ct = 'text/plain;charset=utf-8'
+                    result = compo
                 else:
                     ct = 'text/plain'
+                    result = compo
+                return 0, resp(code, result, msg, ts, ctype=ct, serialize_out=False), 0
+            elif compo_meth_name[:4] == 'html':
+                ct = 'text/html'
+                result = HTML_STYLE + compo
+                return 0, resp(code, result, msg, ts, ctype=ct, serialize_out=False), 0
+            elif compo_meth_name == 'yaml()' or compo_meth_name == 'tree()':
+                ct = 'text/plain;charset=utf-8'
+                # 'font-family: "Courier New",monospace;\n'
                 result = compo
                 return 0, resp(code, result, msg, ts, ctype=ct, serialize_out=False), 0
-            elif compo_meth_name == 'yaml()':
-                ct = 'text/plain;charset=utf-8'
+            elif compo_meth_name == 'fits()':
+                ct = 'application/fits'
                 result = compo
                 return 0, resp(code, result, msg, ts, ctype=ct, serialize_out=False), 0
             elif issubclass(compo.__class__, MediaWrapper):
@@ -501,7 +540,7 @@ def get_component_or_method(paths, mInfo, serialize_out=False):
 
 
 def load_component_at(pos, paths, mInfo):
-    """ paths[pos] is data_type; paths[pos+2] is 'description','meta' ...
+    """ paths[pos] is data_type; paths[pos+2] is 'description', 'meta' ...
 
     Components fetched are not in serialized form.
     """
@@ -519,7 +558,7 @@ def load_component_at(pos, paths, mInfo):
 
 
 def load_product(p, paths, serialize_out=False):
-    """Load product paths[p]:paths[p+1] from paths[0]
+    """Load product paths[p]: paths[p+1] from paths[0]
     """
     FAILED = '"FAILED"' if serialize_out else 'FAILED'
 
@@ -548,7 +587,7 @@ def load_product(p, paths, serialize_out=False):
 
 
 def setOwnerMode(p, username):
-    """ makes UID and GID set to those of serveruser given in the config file. This function is usually done by the initPTS script.
+    """ makes UID and GID set to those of self_username given in the config file. This function is usually done by the initPTS script.
     """
 
     logger.debug('set owner, group to %s, mode to 0o775' % username)
