@@ -4,11 +4,13 @@ from .getswag import swag
 from .httppool_server import resp, excp, checkpath, check_readonly
 from ..model.user import auth, getUsers
 from ..._version import __version__
-from ...dataset.deserialize import deserialize_args
+from ...dataset.deserialize import deserialize_args, deserialize
 from ...pal.poolmanager import PoolManager as PM, DEFAULT_MEM_POOL
 from ...pal.productpool import PoolNotFoundError
 from ...pal.webapi import WebAPI
 from ...pal.urn import parseUrn
+from ...pal.dicthk import HKDBS
+from ...utils.common import lls
 
 from flask import Blueprint, jsonify, request, current_app, url_for, abort
 from werkzeug.exceptions import HTTPException
@@ -30,7 +32,7 @@ pools_api = Blueprint('pools', __name__)
 
 
 ######################################
-####  /user/login POST  ####
+####  /user/login GET, POST  ####
 ######################################
 
 
@@ -53,7 +55,7 @@ def login():
     return resp(200, 'OK', msg, ts, req_auth=True)
 
 ######################################
-####  /user/logout POST  ####
+####  /user/logout GET, POST  ####
 ######################################
 
 
@@ -87,7 +89,7 @@ def get_pools_url():
     logger = current_app.logger
 
     ts = time.time()
-    path = current_app.config['POOLPATH_BASE']
+    path = current_app.config['FULL_BASE_LOCAL_POOLPATH']
     logger.debug('Listing all directories from ' + path)
 
     result = get_name_all_pools(path)
@@ -118,7 +120,7 @@ def get_pools():
     logger = current_app.logger
 
     ts = time.time()
-    path = current_app.config['POOLPATH_BASE']
+    path = current_app.config['FULL_BASE_LOCAL_POOLPATH']
     logger.debug('Listing all directories from ' + path)
 
     result = get_name_all_pools(path)
@@ -157,11 +159,11 @@ def get_registered_pools():
     ---
     """
     ts = time.time()
-    path = current_app.config['POOLPATH_BASE']
+    path = current_app.config['FULL_BASE_LOCAL_POOLPATH']
     current_app.logger.debug('Listing all registered pools.')
 
     # [p.getPoolurl() for p in PM.getMap()()]
-    result = list(PM.getMap().keys())
+    result = list(PM.getMap())
     msg = 'There is/are %d pools registered to the PoolManager.' % len(result)
     code = 200
     return resp(code, result, msg, ts)
@@ -202,7 +204,7 @@ def load_pools(poolnames, usr):
     """
 
     logger = current_app.logger
-    path = current_app.config['POOLPATH_BASE']
+    path = current_app.config['FULL_BASE_LOCAL_POOLPATH']
     pmap = {}
     bad = {}
     logger.debug('loading all from ' + path)
@@ -297,7 +299,7 @@ def wipe_pools(poolnames, usr):
     Returns: a list of successfully removed pools names in `good`, and troubled ones in `bad` with associated exception info.
     """
     logger = current_app.logger
-    path = current_app.config['POOLPATH_BASE']
+    path = current_app.config['FULL_BASE_LOCAL_POOLPATH']
     logger.debug('DELETING pools contents from ' + path)
 
     # alldirs = poolnames if poolnames else get_name_all_pools(path)
@@ -351,38 +353,51 @@ def get_pool_info(poolname, serialize_out=False):
     ts = time.time()
     FAILED = '"FAILED"' if serialize_out else 'FAILED'
 
-    allpools = get_name_all_pools(current_app.config['POOLPATH_BASE'])
+    allpools = get_name_all_pools(
+        current_app.config['FULL_BASE_LOCAL_POOLPATH'])
     if poolname in allpools:
+
         code, result, mes = load_HKdata([poolname], serialize_out=True)
-        # use json.loads to avoid _STID for human
-        result = json.loads(result)
-        # move 'classes' to the last
-        c = result['classes']
-        del result['classes']
-        result['classes'] = c
-        # print(result['classes'])
-        # for n in ['classes', 'urns', 'tags']:
-        #    del result[n]['_STID']
+
+        result = deserialize(result, int_key=True)
+
         # Add url to tags
-        for t, ulist in result['tags'].items():
+        dt_display = {}
+        for t, clses in result['dTags'].items():
             if t == '_STID':
                 continue
-            udict = {}
-            for u in ulist['urns']:
-                pn, cl, sn = parseUrn(u)
-                udict[u] = request.base_url + cl + '/' + str(sn)
-            ulist['urns'] = udict
-       # add urls to urns
-        for u, base in result['urns'].items():
-            if u == '_STID':
+            cdict = {}
+            for cls, sns in clses.items():
+                sndict = dict((int(sn), request.base_url+cls+'/'+sn)
+                              for sn in sns)
+                cdict[cls] = sndict
+            dt_display[t] = cdict
+        display = {'Tags with URLs': dt_display}
+        # add urls to urns
+        ty_display = {}
+        rec_u = 0
+        for cl, sns in result['dTypes'].items():
+            if cl == '_STID':
                 continue
-            base['meta'] = str(base['meta'])
-            pn, cl, sn = parseUrn(u)
-            base['url'] = request.base_url + cl + '/' + str(sn)
+            snd_url = {}
+            # sns is like {'currentSN': 2,
+            # 'sn': {'0': {'tags': [], 'meta': [14, 2779]}, '1': ...
+            rec_u += len(sns['sn'])
+            for sn, snd in sns['sn'].items():
+                # str makes a list to show in ome line
+                base = {'tags': str(snd['tags']),
+                        'meta': str(snd['meta']),
+                        'url': request.base_url + cl + '/' + str(sn)
+                        }
+                snd_url[sn] = base
+            ty_display[cl] = snd_url
+        display['Datatypes with URLs'] = ty_display
         msg = 'Getting pool %s information. %s.' % (poolname, mes)
+        _, count, _ = get_data_count(None, poolname)
+        msg += '%d data items recorded. %d counted.' % (rec_u, count)
     else:
         code, result, msg = 404, FAILED, poolname + ' is not an exisiting Pool ID.'
-    return resp(code, result, msg, ts, False)
+    return resp(code, display, msg, ts, False)
 
 
 ######################################
@@ -416,7 +431,8 @@ def register_pool(pool, usr):
     :returns: code, pool object if successful, message
     """
     poolname = pool
-    fullpoolpath = join(current_app.config['POOLPATH_BASE'], poolname)
+    fullpoolpath = join(
+        current_app.config['FULL_BASE_LOCAL_POOLPATH'], poolname)
     poolurl = current_app.config['POOLURL_BASE'] + poolname
     makenew = usr and usr.role == 'read_write'
     try:
@@ -602,7 +618,8 @@ def load_single_HKdata(paths, serialize_out=True):
     poolname = '/'.join(paths[: -2])
     poolurl = current_app.config['POOLURL_BASE'] + poolname
     # resourcetype = fullname(data)
-
+    if hkname not in HKDBS:
+        raise ValueError('Invalid HK type. Must be one of '+', '.join(HKDBS))
     try:
         poolobj = PM.getPool(poolname=poolname, poolurl=poolurl)
         result = poolobj.readHK(hkname, serialize_out=serialize_out)
@@ -612,54 +629,93 @@ def load_single_HKdata(paths, serialize_out=True):
     return code, result, msg
 
 ######################################
-####  {pool}/count/{kind}          ####
+####  {pool}/count/{data_type}          ####
 ######################################
 
 
-@ pools_api.route('/<string:pool>/count/<string:data_type>', methods=['GET'])
-# @ pools_api.route('/<string:pool>/count/<string:data_type>/', methods=['GET'])
-def count(pool, data_type):
+# @ pools_api.route('/<string:pool>/count/<string:data_type>', methods=['GET'])
+def count_singleXXX(pool, data_type):
     """ Returns the number of given type of data in the given pool.
 
     :data_type:  (part of) dot-separated full class name of data items in pool.
     """
 
+    return count_general(pool=pool, data_type=data_type, logger=current_app.logger)
+
+
+@ pools_api.route('/<string:pool>/count', methods=['GET'])
+@ pools_api.route('/<string:pool>/count/<string:data_type>', methods=['GET'])
+def count(pool, data_type=None):
+    """ Returns the number of all types or of a given type of data in the given pool.
+
+    Parameter
+
+    :data_type:  (part of) dot-separated full class name of data items in pool.
+    """
+    logger = current_app.logger
+    # return count_general(pool=pool, data_type=data_type, logger=current_app.logger)
+
+    # def count_general(pool, data_type, logger):
+    ts = time.time()
+    poolname = pool.strip('/')
+    poolurl = current_app.config['POOLURL_BASE'] + poolname
+    logger.debug(f'get {data_type} count for ' + pool)
+
+    try:
+        poolobj = PM.getPool(poolname=poolname, poolurl=poolurl)
+        result = poolobj.getCount(data_type)
+        code, msg = 200, 'Recorded count of %s returned OK' % (
+            data_type if data_type else 'all types')
+    except Exception as e:
+        code, result, msg = excp(e, serialize_out=False)
+
+    return resp(code, result, msg, ts, serialize_out=False)
+
+
+@ pools_api.route('/<string:pool>/count/', methods=['GET'])
+@ pools_api.route('/<string:pool>/count/<string:data_type>/', methods=['GET'])
+def counted(pool, data_type=None):
+    """ Counts and Returns the number of given type of data in the given pool.
+
+    :data_type1:  (part of) dot-separated full class name of data items in pool.
+    """
+
     logger = current_app.logger
 
     ts = time.time()
     pool = pool.strip('/')
-    logger.debug(f'get {data_type} count for ' + pool)
+    logger.debug(f'count {data_type} type for ' + pool)
 
-    code, result, msg = get_prod_count(data_type, pool)
+    code, result, msg = get_data_count(data_type=data_type, pool_id=pool)
 
     return resp(code, result, msg, ts, serialize_out=False)
 
 
-def get_prod_count(prod_type, pool_id):
-    """ Return the total count for the given product type and pool_id in the directory.
+def get_data_count(data_type, pool_id):
+    """ Return the total count for the given product type, or all types, and pool_id in the directory.
 
-    'prod_type': (part of) 'clsssname',
+    'data_type': (part of) 'clsssname'. `None` for all types.
     'pool_id': 'pool name'
 
     """
 
     logger = current_app.logger
-    logger.debug('### method %s prod_type %s pool %s***' %
-                 (request.method, prod_type, pool_id))
+    logger.debug('### method %s data_type %s pool %s***' %
+                 (request.method, data_type, pool_id))
     res = 0
     nm = []
 
-    path = join(current_app.config['POOLPATH_BASE'], pool_id)
+    path = join(current_app.config['FULL_BASE_LOCAL_POOLPATH'], pool_id)
     if os.path.exists(path):
         for i in os.listdir(path):
-            if i[-1].isnumeric() and prod_type in i:
+            if i[-1].isnumeric() and ((data_type is None) or (data_type in i)):
                 res = res+1
                 nm.append(i)
     else:
         return 400, 'FAILED', f'Pool {pool} not found.'
     s = str(nm)
-    logger.debug(prod_type + ' found '+s)
-    return 200, res, 'Counting %d %s files OK.' % (res, prod_type)
+    logger.debug(str(data_type) + ' found '+s)
+    return 200, res, 'Counted %d %s files OK.' % (res, data_type if data_type else 'all types')
 
 
 ######################################
@@ -667,8 +723,8 @@ def get_prod_count(prod_type, pool_id):
 ######################################
 
 
-@ pools_api.route('/<string:pool>/api/<string:method_args>', methods=['GET'])
-@ pools_api.route('/<string:pool>/api/<string:method_args>/', methods=['GET'])
+@ pools_api.route('/<string:pool>/api/<string:method_args>', methods=['GET', 'POST'])
+@ pools_api.route('/<string:pool>/api/<string:method_args>/', methods=['GET', 'POST'])
 @ auth.login_required(role='read_write')
 def api(pool, method_args):
     """ Call api mathods on the running pool and returns the result.
@@ -678,17 +734,27 @@ def api(pool, method_args):
     logger = current_app.logger
 
     ts = time.time()
-    logger.debug(f'get API {method_args} for {pool}')
-
-    paths = [pool, 'api', method_args]
+    logger.debug('get API for %s ; %s.' % (pool, lls(method_args, 200)))
+    if request.method == 'POST':
+        # long args are sent with POST
+        if request.data is None:
+            result, msg = '"FAILED"', 'No REquest data for command '+request.method
+            code = 400
+            return resp(code, result, msg, ts, serialize_out=True)
+        data = str(request.data, encoding='ascii')
+        paths = [pool, 'api', method_args, data]
+    else:
+        paths = [pool, 'api', method_args]
     lp0 = len(paths)
 
-    code, result, msg = call_pool_Api(paths, serialize_out=False)
+    posted = request.method == 'POST'
+    code, result, msg = call_pool_Api(
+        paths, serialize_out=False, posted=posted)
 
     return resp(code, result, msg, ts, serialize_out=False)
 
 
-def call_pool_Api(paths, serialize_out=False):
+def call_pool_Api(paths, serialize_out=False, posted=False):
     """ Call api mathods on the running pool and returns the result.
 
     return: value if args is pool property; execution result if method. 
@@ -698,25 +764,33 @@ def call_pool_Api(paths, serialize_out=False):
     logger = current_app.logger
     ts = time.time()
 
-    args, kwds = [], {}
+    if posted:
+        code = 200
+        try:
+            m_args, kwds = tuple(deserialize(paths[3]))
+        except ValueError as e:
+            code = 422
+        m_args.insert(0, paths[2])
+    else:
+        args, kwds = [], {}
 
-    # the unquoted args. may have ',' in strings
-    # quoted_m_args = paths[ind_meth+1]
+        # the unquoted args. may have ',' in strings
+        # quoted_m_args = paths[ind_meth+1]
+
+        # from the unquoted url extract the fist path segment.
+        quoted_m_args = request.url.split(
+            paths[0] + '/' + paths[1] + '/')[1].strip('/')
+        logger.debug('get API : %s' % lls(quoted_m_args, 1000))
+        # get command positional arguments and keyword arguments
+        code, m_args, kwds = deserialize_args(
+            quoted_m_args, serialize_out=serialize_out)
 
-    # from the unquoted url extract the fist path segment.
-    quoted_m_args = request.url.split(
-        paths[0] + '/' + paths[1] + '/')[1].strip('/')
-    logger.debug(f'get API {quoted_m_args}')
-    # get command positional arguments and keyword arguments
-    code, m_args, kwds = deserialize_args(
-        quoted_m_args, serialize_out=serialize_out)
     if code != 200:
         result, msg = m_args, kwds
         return 0, resp(422, result, msg, ts, serialize_out=False), 0
 
     method = m_args[0]
-    # if 'getPoolurl' in method:
-    #    __import__('pdb').set_trace()
+
     if method not in WebAPI:
         return 0, resp(400, FAILED,
                        'Unknown web API method: %s.' % method,
@@ -725,7 +799,7 @@ def call_pool_Api(paths, serialize_out=False):
     kwdsexpr = [str(k)+'='+str(v) for k, v in kwds.items()]
     msg = '%s(%s)' % (method, ', '.join(
         chain(map(str, args), kwdsexpr)))
-    logger.debug('WebAPI ' + msg)
+    logger.debug('WebAPI ' + lls(msg, 300))
     # if args and args[0] == 'select':
     #    __import__('pdb').set_trace()
 
@@ -748,3 +822,11 @@ def call_pool_Api(paths, serialize_out=False):
         logger.error(msg)
 
     return 0, resp(code, result, msg, ts, serialize_out=False), 0
+
+
+@pools_api.before_request
+def before_request_callback():
+    path = request.path
+    method = request.method
+
+    print("pools_api " + path + " >>>>>>[" + method + "]<<<<<<")
