@@ -1,16 +1,19 @@
 # -*- coding: utf-8 -*-
 
 from .productpool import ManagedPool, PoolNotFoundError, MetaData_Json_Start, MetaData_Json_End
-from ..utils.common import pathjoin, trbk
+from ..utils.common import pathjoin, trbk, find_all_files
+from .dicthk import HKDBS
 from .urn import makeUrn, Urn, parseUrn
 from ..dataset.deserialize import deserialize
 
+
+import tarfile
 import filelock
 from functools import lru_cache
 import sys
 import shutil
 import mmap
-import time
+import io
 import os
 from os import path as op
 import logging
@@ -73,6 +76,7 @@ class LocalPool(ManagedPool):
             return True
 
         real_poolpath = self.transformpath(self._poolname)
+
         if not op.exists(real_poolpath):
             if self._makenew:
                 os.makedirs(real_poolpath)
@@ -83,7 +87,7 @@ class LocalPool(ManagedPool):
         self._atimes = {}
         self._cached_files = {}
 
-        c, t, u = tuple(self.readHK().values())
+        c, t, u, dTypes, dTags = tuple(self.readHK().values())
 
         logger.debug('created ' + self.__class__.__name__ + ' ' + self._poolname +
                      ' at ' + real_poolpath + ' HK read.')
@@ -91,17 +95,27 @@ class LocalPool(ManagedPool):
         self._classes.update(c)
         self._tags.update(t)
         self._urns.update(u)
-        fp0 = self.transformpath(self._poolname)
-        self.writeHK(fp0)
+        # new ####
+        if len(self._dTypes) or len(self._dTags):
+            raise ValueError('self._dTypes or self._dTags not empty %s %s' % (
+                str(self._dTypes), str(self._dTags)))
+
+        self._dTypes.update(dTypes)
+        self._dTags.update(dTags)
+        # /new ###
+
+        if any(not op.exists(op.join(real_poolpath, hk+'.jsn'))
+               for hk in HKDBS):
+            self.writeHK(real_poolpath)
         return False
 
-    def readmmap(self, filename,  start=None, end=None, close=False, check_time=False):
+    def readmmap(self, filename, start=None, end=None, close=False, check_time=False):
         fp = op.abspath(filename)
         if check_time:
             sr = os.stat(fp)
-        if check_time and fp in self._atimes and (sr.st_mtime_ns <= self._atimes[fp]):
-            # file hasnot changed since last time we read/wrote it.
-            return None
+            if fp in self._atimes and (sr.st_mtime_ns <= self._atimes[fp]):
+                # file hasnot changed since last time we read/wrote it.
+                return None
         try:
             if 1:  # if fp not in self._files or self._files[fp] is None:
                 file_obj = open(fp, mode="r+", encoding="utf-8")
@@ -130,15 +144,16 @@ class LocalPool(ManagedPool):
             self._atimes[fp] = sr.st_mtime_ns
         return js.decode('ascii')
 
-    def readHK(self, hktype=None, serialize_out=False):
+    def readHK(self, hktype=None, serialize_out=False, all_versions=True):
         """
-        loads and returns the housekeeping data
+        loads and returns the housekeeping data, or empty `dict` if not found.
 
-        hktype: one of 'classes', 'tags', 'urns' to return. default is None to return alldirs
+        hktype: one of the mappings listed in `dicthk.HKDBS`.
         serialize_out: if True return serialized form. Default is false.
         """
         if hktype is None:
-            hks = ['classes', 'tags', 'urns']
+            # some new ####
+            hks = HKDBS
         else:
             hks = [hktype]
         fp0 = self.transformpath(self._poolname)
@@ -146,14 +161,14 @@ class LocalPool(ManagedPool):
         hk = {}
         for hkdata in hks:
             fp = op.abspath(pathjoin(fp0, hkdata + '.jsn'))
-            if op.exists(fp):
+            if op.exists(fp) and (all_versions or ('dT' in fp)):
                 js = self.readmmap(fp, check_time=True)
                 if js:
                     if serialize_out:
                         r = js
                     else:
                         from ..dataset.deserialize import deserialize
-                        r = deserialize(js)
+                        r = deserialize(js, int_key=True)
                     self._cached_files[fp] = js
                 else:
                     # the file hasnot changed since last time we r/w it.
@@ -161,12 +176,11 @@ class LocalPool(ManagedPool):
                         self.__getattribute__('_' + hkdata)
             else:
                 if serialize_out:
-                    r = '{"_STID":"ODict"}'
+                    r = '{}'  # '{"_STID":"ODict"}'
                 else:
                     from ..dataset.odict import ODict
-                    r = ODict()
+                    r = {}  # ODict()
             hk[hkdata] = r
-            assert r is not None
         logger.debug('HK read from ' + fp0)
         if serialize_out:
             return '{%s}' % ', '.join(('"%s": %s' % (k, v) for k, v in hk.items())) if hktype is None else hk[hktype]
@@ -228,46 +242,55 @@ class LocalPool(ManagedPool):
         else:
             return l
 
-    def writeHK(self, fp0=None):
+    def writeHK(self, fp0=None, all_versions=True):
         """ save the housekeeping data to disk
         """
         if fp0 is None:
             fp0 = self._poolpath + '/' + self._poolname
         l = 0
-        for hkdata in ['classes', 'tags', 'urns']:
+        for hkdata in HKDBS:
+            if not all_versions and not 'dT' in hkdata:
+                continue
             fp = pathjoin(fp0, hkdata + '.jsn')
             l += self.writeJsonmmap(fp, self.__getattribute__('_' + hkdata),
                                     check_time=True)
         return l
 
-    def setMetaByUrn(self, start, end, urn):
+    def setMetaByUrn(self, start, end, urn=None, datatype=None, sn=None):
         """
-        Sets the location of the meta data of the specified data to the given URN.
+        Sets the location of the meta data of the specified data to the given URN or a pair of data type and serial number.
 
         :data: usually serialized Product.
         """
-        u = urn.urn if issubclass(urn.__class__, Urn) else urn
-        if u not in self._urns:
-            raise ValueError(urn + ' not found in pool ' + self._poolname)
+        u, datatype, sn = self.get_missing(urn=urn, datatype=datatype, sn=sn)
         # char offset of the start and end points of metadata
         if start >= 0 and end > 0:
             mt = [start, end]
         else:
             mt = [None, None]
-        self._urns[u]['meta'] = mt
+        # new ##
+        self._dTypes[datatype]['sn'][sn]['meta'] = mt
+        ####self._urns[u]['meta'] = mt
 
     @ lru_cache(maxsize=1024)
     def getMetaByUrnJson(self, urn, resourcetype, index):
+        """ like `getMetaByUrn` but returns un-deserialized form."""
 
+        urn, datatype, sn = self.get_missing(
+            urn=urn, datatype=resourcetype, sn=index)
         # deserialize(prd[start+len(MetaData_Json_Start):end+len(MetaData_Json_End)])
         try:
-            start, end = tuple(self._urns[urn]['meta'])
+            # new ##
+            # assert tuple(self._urns[urn]['meta']) == \
+            # tuple(self._dTypes[datatype]['sn'][sn]['meta'])
+            start, end = tuple(self._dTypes[datatype]['sn'][sn]['meta'])
+            ####start, end = tuple(self._urns[urn]['meta'])
         except KeyError as e:
-            msg = f"Trouble with {self._poolname}._urns[urn]['meta']"
+            msg = f"Trouble with {self._poolname}...['meta']"
             logger.debug(msg)
-            raise e
-        js = self.schematicLoad(resourcetype=resourcetype,
-                                index=index, start=start, end=end,
+            raise
+        js = self.schematicLoad(resourcetype=datatype,
+                                index=sn, start=start, end=end,
                                 serialize_out=True)
 
         return js
@@ -281,20 +304,15 @@ class LocalPool(ManagedPool):
 
     def getMetaByUrn(self, urn, resourcetype=None, index=None):
         """ 
-        Get all of the meta data belonging to a product of a given URN.
+        Get all of the meta data belonging to a product of a given URN or a pair of data type and serial number.
 
         mh: returns an iterator.
         """
-        if urn is None:
-            return None  # TODO self._meta.keys()
-        #uobj = Urn(urn=urn)
-        if resourcetype is None or index is None:
-            poolname, resourcetype, index = parseUrn(urn)
         m = self.getMetaByUrnJson(urn, resourcetype=resourcetype,
                                   index=index)
         return deserialize(m)  # self._urns[urn]['meta']
 
-    def doSave(self, resourcetype, index, data, tag=None, serialize_in=True, **kwds):
+    def doSave(self, resourcetype, index, data, tags=None, serialize_in=True, **kwds):
         """
         does the media-specific saving.
 
@@ -355,7 +373,7 @@ class LocalPool(ManagedPool):
         except IOError as e:
             logger.error('Remove failed. exc: %s trbk: %s.' %
                          (str(e), trbk(e)))
-            raise e  # needed for undoing HK changes
+            raise  # needed for undoing HK changes
         return 0
 
     def doWipe(self):
@@ -369,9 +387,7 @@ class LocalPool(ManagedPool):
         self._files.clear()
         self._atimes.clear()
         self._cached_files.clear()
-        self._classes.clear()
-        self._tags.clear()
-        self._urns.clear()
+
         # will leave a newly made pool dir
         wipeLocal(self.transformpath(self._poolname))
         return 0
@@ -382,3 +398,36 @@ class LocalPool(ManagedPool):
         """
 
         raise(NotImplementedError())
+
+    def backup(self):
+        """ make a tarfile string into a string """
+
+        fp0 = self.transformpath(self._poolname)
+        logger.info('Making a gz tar file of %s for pool %s.' %
+                    (fp0, self._poolname))
+        with filelock.FileLock(self.lockpath('r')):
+            # Save unsaved changes
+            self.writeHK(fp0)
+            with io.BytesIO() as iob:
+                with tarfile.open(None, 'w|gz', iob) as tf:
+                    tar = tf.add(fp0, arcname='.')
+                file_image = iob.getvalue()
+        return file_image
+
+    def restore(self, tar):
+        """untar the input file to this pool and return the file list."""
+
+        with filelock.FileLock(self.lockpath('w')):
+            fp0 = self.transformpath(self._poolname)
+            self.doWipe()
+            with io.BytesIO(tar) as iob:
+                with tarfile.open(None, 'r|gz', iob) as tf:
+                    tf.extractall(fp0)
+            allf = find_all_files(fp0)
+            # read into memory
+            self._classes, self._tags, self._urns, self._dTypes, self._dTags = tuple(
+                self.readHK().values())
+        logger.info('Restored from a gz tar file to %s for pool %s. %d files.' %
+                    (fp0, self._poolname, len(allf)))
+
+        return allf
