@@ -1,5 +1,6 @@
 
 # -*- coding: utf-8 -*-
+import pdb
 
 from ..utils.getconfig import getConfig
 from ..utils.common import lls
@@ -21,22 +22,29 @@ pc = getConfig()
 DEFAULT_MEM_POOL = 'defaultmem'
 # localpool
 DEFAULT_POOL = 'fdi_pool_' + __name__ + getpass.getuser()
-Invalid_Pool_Names = ['pools', 'urn', 'api']
+Invalid_Pool_Names = ['pools', 'urn', 'URN', 'api']
 
 
-def remoteRegister(p, poolurl):
+def remoteRegister(poolurl, auth=None):
     logger.debug('Register %s on the server', poolurl)
+    if poolurl.endswith('/'):
+        poolurl = poolurl[:-1]
     try:
-        res, msg = put_on_server('urn:::0', poolurl, 'register_pool')
+        res, msg = put_on_server(
+            'urn:::0', poolurl, 'register_pool', auth=auth)
     except ConnectionError as e:
         res, msg = 'FAILED', str(e)
+        logger.error(poolurl + ' ' + msg)
+        raise
     if res == 'FAILED':
+        np = '<' + auth.username + ' ' + auth.password + \
+            '>' if auth else '<no authorization>'
         raise RuntimeError(
-            'Registering ' + poolurl + ' failed.  ' + msg)
+            'Registering ' + poolurl + ' failed with auth ' + np + ' , ' + msg)
     return res, msg
 
 
-def remoteUnregister(poolurl):
+def remoteUnregister(poolurl, auth=None):
     """ this method does not reference pool object. """
     if not poolurl.lower().startswith('http'):
         logger.warning('Ignored: %s not for a remote pool.' % poolurl)
@@ -47,7 +55,8 @@ def remoteUnregister(poolurl):
     #o = deserialize(x.text)
     urn = 'urn:::0'
     try:
-        res, msg = delete_from_server(urn, poolurl, 'unregister_pool')
+        res, msg = delete_from_server(
+            urn, poolurl, 'unregister_pool', auth=auth)
     except ConnectionError as e:
         res, msg = 'FAILED', str(e)
     if res == 'FAILED':
@@ -72,7 +81,7 @@ This is done by calling the getPool() method, which will return an existing pool
     # pc['node']['host']+':'+str(pc['node']['port'])+pc['baseurl']
     p = getConfig(name='').strip('/').split('://')[1]
     PlacePaths = {
-        'file': pc['base_poolpath'],
+        'file': pc['base_local_poolpath'],
         'mem': '/',
         'http': p,
         'https': p,
@@ -81,7 +90,7 @@ This is done by calling the getPool() method, which will return an existing pool
     del p
 
     @classmethod
-    def getPool(cls, poolname=None, poolurl=None, pool=None, makenew=True, **kwds):
+    def getPool(cls, poolname=None, poolurl=None, pool=None, makenew=True, auth=None, **kwds):
         """ returns an instance of pool according to name or path of the pool.
 
         Returns the pool object if the pool is registered. Creates the pool if it does not already exist. the same poolname-path always get the same pool. Http pools will be registered on the sserver side.
@@ -104,7 +113,7 @@ If poolname is missing it is derived from poolurl; if poolurl is also absent, Va
                     'Pool name %s and pool object cannot be both given.' % poolname)
             poolname, poolurl, p = pool._poolname, pool._poolurl, pool
             if poolurl.lower().startswith('http'):
-                res, msg = remoteRegister(p, poolurl)
+                res, msg = remoteRegister(poolurl, p.auth)
         else:
             # quick decisions can be made knowing poolname only
             if poolname == DEFAULT_MEM_POOL:
@@ -150,8 +159,12 @@ If poolname is missing it is derived from poolurl; if poolurl is also absent, Va
             elif schm in ('http', 'https'):
                 from . import httpclientpool
                 p = httpclientpool.HttpClientPool(
-                    poolname=poolname, poolurl=poolurl, **kwds)
-                res, msg = remoteRegister(p, poolurl)
+                    poolname=poolname, poolurl=poolurl, auth=auth, **kwds)
+                res, msg = remoteRegister(poolurl, auth=p.auth)
+            elif schm == 'csdb':
+                from . import publicclientpool
+                p = publicclientpool.PublicClientPool(poolurl=poolurl)
+                # res, msg = remoteRegister(poolurl, auth=p.auth)
             else:
                 raise NotImplementedError(schm + ':// is not supported')
         #print(getweakrefs(p), id(p), '////')
@@ -210,9 +223,10 @@ If poolname is missing it is derived from poolurl; if poolurl is also absent, Va
 
         if nwr == 1:
             # this is the only reference. unregister remote first.
-            poolurl = cls._GlobalPoolList[poolname]._poolurl
+            thepool = cls._GlobalPoolList[poolname]
+            poolurl = thepool._poolurl
             if poolurl.lower().startswith('http'):
-                code = remoteUnregister(poolurl)
+                code = remoteUnregister(poolurl, thepool.auth)
             else:
                 code = 0
         elif nwr > 1:
