@@ -52,13 +52,7 @@ def makeLockpath(direc, op='w'):
 
     lp = pathjoin(Lock_Path_Base, direc.replace('/', '_'))
 
-    if 1:
-        return lp+'.read' if op == 'r' else lp+'.write'
-    else:
-        if not os.path.exists(lp):
-            os.makedirs(lp)
-            lf = pathjoin(lp, 'lock')
-        return lf+'.read' if op == 'r' else lf+'.write'
+    return lp+'.read' if op == 'r' else lp+'.write'
 
 
 class ProductPool(Definable, Taggable, Versionable):
@@ -82,7 +76,7 @@ When implementing a ProductPool, the following rules need to be applied:
         * poolurl: needed to initialize.
 
         """
-        super(ProductPool, self).__init__(**kwds)
+        super().__init__(**kwds)
 
         self.setPoolname(poolname)
         self.setPoolurl(poolurl)
@@ -188,7 +182,7 @@ When implementing a ProductPool, the following rules need to be applied:
         """
         Returns pool definition info which contains pool type and other pool specific configuration parameters
         """
-        return super(ProductPool, self).getDefinition()
+        return super().getDefinition()
 
     def getId(self):
         """
@@ -256,11 +250,13 @@ When implementing a ProductPool, the following rules need to be applied:
         Saves specified product and returns the designated ProductRefs or URNs.
 
         Saves a product or a list of products to the pool, possibly under the
-        supplied tag, and returns the reference (or a list of references if
+        supplied tag(s), and returns the reference (or a list of references if
         the input is a list of products), or Urns if geturnobjs is True.
 
         See pal document for pool structure.
 
+
+        tag: a tag if given as a string type; a list of tags if given a list.
         serialize_out: if True returns contents in serialized form.
         """
 
@@ -327,7 +323,19 @@ When implementing a ProductPool, the following rules need to be applied:
 
         raise(NotImplementedError)
 
-    def getCount(self, typename):
+    @property
+    def count(self):
+        """ for property getter
+        """
+        return self.getCount()
+
+    @count.setter
+    def count(self, count):
+        """ for property setter
+        """
+        raise ValueError('Pool.count is read-only.')
+
+    def getCount(self, typename=None):
         """
         Return the number of URNs for the product type.
         """
@@ -346,17 +354,11 @@ When implementing a ProductPool, the following rules need to be applied:
         """
         raise(NotImplementedError)
 
-    def remove(self, urn):
+    def remove(self, urn=None, resourcetype=None, index=None):
         """
-        Removes a Product belonging to specified URN.
+        Removes a Product belonging to specified URN or a pair of data type and serial number.
         """
-        poolname, resource, index = parseUrn(urn)
-
-        if self._poolname != poolname:
-            raise ValueError(
-                urn + ' is not from the pool ' + self._poolname)
-
-        res = self.schematicRemove(urn, resourcetype=resource, index=index)
+        res = self.schematicRemove(urn, resourcetype=resourcetype, index=index)
         return res
 
     def schematicWipe(self):
@@ -429,7 +431,7 @@ class ManagedPool(ProductPool, DictHk):
     """ A ProductPool that manages its internal house keeping. """
 
     def __init__(self, **kwds):
-        super(ManagedPool, self).__init__(**kwds)
+        super().__init__(**kwds)
         # {type|classname -> {'sn:[sn]'}}
 
     def setup(self):
@@ -454,6 +456,8 @@ class ManagedPool(ProductPool, DictHk):
         if super().setup():
             return True
         self._classes = dict()
+        # new ##
+        self._dTypes = dict()
         return False
 
     def getPoolpath(self):
@@ -496,13 +500,24 @@ class ManagedPool(ProductPool, DictHk):
         """
         Decrement the reference count of a ProductRef.
         """
+        # new ###
+        poolname, dt, sn = parseUrn(urn, int_index=True)
+        # assert self._urns[ref.urn]['refcnt'] == self._dType[dt]['sn'][sn]['refcnt']
+        self._dType[dt]['sn'][sn]['refcnt'] -= 1
+        # /new ###
+        # self._urns[ref.urn]['refcnt'] -= 1
 
-        self._urns[ref.urn]['refcnt'] -= 1
-
-    def exists(self, urn):
+    def exists(self, urn, resourcetype=None, index=None):
         """
         Determines the existence of a product with specified URN.
         """
+        # new ###
+        try:
+            urn, datatype, sn = self.get_missing(urn, resourcetype, index)
+        except KeyError:
+            return False
+        # return True
+        # /new#
         return urn in self._urns
 
     def getProductClasses(self):
@@ -510,18 +525,26 @@ class ManagedPool(ProductPool, DictHk):
         Returns all Product classes found in this pool.
         mh: returns an iterator.
         """
+        # new ###
+        assert list(self._classes.keys()) == list(self._dTypes.keys())
         return self._classes.keys()
 
-    def getCount(self, typename):
+    def getCount(self, typename=None):
         """
         Return the number of URNs for the product type.
         """
         try:
-            return len(self._classes[typename]['sn'])
+            # new ###
+            # assert len(self._classes[typename]['sn']) == len(
+            # self._dTypes[typename]['sn'])
+            if typename:
+                return len(self._dTypes[typename]['sn'])
+            else:
+                return sum(len(dt['sn']) for dt in self._dTypes.values())
         except KeyError:
             return 0
 
-    def doSave(self, resourcetype, index, data, tag=None, serialize_in=True, **kwds):
+    def doSave(self, resourcetype, index, data, tags=None, serialize_in=True, **kwds):
         """ to be implemented by subclasses to do the action of saving
         """
         raise(NotImplementedError)
@@ -530,56 +553,122 @@ class ManagedPool(ProductPool, DictHk):
         """
         Returns the reference count of a ProductRef.
         """
-        return self._urns[ref.urn]['refcnt']
+        # new ###
+        poolname, dt, sn = parseUrn(urn, int_index=False)
+        _snd = self._dType[dt]['sn'][sn]
+        # assert self._urns[ref.urn]['refcnt'] == _snd['refcnt']
+        return _snd['refcnt']
+        # /new ###
+        # return self._urns[ref.urn]['refcnt']
 
     def isEmpty(self):
         """
         Determines if the pool is empty.
         """
-        return len(self._urns) == 0
+        # new ###
+        # assert len(self._urns) == len(self._dTypes)
+        # return len(self._urns) == 0
+        return len(self._dTypes) == 0
 
-    def loadDescriptors(self, urn):
+    def loadDescriptors(self, urn, resourcetype=None, index=None):
         """
         Loads the descriptors belonging to specified URN.
         """
-        return self._urns[urn]
+        # new ###
+        urn, datatype, sn = self.get_missing(urn, resourcetype, index)
+        return self._dTypes
+        # return self._urns[urn]
 
-    def setMetaByUrn(self, start, end, urn):
+    def setMetaByUrn(self, start, end, urn=None, datatype=None, sn=None):
         """
-        Sets the location of the meta data of the specified data to the given URN.
+        Sets the location of the meta data of the specified data to the given URN or a pair of data type and serial number.
 
         :data: usually un/serialized Product.
+
+        Return
+        :urn: stirng
+        :datatype: class name
+        :sn: serial number in string.
         """
-        u = urn.urn if issubclass(urn.__class__, Urn) else urn
-        if u not in self._urns:
-            raise ValueError(urn + ' not found in pool ' + self._poolname)
 
         raise NotImplemented
 
-    def getMetaByUrn(self, urn, resourcetype=None, index=None):
-        """ 
+    def getMetaByUrn(self, urn=None, resourcetype=None, index=None):
+        """
         Get all of the meta data belonging to a product of a given URN.
 
         mh: returns an iterator.
         """
         raise NotImplemented
 
-    def meta(self,  urn):
+    def meta(self, *args, **kwds):
         """
         Loads the meta-data info belonging to the product of specified URN.
         """
-        return self.getMetaByUrn(urn)
+        return self.getMetaByUrn(*args, **kwds)
 
     def reference(self, ref):
         """
         Increment the reference count of a ProductRef.
         """
-        self._urns[ref.urn]['refcnt'] += 1
+        # new ###
+        poolname, dt, sn = parseUrn(ref.urn, int_index=True)
+        _snd = self._dType[dt]['sn'][sn]
+        if 'refcnt' not in _snd:
+            _snd['refcnt'] = 0
+        ####assert self._urns[ref.urn]['refcnt'] == _snd['refcnt']
+        _snd['refcnt'] += 1
+        # /new ###
+        if 0:
+            if 'refcnt' not in self._urns:
+                self._urns['refcnt'] = 0
+            self._urns[ref.urn]['refcnt'] += 1
 
     def saveOne(self, prd, tag, geturnobjs, serialize_in, serialize_out, res, kwds):
         """
         Save one product.
 
+            # get the latest HK
+
+            the new classes and tags.
+               dTypes combines old classes and urns, is defined as::
+                     $class_name0:
+                            'currentSN': $sn
+                            'sn':
+                                $sn0:
+                                   tags: [$tag1, $tag2, ...
+                                   meta: [$start, $end]
+                                   refcnt: $count
+                                $sn1:
+                                   tags: [$tag1, $tag2, ...
+                                   meta: [$start, $end]
+                                   refcnt: $count
+               example::
+                    foo.bar.Bar:
+                            'currentSN': 1
+                            0:
+                               tags: ['cat', 'white']
+                               meta: [123, 456]
+                               refcnt: 0
+                            1:
+                               tags: ['dog', 'white']
+                               meta: [321, 765]
+                               refcnt: 0
+                    foo.baz.Baz:
+                            'currentSN': 34
+                            34:
+                               tags: ['tree', 'green']
+                               meta: [100, 654]
+                               refcnt: 1
+     `dTags` differs from `tag` 1. uses dType:[sn], instead of urn, so there is no poolname anywhere, 2. simplify by removing second level dict::
+                     $tag_name0:
+                           $class_name1:[$sn1, $sn2...]
+                           $class_name2:[$sn3, ...]
+              exxample::
+                     'cat': { 'foo.bar.Bar':[0] }
+                     'white': { 'foo.bar.Bar'; [0, 1] }
+                     'dog': ...
+
         :res: list of result.
         :serialize_out: if True returns contents in serialized form.
         """
@@ -595,41 +684,80 @@ class ManagedPool(ProductPool, DictHk):
         with filelock.FileLock(self.lockpath('w')), \
                 filelock.FileLock(self.lockpath('r')):
 
-            # get the latest HK
-            self._classes, self._tags, self._urns = tuple(
-                self.readHK().values())
-            c, t, u = self._classes, self._tags, self._urns
+            # some new ####
+            self._classes, self._tags, self._urns, \
+                self._dTypes, self._dTags = tuple(
+                    self.readHK().values())
+            # /some new ####
+            if 0:
+                c, t, u = self._classes, self._tags, self._urns
 
-            if pn in c:
-                sn = (c[pn]['currentSN'] + 1)
+                if pn in c:
+                    sn = (c[pn]['currentSN'] + 1)
+                else:
+                    sn = 0
+                    c[pn] = dict(sn=[])
+
+                c[pn]['currentSN'] = sn
+                c[pn]['sn'].append(sn)
+
+                urn = makeUrn(poolname=self._poolname, typename=pn, index=sn)
+
+                if urn not in u:
+                    u[urn] = dict(tags=[])
+
+            # new+old ###
+            if tag is None:
+                tags = []
+            elif issubclass(tag.__class__, str):
+                tags = [tag]
+            elif issubclass(tag.__class__, list):
+                tags = tag
             else:
-                sn = 0
-                c[pn] = dict(sn=[])
+                raise TypeError('Bad type for tag: %s.' %
+                                tag.__class__.__name__)
+            # new ####
+            dTypes, dTags = self._dTypes, self._dTags
 
-            c[pn]['currentSN'] = sn
-            c[pn]['sn'].append(sn)
+            if pn in dTypes:
+                _sn = dTypes[pn]['currentSN'] + 1
+            else:
+                _sn = 0
+                dTypes[pn] = {
+                    'currentSN': _sn,
+                    'sn': {}
+                }
 
-            urn = makeUrn(poolname=self._poolname, typename=pn, index=sn)
+            snd = dTypes[pn]['sn']
+            if _sn not in snd:
+                snd[_sn] = {
+                    'tags': [],
+                    'meta': []
+                }
 
-            if urn not in u:
-                u[urn] = dict(tags=[])
+            dTypes[pn]['currentSN'] = _sn
+            urn = makeUrn(poolname=self._poolname, typename=pn, index=_sn)
+            # /new #####
+
+            for t in tags:
+                self.setTag(t, urn)
 
-            if tag is not None:
-                self.setTag(tag, urn)
             try:
                 # save prod and HK
                 self.doSave(resourcetype=pn,
-                            index=sn,
+                            index=_sn,
                             data=prd,
-                            tag=tag,
+                            tags=tags,
                             serialize_in=serialize_in,
                             serialize_out=serialize_out,
                             **kwds)
             except ValueError as e:
                 msg = 'product ' + urn + ' saving failed.' + str(e) + trbk(e)
                 logger.debug(msg)
-                self._classes, self._tags, self._urns = tuple(
-                    self.readHK().values())
+                # some new ##
+                self._classes, self._tags, self._urns, \
+                    self._dTypes, self._dTags = tuple(
+                        self.readHK().values())
                 raise e
 
         if geturnobjs:
@@ -721,33 +849,52 @@ class ManagedPool(ProductPool, DictHk):
         """ do the scheme-specific removing
         """
 
-        prod = resourcetype
-        sn = index
+        urn, datatype, sn = self.get_missing(urn, resourcetype, index)
 
         with filelock.FileLock(self.lockpath('w')),\
                 filelock.FileLock(self.lockpath('r')):
 
             # get the latest HK
-            self._classes, self._tags, self._urns = tuple(
-                self.readHK().values())
-            c, t, u = self._classes, self._tags, self._urns
+            # some new ####
+            self._classes, self._tags, self._urns, \
+                self._dTypes, self._dTags = tuple(
+                    self.readHK().values())
+            if 0:
+                c, t, u = self._classes, self._tags, self._urns
+                if urn not in u:
+                    raise ValueError(
+                        '%s not found in pool %s.' % (urn, self.getId()))
 
-            if urn not in u:
+            # new ####
+            dTypes, dTags = self._dTypes, self._dTags
+
+            if sn not in dTypes[datatype]['sn']:
                 raise ValueError(
-                    '%s not found in pool %s.' % (urn, self.getId()))
+                    '%d not found in pool %s.' % (sn, self.getId()))
+            # /new ####
 
-            self.removeUrn(urn)
-            c[prod]['sn'].remove(sn)
+            self.removeUrn(urn, datatype=datatype, sn=sn)
 
-            if len(c[prod]['sn']) == 0:
-                del c[prod]
+            if 0:
+                c[datatype]['sn'].remove(sn)
+
+                if len(c[datatype]['sn']) == 0:
+                    del c[datatype]
+            # new ####
+            if len(dTypes[datatype]['sn']) == 0:
+                del dTypes[datatype]
+            else:
+                ####assert list(dTypes[datatype]['sn']) == list(c[datatype]['sn'])
+                pass
+            # /new ####
             try:
-                self.doRemove(resourcetype=prod, index=sn)
+                self.doRemove(resourcetype=datatype, index=sn)
             except Exception as e:
                 msg = 'product ' + urn + ' removal failed'
                 logger.debug(msg)
-                self._classes, self._tags, self._urns = tuple(
-                    self.readHK().values())
+                self._classes, self._tags, self._urns, \
+                    self._dTypes, self._dTags = tuple(
+                        self.readHK().values())
                 raise e
         return 0
 
@@ -762,9 +909,14 @@ class ManagedPool(ProductPool, DictHk):
         with filelock.FileLock(self.lockpath('w')),\
                 filelock.FileLock(self.lockpath('r')):
             try:
-                self._classes.clear()
-                self._tags.clear()
-                self._urns.clear()
+                if 0:
+                    self._classes.clear()
+                    self._tags.clear()
+                    self._urns.clear()
+                # new ##
+                self._dTypes.clear()
+                self._dTags.clear()
+                # /new ##
                 self.doWipe()
             except Exception as e:
                 msg = self.getId() + 'wiping failed'
@@ -773,15 +925,19 @@ class ManagedPool(ProductPool, DictHk):
         logger.debug('Done.')
         return 0
 
-    def meta_filter(self, q, typename=None, reflist=None, urnlist=None, snlist=None):
+    def meta_filter(self, q, typename=None, reflist=None, urnlist=None, snlist=None, datatypes=None):
         """ returns filtered collection using the query.
 
         q is a MetaQuery
-        valid inputs: typename and ns list; productref list; urn list
+        valid inputs: typename and ns list; productref list; urn list; datatypes dict.
+
+        :typename: data type (class name)
+        :reflist: list of ProductRefs
+        :urnlist: list of URNs
+        :datatypes:  dict of {typename:sn_list}
         """
 
         ret = []
-        u = self._urns
         qw = q.getWhere()
 
         if reflist:
@@ -814,37 +970,50 @@ class ManagedPool(ProductPool, DictHk):
                     if qw(m):
                         ret.append(ProductRef(urn=urn, meta=m))
                 return ret
-        elif snlist:
+        elif snlist or datatypes:
             if isinstance(qw, str):
                 code = compile(qw, 'py', 'eval')
-                for n in snlist:
-                    urn = makeUrn(poolname=self._poolname,
-                                  typename=typename, index=n)
-                    m = self.getMetaByUrn(urn)
-                    if eval(code):
-                        ret.append(ProductRef(urn=urn, meta=m))
+                if snlist:
+                    datatypes = {typename: snlist}
+                for cls in datatypes:
+                    snlist = datatypes[cls]
+                    for n in snlist:
+                        urn = makeUrn(poolname=self._poolname,
+                                      typename=typename, index=n)
+                        m = self.getMetaByUrn(urn)
+                        if eval(code):
+                            ret.append(ProductRef(urn=urn, meta=m))
                 return ret
             else:
-                for n in snlist:
-                    urn = makeUrn(poolname=self._poolname,
-                                  typename=typename, index=n)
-                    m = self.getMetaByUrn(urn)
-                    if qw(m):
-                        ret.append(ProductRef(urn=urn, meta=m))
+                if snlist:
+                    datatypes = {typename: snlist}
+                for cls in datatypes:
+                    snlist = datatypes[cls]
+                    for n in snlist:
+                        urn = makeUrn(poolname=self._poolname,
+                                      typename=typename, index=n)
+                        m = self.getMetaByUrn(urn)
+                        if qw(m):
+                            ret.append(ProductRef(urn=urn, meta=m))
                 return ret
         else:
             raise('Must give a list of ProductRef or urn or sn')
 
-    def prod_filter(self, q, cls=None, reflist=None, urnlist=None, snlist=None):
+    def prod_filter(self, q, cls=None, reflist=None, urnlist=None, snlist=None, datatypes=None):
         """ returns filtered collection using the query.
 
         q: an AbstractQuery.
-        valid inputs: cls and ns list; productref list; urn list
+        valid inputs: cls and ns list; productref list; urn list; datatypes dict.
+
+        :cls: type. data type
+        :reflist: list of ProductRefs
+        :urnlist: list of URNs
+        :datatypes:  dict of {cls:sn_list}
         """
 
         ret = []
+        # will add query variable (e.g. 'p') to Global name space
         glbs = globals()
-        u = self._urns
         qw = q.getWhere()
         var = q.getVariable()
         if var in glbs:
@@ -890,31 +1059,56 @@ class ManagedPool(ProductPool, DictHk):
                 if savevar != 'not in glbs':
                     glbs[var] = savevar
                 return ret
-        elif snlist:
+        elif snlist or datatypes:
             if isinstance(qw, str):
                 code = compile(qw, 'py', 'eval')
-                for n in snlist:
-                    urno = Urn(cls=cls, poolname=self._poolname, index=n)
-                    pref = ProductRef(urn=urno)
-                    glbs[var] = pref.getProduct()
-                    if eval(code):
-                        ret.append(pref)
-                if savevar != 'not in glbs':
-                    glbs[var] = savevar
+                if snlist:
+                    datatypes = {cls.__name__: snlist}
+                for typename in datatypes:
+                    snlist = datatypes[typename]
+                    cls = Class_Look_Up[typename.rsplit('.', 1)[-1]]
+                    for n in snlist:
+                        urno = Urn(cls=cls, poolname=self._poolname, index=n)
+                        pref = ProductRef(urn=urno)
+                        glbs[var] = pref.getProduct()
+                        if eval(code):
+                            ret.append(pref)
+                    if savevar != 'not in glbs':
+                        glbs[var] = savevar
                 return ret
             else:
-                for n in snlist:
-                    urno = Urn(cls=cls, poolname=self._poolname, index=n)
-                    pref = ProductRef(urn=urno)
-                    glbs[var] = pref.getProduct()
-                    if qw(glbs[var]):
-                        ret.append(pref)
-                if savevar != 'not in glbs':
-                    glbs[var] = savevar
+                if snlist:
+                    datatypes = {cls.__name__: snlist}
+                for typename in datatypes:
+                    snlist = datatypes[typename]
+                    cls = glbs[typename]
+                    for n in snlist:
+                        urno = Urn(cls=cls, poolname=self._poolname, index=n)
+                        pref = ProductRef(urn=urno)
+                        glbs[var] = pref.getProduct()
+                        if qw(glbs[var]):
+                            ret.append(pref)
+                    if savevar != 'not in glbs':
+                        glbs[var] = savevar
                 return ret
         else:
             raise('Must give a list of ProductRef or urn or sn')
 
+    def where(self, qw, prod='BaseProduct', urns=None):
+        q = AbstractQuery(prod, 'p', qw)
+        # if urns is None:
+        # new ###
+        datatypes = dict((k, list(v['sn'].keys()))
+                         for k, v in self._dTypes.items())
+        if 0:
+            urns = self._urns.keys()
+            res = self.prod_filter(q, prod, urnlist=urns)
+        # new ###
+        res2 = self.prod_filter(q, prod, datatypes=datatypes)
+
+        #### assert [r.urn for r in res] == [r.urn for r in res2]
+        return [r.urn for r in res2]
+
     def doSelect(self, query, results=None):
         """
         to be implemented by subclasses to do the action of querying.
@@ -925,9 +1119,9 @@ class ManagedPool(ProductPool, DictHk):
         """
         do the scheme-specific querying.
         """
-        isMQ = issubclass(query.__class__, MetaQuery)
-        isAQ = issubclass(query.__class__, AbstractQuery)
-        if not isMQ and not isAQ:
+        is_MetaQ = issubclass(query.__class__, MetaQuery)
+        is_AbstQ = issubclass(query.__class__, AbstractQuery)
+        if not is_MetaQ and not is_AbstQ:
             raise TypeError('not a Query')
         lgb = Classes.mapping
         t, v, w, a = query.getType(), query.getVariable(
@@ -936,20 +1130,26 @@ class ManagedPool(ProductPool, DictHk):
         if results:
             this = (x for x in results if x.urnobj.getPoolId()
                     == self._poolname)
-            if isMQ:
+            if is_MetaQ:
                 ret += self.meta_filter(q=query, reflist=this)
             else:
                 ret += self.prod_filter(q=query, reflist=this)
         else:
-            for cname in self._classes:
-                cls = lgb[cname.split('.')[-1]]
+            # new ##
+            #### assert list(self._dTypes) == list(self._classes)
+            for cname in self._dTypes:
+                cls = lgb[cname.rsplit('.', 1)[-1]]
                 if issubclass(cls, t):
-                    if isMQ:
+                    ###snlist = self._classes[cname]['sn']
+                    # new ###
+                    ####assert snlist == list(self._dTypes[cname]['sn'])
+                    snlist = list(self._dTypes[cname]['sn'])
+                    if is_MetaQ:
                         ret += self.meta_filter(q=query, typename=cname,
-                                                snlist=self._classes[cname]['sn'])
+                                                snlist=snlist)
                     else:
                         ret += self.prod_filter(q=query, cls=cls,
-                                                snlist=self._classes[cname]['sn'])
+                                                snlist=snlist)
 
         return ret
 
@@ -968,7 +1168,6 @@ class ManagedPool(ProductPool, DictHk):
         return OrderedDict(
             poolname=getattr(self, '_poolname', 'unknown'),
             poolurl=getattr(self, '_poolurl', 'unknown'),
-            _classes=self._classes,
-            _urns=self._urns,
-            _tags=self._tags,
+            _dTypes=self._dTypes,
+            _dTags=self._dTags,
         )
