@@ -5,7 +5,6 @@ import getpass
 import os
 from os.path import expanduser, expandvars
 
-# logging level for server or possibly by client
 pnsconfig = dict()
 
 ###########################################
@@ -22,19 +21,37 @@ EXTHOST = '172.17.0.1'
 EXTPORT = 9876
 EXTRO_USER = ''
 EXTRO_PASS = ''
+SELF_HOST = '172.17.0.2'
+SELF_PORT = 9876
+SELF_USER = 'fdi'
+SELF_PASS = ''
 MQUSER = ''
 MQPASS = ''
 MQHOST = '172.17.0.1'
 MQPORT = 9876
 
-BASE_POOLPATH = '/tmp'
+BASE_LOCAL_POOLPATH = '/tmp'
 SERVER_POOLPATH = '/tmp/data'
 
 SCHEME = 'http'
-API_VERSION = 'v0.11'
+API_VERSION = 'v0.14'
 API_BASE = '/fdi'
 
-LOGGING_LEVEL = logging.INFO
+pnsconfig['server_scheme'] = 'server'
+
+pnsconfig['cloud_token'] = '/tmp/.cloud_token'
+pnsconfig['cloud_username'] = 'mh'
+pnsconfig['cloud_password'] = 'G44G898g'
+pnsconfig['cloud_host'] = '123.56.102.90'
+pnsconfig['cloud_port'] = 31702
+
+pnsconfig['cloud_scheme'] = 'csdb'
+pnsconfig['cloud_api_version'] = 'v1'
+pnsconfig['cloud_api_base'] = '/csdb'
+pnsconfig['cloud_baseurl'] = pnsconfig['cloud_api_base'] + \
+    '/' + pnsconfig['cloud_api_version']
+
+LOGGER_LEVEL = logging.INFO
 
 # base url for webserver. Update version if needed.
 pnsconfig['scheme'] = SCHEME
@@ -54,78 +71,61 @@ pnsconfig['lookup'] = poolurl_of
 # this base pool path will be added at the beginning of your pool urn when you init a pool like:
 # pstore = PoolManager.getPool('/demopool_user'), it will create a pool at /data.demopool_user/
 # User can disable  basepoolpath by: pstore = PoolManager.getPool('/demopool_user', use_default_poolpath=False)
-pnsconfig['base_poolpath'] = BASE_POOLPATH
+pnsconfig['base_local_poolpath'] = BASE_LOCAL_POOLPATH
 pnsconfig['server_poolpath'] = SERVER_POOLPATH  # For server
 pnsconfig['defaultpool'] = 'default'
-pnsconfig['logginglevel'] = LOGGING_LEVEL
-
-# message queue config
-pnsconfig['mqtt'] = dict(
-    host='x.x.x.x',
-    port=31876,
-    username='foo',
-    passwd='bar',
-)
+pnsconfig['loggerlevel'] = LOGGER_LEVEL
 
 # choose from pre-defined.
-conf = ['dev', 'server_test', 'external', 'production', 'public'][1]
+conf = ['dev', 'external', 'production', 'public'][0]
 
 # modify
 if conf == 'dev':
     # username, passwd, flask ip, flask port
     pnsconfig['node'] = {'username': 'foo', 'password': 'bar',
-                         'host': '127.0.0.1', 'port': 5000,
-                         'ro_username': 'poolro', 'ro_password': '',
+                         'host': '0.0.0.0', 'port': 9885,
+                         'ro_username': 'poolrw', 'rw_password': 'k/p=0',
+                         'ro_username': 'poolro', 'ro_password': 'only5%',
                          }
 
     # server permission user
-    pnsconfig['serveruser'] = 'mh'
-    pnsconfig['base_poolpath'] = '/tmp'
+    pnsconfig['base_local_poolpath'] = '/tmp'
     pnsconfig['server_poolpath'] = '/tmp/data'  # For server
+    # server's own
+    pnsconfig['self_host'] = pnsconfig['node']['host']
+    pnsconfig['self_port'] = pnsconfig['node']['port']
+    pnsconfig['self_username'] = pnsconfig['node']['username']
+    pnsconfig['self_password'] = pnsconfig['node']['password']
     # PTS app permission user
     pnsconfig['ptsuser'] = 'mh'
     # on pns server
     home = '/home/' + pnsconfig['ptsuser']
-
-elif conf == 'server_test':
-    pnsconfig['node'] = {'username': 'foo', 'password': 'bar',
-                         'host': '127.0.0.1', 'port': 9881,
-                         'ro_username': 'ro', 'ro_password': '',
-                         }
-    # server permission user
-    pnsconfig['serveruser'] = 'apache'
-    # PTS app permission user
-    pnsconfig['ptsuser'] = 'pns'
-    # on pns server
-    home = '/home/' + pnsconfig['ptsuser']
 elif conf == 'external':
     # wsgi behind apach2. cannot use env vars
     pnsconfig['node'] = {'username': EXTUSER, 'password': EXTPASS,
                          'host': EXTHOST, 'port': EXTPORT,
                          'ro_username': EXTRO_USER, 'ro_password': EXTRO_PASS,
                          }
-    # message queue config
-    pnsconfig['mqtt'] = dict(
-        host=MQHOST,
-        port=MQPORT,
-        username=MQUSER,
-        passwd=MQPASS,
-    )
-    # server permission user
-    pnsconfig['serveruser'] = 'apache'
-    pnsconfig['base_poolpath'] = BASE_POOLPATH
     pnsconfig['server_poolpath'] = SERVER_POOLPATH  # For server
+    # server's own
+    pnsconfig['self_host'] = SELF_HOST
+    pnsconfig['self_port'] = SELF_PORT
+    pnsconfig['self_username'] = SELF_USER
+    pnsconfig['self_password'] = SELF_PASS
     # PTS app permission user
     pnsconfig['ptsuser'] = 'pns'
     # on pns server
     home = '/home/' + pnsconfig['ptsuser']
 elif conf == 'production':
-    pnsconfig['node'] = {'username': 'foo', 'password': 'bar',
+    pnsconfig['node'] = {'username': 'fdi', 'password': 'bar',
                          'host': '10.0.10.114', 'port': 9885,
                          'ro_username': 'ro', 'ro_password': '',
                          }
-    # server permission user
-    pnsconfig['serveruser'] = 'apache'
+    # server's own
+    pnsconfig['self_host'] = pnsconfig['node']['host']
+    pnsconfig['self_port'] = pnsconfig['node']['port']
+    pnsconfig['self_username'] = pnsconfig['node']['username']
+    pnsconfig['self_password'] = pnsconfig['node']['password']
     # PTS app permission user
     pnsconfig['ptsuser'] = 'pns'
     # on pns server
@@ -135,8 +135,11 @@ elif conf == 'public':
                          'host': '123.56.102.90', 'port': 31702,
                          'ro_username': 'ro', 'ro_password': 'only5%',
                          }
-    # server permission user
-    pnsconfig['serveruser'] = 'apache'
+    # server's own
+    pnsconfig['self_host'] = pnsconfig['node']['host']
+    pnsconfig['self_port'] = pnsconfig['node']['port']
+    pnsconfig['self_username'] = pnsconfig['node']['username']
+    pnsconfig['self_password'] = pnsconfig['node']['password']
     # PTS app permission user
     pnsconfig['ptsuser'] = 'pns'
     # on pns server
@@ -148,7 +151,6 @@ else:
 # See document in :class:`Classes`
 pnsconfig['userclasses'] = ''
 
-
 ########### PNS-specific setup ############
 
 phome = join(home, 'pns')
@@ -173,3 +175,20 @@ del phome, h
 
 # seconds
 pnsconfig['timeout'] = 10
+
+############## project specifig ####################
+# message queue config
+pnsconfig['mqtt'] = dict(
+    host='0.0.0.0',
+    port=31876,
+    username='foo',
+    passwd='bar',
+)
+
+# OSS config
+pnsconfig['oss'] = dict(
+    access_key_id=os.getenv('OSS_ACCESS_KEY_ID'),
+    access_key_secret=os.getenv('OSS_ACCESS_KEY_SECRET'),
+    bucket_name=os.getenv('OSS_BUCKET'),
+    endpoint=os.getenv('OSS_ENDPOINT'),
+)
