@@ -6,10 +6,10 @@ import logging
 import sys
 from requests.auth import HTTPBasicAuth
 
-from fdi.dataset.serializable import serialize
-from fdi.dataset.deserialize import deserialize
-from fdi.pal.urn import parseUrn, parse_poolurl
-from fdi.utils.getconfig import getConfig
+from ..dataset.serializable import serialize
+from ..dataset.deserialize import deserialize
+from ..pal.urn import parseUrn, parse_poolurl
+from ..utils.getconfig import getConfig
 from ..pal.webapi import WebAPI
 
 if sys.version_info[0] >= 3:  # + 0.1 * sys.version_info[1] >= 3.3:
@@ -26,19 +26,21 @@ else:
 logger = logging.getLogger(__name__)
 #logger.debug('level %d' % (logger.getEffectiveLevel()))
 
+POST_PRODUCT_TAG_NAME = 'FDI-Product-Tags'
 
 pcc = getConfig()
 defaulturl = 'http://' + pcc['node']['host'] + \
     ':' + str(pcc['node']['port']) + pcc['baseurl']
-AUTHUSER = pcc['node']['username']
-AUTHPASS = pcc['node']['password']
+
+pccnode = pcc['node']
 
 
 @functools.lru_cache(maxsize=16)
-def getAuth(user=AUTHUSER, password=AUTHPASS):
+def getAuth(user, password):
     return HTTPBasicAuth(user, password)
 
 
+@functools.lru_cache(maxsize=64)
 def urn2fdiurl(urn, poolurl, contents='product', method='GET'):
     """ Returns URL for accessing pools with a URN.
 
@@ -75,7 +77,7 @@ def urn2fdiurl(urn, poolurl, contents='product', method='GET'):
     """
 
     poolname, resourcecn, index = parseUrn(
-        urn) if len(urn) > 7 else ('', '', '0')
+        urn) if urn and (len(urn) > 7) else ('', '', '0')
     indexs = str(index)
     poolpath, scheme, place, pn, un, pw = parse_poolurl(
         poolurl, poolhint=poolname)
@@ -101,7 +103,7 @@ def urn2fdiurl(urn, poolurl, contents='product', method='GET'):
             ret = poolurl + '/hk/'
         elif contents in ['classes', 'urns', 'tags']:
             ret = poolurl + '/hk/' + contents
-        elif contents.split('__')[0] in WebAPI:
+        elif contents.split('__', 1)[0] in WebAPI:
             # append a '/' for flask
             ret = poolurl + '/api/' + contents + '/'
         else:
@@ -110,6 +112,9 @@ def urn2fdiurl(urn, poolurl, contents='product', method='GET'):
     elif method == 'POST':
         if contents == 'product':
             ret = baseurl + poolname + '/'
+        elif contents.split('__', 1)[0] in WebAPI:
+            # append a '/' for flask
+            ret = poolurl + '/api/' + contents.split('__', 1)[0] + '/'
         else:
             raise ValueError(
                 'No such method and contents composition: ' + method + ' / ' + contents)
@@ -142,65 +147,118 @@ def urn2fdiurl(urn, poolurl, contents='product', method='GET'):
 # Store tag in headers, maybe that's  not a good idea
 
 
-def save_to_server(data, urn, poolurl, tag, no_serial=False):
+def post_to_server(data, urn, poolurl, contents='product', headers=None,
+                   no_serial=False, result_only=False, auth=None):
+    """Post data to server with  tag in headers
+
+    data: goes to the request body
+    urn: to extract poolname, product type, and index if any of these are needed
+    poolurl: the only parameter must be provided
+    contents: type of request. Default 'api'.
+    headers: request header dictionary. Default None.
+    no_serial: do not serialize the data.
+    result_only: only return requests result, no code and msg. Default False.
+
+    """
+    if auth is None:
+        auth = getAuth(pccnode['username'], pccnode['password'])
+    api = urn2fdiurl(urn, poolurl, contents=contents, method='POST')
+    # print('POST API: ' + api)
+    if headers is None:
+        headers = {}
+    sd = data if no_serial else serialize(data)
+    res = requests.post(api, auth=auth, data=sd, headers=headers)
+    # print(res)
+    if result_only:
+        return res
+    result = deserialize(res.text)
+    if issubclass(result.__class__, dict):
+        return res.status_code, result['result'], result['msg']
+    else:
+        return res.status_code, 'FAILED', result
+
+
+def save_to_server(data, urn, poolurl, tag, no_serial=False, auth=None):
     """Save product to server with putting tag in headers
 
     data: goes to the request body
     urn: to extract poolname, product type, and index if any of these are needed
     poolurl: the only parameter must be provided
-    tag: go with the produc
-ts into the pool
+    tag: go with the products into the pool
     no_serial: do not serialize the data.
     """
-    auth = getAuth()
-    api = urn2fdiurl(urn, poolurl, contents='product', method='POST')
-    # print('POST API: ' + api)
-    headers = {'tags': tag}
-    sd = data if no_serial else serialize(data)
-    res = requests.post(
-        api, auth=auth, data=sd, headers=headers)
-    # print(res)
+    headers = {POST_PRODUCT_TAG_NAME: serialize(tag)}
+    res = post_to_server(data, urn, poolurl, contents='product',
+                         headers=headers, no_serial=no_serial, result_only=True, auth=auth)
     return res
+    # auth = getAuth(pccnode['username'], pccnode['password'])
+    # api = urn2fdiurl(urn, poolurl, contents='product', method='POST')
+    # # print('POST API: ' + api)
+    # headers = {'tags': tag}
+    # sd = data if no_serial else serialize(data)
+    # res = requests.post(
+    #     api, auth=auth, data=sd, headers=headers)
+    # # print(res)
+    # return res
 
 
-def read_from_server(urn, poolurl, contents='product'):
+def read_from_server(urn, poolurl, contents='product', auth=None):
     """Read product or hk data from server
 
     urn: to extract poolname, product type, and index if any of these are needed
     poolurl: the only parameter must be provided
     """
-    auth = getAuth()
+    if auth is None:
+        auth = getAuth(pccnode['username'], pccnode['password'])
     api = urn2fdiurl(urn, poolurl, contents=contents)
     # print("GET REQUEST API: " + api)
     res = requests.get(api, auth=auth)
     result = deserialize(res.text)
-    code = result['code'] if 'code' in result else 200
-    return code, result['result'], result['msg']
+    if issubclass(result.__class__, dict):
+        return res.status_code, result['result'], result['msg']
+    else:
+        return res.status_code, 'FAILED', result
 
 
-def put_on_server(urn, poolurl, contents='pool'):
+def put_on_server(urn, poolurl, contents='pool', auth=None):
     """Register the pool on the server.
 
     urn: to extract poolname, product type, and index if any of these are needed
     poolurl: the only parameter must be provided
     """
-    auth = getAuth()
+    if auth is None:
+        auth = getAuth(pccnode['username'], pccnode['password'])
     api = urn2fdiurl(urn, poolurl, contents=contents, method='PUT')
     # print("DELETE REQUEST API: " + api)
     res = requests.put(api, auth=auth)
     result = deserialize(res.text)
-    return result['result'], result['msg']
+    if issubclass(result.__class__, dict):
+        return result['result'], result['msg']
+    else:
+        return 'FAILED', result
 
 
-def delete_from_server(urn, poolurl, contents='product'):
+def delete_from_server(urn, poolurl, contents='product', auth=None):
     """Remove a product or pool from server
 
     urn: to extract poolname, product type, and index if any of these are needed
     poolurl: the only parameter must be provided
     """
-    auth = getAuth()
+    if auth is None:
+        auth = getAuth(pccnode['username'], pccnode['password'])
     api = urn2fdiurl(urn, poolurl, contents=contents, method='DELETE')
     # print("DELETE REQUEST API: " + api)
     res = requests.delete(api, auth=auth)
     result = deserialize(res.text)
-    return result['result'], result['msg']
+    if issubclass(result.__class__, dict):
+        return result['result'], result['msg']
+    else:
+        return 'FAILED', result
+
+
+def getCacheInfo():
+    info = {}
+    for i in ['getAuth', 'urn2fdiurl']:
+        info[i] = i.cache_info()
+
+    return info
