@@ -6,9 +6,13 @@ from .. import dataset
 
 import hashlib
 import array
+import datetime
 import traceback
 import pprint
+import textwrap
 import copy
+import os
+from pathlib import Path
 import pwd
 import logging
 from functools import lru_cache
@@ -45,27 +49,29 @@ def trbk2(e):
 
 
 def bstr(x, length=0, tostr=True, quote="'", level=0,
-         tablefmt='rst', tablefmt1='simple', tablefmt2='rst',
-         width=0, heavy=True, yaml=False,
+         width=0, heavy=True, yaml=False, html=False,
          **kwds):
     """ returns the best string representation.
     if the object is a string, return single-quoted; if has toString(), use it; else returns str(). Length limited by lls(lls)
     """
 
-    s = issubclass(x.__class__, str) if PY3 else issubclass(
+    is_str = issubclass(x.__class__, str) if PY3 else issubclass(
         x.__class__, (str, unicode))
 
-    if s:
+    if is_str:
+        # is a string (or unicode if not python3)
         r = quote + x + quote
     elif tostr and hasattr(x, 'toString') and not issubclass(x.__class__, type):
+        # has toString()
         r = x.toString(level=level,
-                       tablefmt=tablefmt, tablefmt1=tablefmt1,
-                       tablefmt2=tablefmt2, width=width, heavy=heavy,
+                       width=width, heavy=heavy,
                        **kwds)
     elif issubclass(x.__class__, (bytes, bytearray, memoryview)):
         r = x.hex()
     else:
         r = ydump(x) if yaml else str(x)
+        if html:
+            r = '<pre>%s</pre>' % r
     return lls(r, length=length)
 
 
@@ -84,19 +90,28 @@ def lls(s, length=80):
 
 """ https://stackoverflow.com/a/2718268
 LHan = [[0x2E80, 0x2E99],    # Han # So  [26] CJK RADICAL REPEAT, CJK RADICAL RAP
-        [0x2E9B, 0x2EF3],    # Han # So  [89] CJK RADICAL CHOKE, CJK RADICAL C-SIMPLIFIED TURTLE
+        # Han # So  [89] CJK RADICAL CHOKE, CJK RADICAL C-SIMPLIFIED TURTLE
+        [0x2E9B, 0x2EF3],
         [0x2F00, 0x2FD5],    # Han # So [214] KANGXI RADICAL ONE, KANGXI RADICAL FLUTE
         0x3005,              # Han # Lm       IDEOGRAPHIC ITERATION MARK
         0x3007,              # Han # Nl       IDEOGRAPHIC NUMBER ZERO
-        [0x3021, 0x3029],    # Han # Nl   [9] HANGZHOU NUMERAL ONE, HANGZHOU NUMERAL NINE
-        [0x3038, 0x303A],    # Han # Nl   [3] HANGZHOU NUMERAL TEN, HANGZHOU NUMERAL THIRTY
+        # Han # Nl   [9] HANGZHOU NUMERAL ONE, HANGZHOU NUMERAL NINE
+        [0x3021, 0x3029],
+        # Han # Nl   [3] HANGZHOU NUMERAL TEN, HANGZHOU NUMERAL THIRTY
+        [0x3038, 0x303A],
         0x303B,              # Han # Lm       VERTICAL IDEOGRAPHIC ITERATION MARK
-        [0x3400, 0x4DB5],    # Han # Lo [6582] CJK UNIFIED IDEOGRAPH-3400, CJK UNIFIED IDEOGRAPH-4DB5
-        [0x4E00, 0x9FC3],    # Han # Lo [20932] CJK UNIFIED IDEOGRAPH-4E00, CJK UNIFIED IDEOGRAPH-9FC3
-        [0xF900, 0xFA2D],    # Han # Lo [302] CJK COMPATIBILITY IDEOGRAPH-F900, CJK COMPATIBILITY IDEOGRAPH-FA2D
-        [0xFA30, 0xFA6A],    # Han # Lo  [59] CJK COMPATIBILITY IDEOGRAPH-FA30, CJK COMPATIBILITY IDEOGRAPH-FA6A
-        [0xFA70, 0xFAD9],    # Han # Lo [106] CJK COMPATIBILITY IDEOGRAPH-FA70, CJK COMPATIBILITY IDEOGRAPH-FAD9
-        [0x20000, 0x2A6D6],  # Han # Lo [42711] CJK UNIFIED IDEOGRAPH-20000, CJK UNIFIED IDEOGRAPH-2A6D6
+        # Han # Lo [6582] CJK UNIFIED IDEOGRAPH-3400, CJK UNIFIED IDEOGRAPH-4DB5
+        [0x3400, 0x4DB5],
+        # Han # Lo [20932] CJK UNIFIED IDEOGRAPH-4E00, CJK UNIFIED IDEOGRAPH-9FC3
+        [0x4E00, 0x9FC3],
+        # Han # Lo [302] CJK COMPATIBILITY IDEOGRAPH-F900, CJK COMPATIBILITY IDEOGRAPH-FA2D
+        [0xF900, 0xFA2D],
+        # Han # Lo  [59] CJK COMPATIBILITY IDEOGRAPH-FA30, CJK COMPATIBILITY IDEOGRAPH-FA6A
+        [0xFA30, 0xFA6A],
+        # Han # Lo [106] CJK COMPATIBILITY IDEOGRAPH-FA70, CJK COMPATIBILITY IDEOGRAPH-FAD9
+        [0xFA70, 0xFAD9],
+        # Han # Lo [42711] CJK UNIFIED IDEOGRAPH-20000, CJK UNIFIED IDEOGRAPH-2A6D6
+        [0x20000, 0x2A6D6],
         [0x2F800, 0x2FA1D]]  # Han # Lo [542] CJK COMPATIBILITY IDEOGRAPH-2F800, CJK COMPATIBILITY IDEOGRAPH-2FA1D
 """
 
@@ -108,18 +123,7 @@ def wcw(char):
     return wcwidth.wcwidth(char)
 
 
-def wls(st, width=15, fill=None, unprintable='#'):
-    """ generates a string comtaining width-limited strings separated with '\n'.
-
-    Identifies Line-breaks with `str.splitlines` https://docs.python.org/3.6/library/stdtypes.html#str.splitlines
-    Removes trailing line-breaks.
-
-    :st: input string.
-    :width: if > 0  returns the str with '\n' inserted every width chars. Or else return the input ``st``. Default is 15. A CJK characters occupies 2 in widths.
-    :unprintable: substitute unprintable characters with is. default is '#'.
-    """
-    if width <= 0 or len(st) == 0:
-        return st
+def wcw_wls(st, width=15, fill=None, linebreak='\n', unprintable='#'):
     line = []
     for s in st.splitlines():
         lens = len(s)
@@ -138,23 +142,23 @@ def wls(st, width=15, fill=None, unprintable='#'):
                 l += w
             else:
                 l += w
-            #print(i, c, l, lasti, s)
+            # print(i, c, l, lasti, s)
             if l == width:
                 line.append(c)
-                line.append('\n')
+                line.append(linebreak)
                 lasti, l = i+1, 0
             elif l > width:
                 if width < 2:
                     # print wide characters even they are too wide for width==1
                     line.append(c)
-                    line.append('\n')
+                    line.append(linebreak)
                     lasti = i+1
                     l = 0
                 else:
                     # set line pointer to this char
                     if fill:
                         line.append((width-l0) * fill)
-                    line.append('\n')
+                    line.append(linebreak)
                     line.append(c)
                     lasti = i
                     l = w
@@ -163,14 +167,40 @@ def wls(st, width=15, fill=None, unprintable='#'):
         if len(line) == 0 or line[-1] != '\n':
             if fill:
                 line.append((width-l) * fill)
-            line.append('\n')
-        # print(line)
-    return ''.join(line[:-1])
+            line.append(linebreak)
+        # print('*****', line)
+    end = len(linebreak)
+    return ''.join(line[:-end])
 
 
-def mstr(obj, level=0, width=1, excpt=None, indent=4, depth=0,
-         tablefmt='rst', tablefmt1='simple', tablefmt2='rst',
-         **kwds):
+def wls(st, width=15, fill=None, linebreak='\n', unprintable='#'):
+    """ generates a string comtaining width-limited strings separated with '\n'.
+
+    Identifies Line-breaks with `str.splitlines` https://docs.python.org/3.6/library/stdtypes.html#str.splitlines
+    Removes trailing line-breaks.
+
+    :st: input string. If not a string, ```str(st)``` is used.
+    :width: if > 0  returns the str with `linebreak` inserted every width chars max. Or else return the input ``st``. Default width is 15. A CJK characters occupies 2 in widths.
+    :linebreak: line-break character. default `\n`
+    :unprintable: substitute unprintable characters with this, only active if wide or unprintable characters are found. default is '#'.
+    """
+    if not issubclass(st.__class__, str):
+        st = str(st)
+    if width <= 0 or len(st) == 0:
+        return st
+    if len(st.encode('utf8')) == len(st) and fill is None:
+        lines = []
+        [lines.extend(textwrap.wrap(s, width=width, replace_whitespace=False,
+                                    drop_whitespace=False))
+         for s in st.splitlines()]
+        return linebreak.join(lines)
+    else:
+        # string has CJK characters
+        return wcw_wls(st, width=width, fill=fill,
+                       linebreak=linebreak, unprintable=unprintable)
+
+
+def mstr(obj, level=0, width=1, excpt=None, indent=4, depth=0, **kwds):
     """ Makes a presentation string at a detail level.
 
     'tablefmt' is needed to be passed in recursive calls under some conditions it is used.
@@ -184,19 +214,12 @@ def mstr(obj, level=0, width=1, excpt=None, indent=4, depth=0,
         if not hasattr(obj, 'items'):
             return bstr(obj, level=level, **kwds)
         if issubclass(obj.__class__, dataset.metadata.MetaData):
-            return obj.toString(level=level,
-                                tablefmt=tablefmt, tablefmt1=tablefmt1,
-                                tablefmt2=tablefmt2,
-                                **kwds)
+            return obj.toString(level=level, **kwds)
         s = ['%s= {%s}' % (mstr(k, level=level, excpt=excp,
                                 indent=indent, depth=depth+1, quote='',
-                                tablefmt=tablefmt, tablefmt1=tablefmt1,
-                                tablefmt2=tablefmt2,
                                 **kwds),
                            mstr(v, level=level, excpt=excp,
                                 indent=indent, depth=depth+1,
-                                tablefmt=tablefmt, tablefmt1=tablefmt1,
-                                tablefmt2=tablefmt2,
                                 **kwds))
              for k, v in obj.items() if k not in excp]
         if len(''.join(s)) < 70:
@@ -218,10 +241,8 @@ def mstr(obj, level=0, width=1, excpt=None, indent=4, depth=0,
             data = obj
 
         s = [pat % (mstr(k, level=level, excpt=excp,
-                         tablefmt=tablefmt, tablefmt1=tablefmt1, tablefmt2=tablefmt2,
                          indent=indent, depth=depth+1, quote='', **kwds),
                     mstr(v, level=level, excpt=excp,
-                         tablefmt=tablefmt, tablefmt1=tablefmt1, tablefmt2=tablefmt2,
                          indent=indent, depth=depth+1, **kwds))
              for k, v in data.items() if k not in excp]
         sep = ',\n' if depth == 0 else ', '
@@ -229,10 +250,8 @@ def mstr(obj, level=0, width=1, excpt=None, indent=4, depth=0,
     else:
         if not hasattr(obj, 'items'):
             return mstr(obj, level=1,
-                        tablefmt=tablefmt, tablefmt1=tablefmt1, tablefmt2=tablefmt2,
                         **kwds)
         s = ['%s' % (mstr(k, level=level, excpt=excp, quote='',
-                          tablefmt=tablefmt, tablefmt1=tablefmt1, tablefmt2=tablefmt2,
                           **kwds))
              for k, v in obj.items() if k not in excp]
         return ', '.join(s)
@@ -454,14 +473,14 @@ def attrstr1(p, v, missingval='', ftime=False, state=True, width=1, **kwds):
 def exprstrs(param, v='_value', extra=False, **kwds):
     """ Generates a set of strings for param.toString().
 
-    :param: Parameeter or xDstaset.
+    :param: Parameter or xDstaset.
     :extra: Whether to include less often used attributes suc as ```fits_keyword```.
     """
     if issubclass(param.__class__, dataset.metadata.Parameter):
         extra_attrs = copy.copy(param._all_attrs)
     elif issubclass(param.__class__, (dataset.arraydataset.ArrayDataset,
                                       dataset.tabledataset.TableDataset,
-                                      dataset.unstructureddataset.UnstrcturedDataset)):
+                                      dataset.unstructureddataset.UnstructuredDataset)):
         extra_attrs = dict((n, v['default'])
                            for n, v in param.zInfo['metadata'].items())
     else:
@@ -636,46 +655,42 @@ def findShape(data, element_seq=(str)):
     return tuple(shape)
 
 
-def guess_value(input_string, parameter=False, last=str):
-    """ Returns guessed value from a string.
-
-    | input | output |
-    | ```'None'``` | `None` |
-    | integer | `int()` |
-    | float | `float()` |
-    | ```'True'```, ```'False```` | `True`, `False` |
-    | string starting with ```'0x'``` | `hex()` |
-    | else | run `last`(input_string) |
+def find_all_files(datadir, verbose=False, include=None, exclude=None, not_if=None):
+    """ returns a list of names of all files in `datadir`.
 
+    :name: of starting directory
+    :include: only if a file name has any of these sub-strings.
+    :exclude: only if a file name has not any of these sub-strings. Empty strings are removed.
+    :not_if: a function that returns true if given a name of unwanted file. default is None, ```os.is_dir```, which excludes directories.
     """
-    from ..dataset.numericparameter import NumericParameter, BooleanParameter
-    from ..dataset.dateparameter import DateParameter
-    from ..dataset.stringparameter import StringParameter
-    from ..dataset.metadata import Parameter
-    if input_string == 'None':
-        res = None
-    elif input_string == '':
-        if parameter:
-            return StringParameter(value=input_string)
-        else:
-            return input_string
-    else:
-        try:
-            res = int(input_string)
-            return NumericParameter(value=res) if parameter else res
-        except ValueError:
-            try:
-                res = float(input_string)
-                return NumericParameter(value=res) if parameter else res
-            except ValueError:
-                # string, bytes, bool
-                if input_string.startswith('0x'):
-                    res = bytes.fromhex(input_string[2:])
-                    return NumericParameter(value=res) if parameter else res
-                elif input_string in ['True', 'False']:
-                    res = bool(input_string)
-                    return BooleanParameter(value=res) if parameter else res
-                else:
-                    res = last(input_string)
-                    return Parameter(value=res) if parameter else res
-    return None
+
+    allf = []
+    if not_if is None:
+        not_if = os.path.isdir
+    # def ok(f, inc, exc, verbose=False):
+    #     if inc and all(i not in f for i in inc):
+    #         return False
+    #     if exc and any(e in f for e in exc):
+    #         return False
+    #     return True
+    if not include:
+        include = '*'
+    inc = Path(datadir).glob(include)
+    #li = list(inc)
+    #print("find", len(inc))
+
+    if exclude is None:
+        exclude = []
+    allf = list(str(f) for f in inc if not any(
+        e in f.name for e in exclude if e != '') and not (not_if(f)))
+
+    # for root, dirs, files in os.walk(datadir):
+    #     if verbose:
+    #         print("In ", root, "...", end=" ")
+    #         print("find", len(files), "non-dir files", end=' ')
+    #         print("and", len(dirs), "dirs")
+    #     allf += [os.path.join(root, f)
+    #              for f in files if ok(f, include, exclude)]
+    if verbose:
+        print('Find %d files total.' % len(allf))
+    return allf
