@@ -6,6 +6,7 @@ from fdi.pns.jsonio import getJsonObj, postJsonObj, putJsonObj, commonheaders
 from fdi.utils.common import lls
 from fdi.pns.jsonio import auth_headers
 from fdi.httppool.model.user import User
+from fdi.pal.publicclientpool import PublicClientPool
 
 import pytest
 import importlib
@@ -46,7 +47,7 @@ def pc(getConfig):
     """ get configuration.
 
     """
-    return getConfig()
+    return getConfig(force=True)
 
 
 def checkserver(aburl, excluded=None):
@@ -76,7 +77,7 @@ def checkserver(aburl, excluded=None):
         server_type = 'live'
     return server_type
 
-    # assert 'result' is not None, 'please start the server to refresh.'
+    # assert 'measurements' is not None, 'please start the server to refresh.'
     # initialize test data.
 
 
@@ -225,6 +226,7 @@ def client(server_app, mock_app):
     if server_app == None:
         yield requests
     else:
+        logger.info('**** mock_app as client *****')
         with mock_app.test_client() as client:
             with mock_app.app_context():
                 # mock_app.preprocess_request()
@@ -236,3 +238,15 @@ def client(server_app, mock_app):
 def demo_product():
     v = get_demo_product()
     return v, get_related_product()
+
+
+csdb_pool_id = 'test_csdb'
+
+
+@pytest.fixture(scope="module")
+def csdb(pc):
+    url = pc['cloud_scheme'] + ':///' + csdb_pool_id
+    # pc['cloud_host'] + ':' + \
+    # str(pc['cloud_port'])
+    test_pool = PublicClientPool(poolurl=url)
+    return test_pool, url
