@@ -29,7 +29,7 @@ from fdi.dataset.quantifiable import Quantifiable
 from fdi.dataset.listener import EventSender, EventTypes, EventType, EventTypeOf, MetaDataListener, EventListener
 from fdi.dataset.messagequeue import MqttRelayListener, MqttRelaySender
 from fdi.dataset.composite import Composite
-from fdi.dataset.metadata import Parameter, MetaData, make_jsonable
+from fdi.dataset.metadata import Parameter, MetaData, make_jsonable, guess_value
 from fdi.dataset.metadataholder import MetaDataHolder
 from fdi.dataset.numericparameter import NumericParameter, BooleanParameter
 from fdi.dataset.stringparameter import StringParameter
@@ -52,7 +52,7 @@ from fdi.dataset.baseproduct import BaseProduct
 from fdi.dataset.product import Product
 from fdi.dataset.browseproduct import BrowseProduct
 from fdi.dataset.readonlydict import ReadOnlyDict
-from fdi.dataset.unstructureddataset import UnstrcturedDataset
+from fdi.dataset.unstructureddataset import UnstructuredDataset
 from fdi.dataset.testproducts import SP, get_demo_product
 from fdi.utils.checkjson import checkjson
 from fdi.utils.loadfiles import loadMedia
@@ -70,27 +70,28 @@ locale.setlocale(locale.LC_ALL, 'en_US.UTF-8')
 
 Classes.updateMapping()
 
-# make format output in /tmp/output.py
-mk_output = 0
+# make format output in /tmp/outputs.py
+mk_outputs = 0
+output_write = 'tests/outputs.py'
 
-if mk_output:
-    with open('/tmp/output.py', 'wt', encoding='utf-8') as f:
+if mk_outputs:
+    with open(output_write, 'wt', encoding='utf-8') as f:
         f.write('# -*- coding: utf-8 -*-\n')
 
 if __name__ == '__main__' and __package__ is None:
     # run by python3 tests/test_dataset.py
 
-    if not mk_output:
-        from outputs import nds20, nds30, nds2, nds3, out_Dataset, out_ArrayDataset, out_TableDataset, out_CompositeDataset, out_FineTime, out_MetaData
+    if not mk_outputs:
+        from outputs import nds2, nds3, out_Dataset, out_ArrayDataset, out_TableDataset, out_CompositeDataset, out_FineTime, out_MetaData
 else:
     # run by pytest
 
     # This is to be able to test w/ or w/o installing the package
     # https://docs.python-guide.org/writing/structure/
-    from pycontext import fdi
-
-    if not mk_output:
-        from outputs import nds20, nds30, nds2, nds3, out_Dataset, out_ArrayDataset, out_TableDataset, out_CompositeDataset, out_FineTime, out_MetaData
+    #from pycontext import fdi
+    import fdi
+    if not mk_outputs:
+        from outputs import nds2, nds3, out_Dataset, out_ArrayDataset, out_TableDataset, out_CompositeDataset, out_FineTime, out_MetaData
 
     import logging
     import logging.config
@@ -143,7 +144,7 @@ def test_deepcmp():
     d2 = {i2: 3982, t2: i1, 6666.2: t2, (765, 3982, 'd', 'p'): l2}
     d3 = {i2: 3982, '4': i1, i3: t1, (765, 3982, 'd'): l1}
     d4 = {i2: 3982, t2: i1, 6666.2: t2, (765, 3982, 'd', 'p'): d1}
-    r = deepcmp(i1, i2)
+    r = deepcmp(i1, i2)  # , verbose=1, brief=0)
     assert r is None
     r = deepcmp(t1, t2)
     assert r is None
@@ -185,7 +186,7 @@ def test_serialization():
     v = Ellipsis
     checkjson(v)
     v = b'\xde\xad\xbe\xef'
-    checkjson(v)
+    # checkjson(v)
     v = array.array('d', [1.2, 42])
     checkjson(v)
     v = [1.2, 'ww']
@@ -289,16 +290,26 @@ def test_ndprint():
     s[0][1][2] = [5, 4, 3, 2, 1]
     s[0][1][3] = [0, 0, 0, 3, 0]
     v = ndprint(s, trans=False, headers=[], tablefmt2='plain')
-    if mk_output:
+    ts = v
+    if mk_outputs:
         print(v)
         # print(nds2)
+        with open(output_write, 'a') as f:
+            clsn = 'nds2'
+            f.write('%s = """%s"""\n' % (clsn, ts))
     else:
-        assert v == nds2
+        assert ts == nds2
     v = ndprint(s, headers=[], tablefmt2='plain')
-    if mk_output:
+    ts = '\nnds3\n'
+    ts += v
+    ts += '\n'
+    if mk_outputs:
         print(v)
+        with open(output_write, 'a') as f:
+            clsn = 'nds3'
+            f.write('%s = """%s"""\n' % (clsn, ts))
     else:
-        assert v == nds3
+        assert ts == nds3
         # pprint.pprint(s)
 
 
@@ -489,19 +500,17 @@ def test_MqttRelay_mqtt(mocksndrlsnr):
     # MQ Relay
     v = MqttRelayListener(topics="test.mq.bounce2", host=None,
                           port=None, username=None, passwd=None,
-                          client_id=None, callback=None, qos=1,
+                          client_id='foo', callback=None, qos=1,
                           userdata=None, clean_session=None,)
-    mqhost = v.mq._h
+    # mqhost = v.mq._host
     mf = MockFileWatcher()
     mf.addListener(v)
     w = MqttRelaySender(topics="test.mq.bounce2", host=None,
                         port=None, username=None, passwd=None,
-                        client_id=None, callback=None, qos=1,
+                        client_id='bar', callback=None, qos=1,
                         userdata=None, clean_session=None,)
-    mquser = w.mq.username
+    # mquser = w.username
     w.addListener(l1)
-    w.last_msg = None
-    test123 = ''
 
     def snd():
         # send source_path
@@ -511,17 +520,20 @@ def test_MqttRelay_mqtt(mocksndrlsnr):
         t0 = time.time()
         while w.last_msg is None and (time.time()-t0 < 3):
             time.sleep(0.2)
-    t1 = threading.Thread(target=snd)
-    t2 = threading.Thread(target=rcv)
-    t1.start()
-    t2.start()
-    t2.join()
-    t1.join()
-    assert not t1.is_alive()
-    assert not t2.is_alive()
-    print(w.last_msg)
-    assert test123 == "['foo'] changed."
-    assert w.last_msg == ['foo']
+    for ii in range(3):
+        w.last_msg = None
+        test123 = ''
+        t1 = threading.Thread(target=snd)
+        t2 = threading.Thread(target=rcv)
+        t1.start()
+        t2.start()
+        t2.join()
+        t1.join()
+        assert not t1.is_alive()
+        assert not t2.is_alive()
+        print(w.last_msg)
+        assert test123 == "['foo'] changed."
+        assert w.last_msg == ['foo']
 
 
 def test_datatypes():
@@ -547,7 +559,7 @@ def test_datatypes():
     # assignment
     v.components = [0xaa, 1, 1e2]
     assert v.components == [0xaa, 1, 1e2]
-    h = hash(v)
+
     checkjson(v)
 
     # Quaternion
@@ -557,7 +569,6 @@ def test_datatypes():
     a1 = -1
     v2 = Quaternion([a1, 1+0, 1-a1+0.3, 4.5])
     assert v == v2
-    h = hash(v)
     checkjson(v)
 
 
@@ -584,7 +595,7 @@ def test_Parameter_init():
 
     # if value is None v.value is set to default
     v = Parameter(description=a1, value=None, typ_=a3, default=a4, valid=a5)
-    assert v.value == v.default
+    assert v.value is None
 
     # with no argument
     v = Parameter()
@@ -819,12 +830,8 @@ def test_Parameter_features():
     b2 = a2 + 0  # make a copy
     v1.value = b2  # change it back
     v1.description = 'changed'
-    if 0:
-        assert not v.equals(v1)
-        assert v != v1
-    else:
-        assert v.equals(v1)
-        assert v == v1
+    assert not v.equals(v1)
+    assert v != v1
 
     # comparison with simplified syntax w/o '.value'
     x = 4
@@ -1048,10 +1055,11 @@ def test_DateParameter():
     assert v.description == 'UNKNOWN'
     assert v.value == v.default
     assert v.type == 'finetime'
-    def0 = FineTime(0)
+    def0 = None
     assert v.default == def0
     assert v.valid is None
-    assert v.typecode == 'Q'
+    assert v.typecode == '%Y-%m-%dT%H:%M:%S.%f'
+    v = DateParameter(789)
     assert v.value.format == FineTime.DEFAULT_FORMAT
 
     a1 = 'a test DateParameter'
@@ -1073,11 +1081,25 @@ def test_DateParameter():
     v.value = a8
     assert v.value == FineTime(a8)
 
+    # set DateParameter of a product with value to FineTime
+    p = Product()
+    p.creationDate = 3
+    assert p.meta['creationDate'].value.tai == 3
+    p.meta['creationDate'].typecode = '%Y'
+    p.creationDate = '2022'
+    assert p.meta['creationDate'].value.isoutc(
+    ) == '2022-01-01T00:00:00.000000'
+    setattr(p, 'creationDate', '1989')
+    assert p.meta['creationDate'].value.isoutc(
+    ) == '1989-01-01T00:00:00.000000'
+
     v = DateParameter('2019-02-19T01:02:03.457')
     assert v.value.tai == 1929229360457000
     v.value.format = '%Y'
     # print('********', v.value.isoutc(), ' ** ', v, '*******', v.toString(1))
-
+    assert v.value.isoutc() == '2019-02-19T01:02:03.457000'
+    assert str(
+        v) == 'DateParameter(finetime: 2019\n1929229360457000 <>, "UNKNOWN", default= None, valid= None tcode=%Y-%m-%dT%H:%M:%S.%f)'
     with pytest.raises(TypeError):
         DateParameter(3.3)
     vv = copy.copy(v)
@@ -1090,7 +1112,7 @@ def test_StringParameter():
     assert v.description == 'UNKNOWN'
     assert v.value == v.default
     assert v.type == 'string'
-    assert v.default == ''
+    assert v.default is None
     assert v.valid is None
     assert v.typecode == 'B'
 
@@ -1118,6 +1140,13 @@ def test_StringParameter():
     checkjson(v)
 
 
+def test_guess_value():
+    assert guess_value('') == ''  # StringParameter('')
+    assert guess_value('None', parameter=True) == Parameter(None)
+    assert guess_value('nul', parameter=True) == Parameter(None)
+    assert guess_value(None, parameter=True) == Parameter(None)
+
+
 def test_MetaData():
     # creation
     a1 = 'age'
@@ -1142,6 +1171,9 @@ def test_MetaData():
                                  value=2.3, unit='sec')
     v['birthday'] = Parameter(description='was made on',
                               value=FineTime('2020-09-09T12:34:56.789098'))
+    # only Parameters are taken
+    with pytest.raises(TypeError):
+        v['bomb'] = 'BOMM'
     # names of all parameters
     assert [n for n in v] == [a1, 'time', 'birthday']
 
@@ -1187,8 +1219,10 @@ def test_MetaData():
     v = v.meta
     ts += '\n'
     ts += v.toString(extra=True)
-    if mk_output:
-        with open('/tmp/output.py', 'a', encoding='utf-8') as f:
+    ts += '\ntablefmt = html\n'
+    ts += v.toString(tablefmt1='html')
+    if mk_outputs:
+        with open(output_write, 'a', encoding='utf-8') as f:
             clsn = 'out_MetaData'
             f.write('%s = """%s"""\n' % (clsn, ts))
         print(ts)
@@ -1203,6 +1237,7 @@ def test_MetaData():
                 break
             print(ts[i:])
             print(out_MetaData[i:])
+            assert ts[i:] == out_MetaData[i:]
             assert False
 
     # listeners
@@ -1344,9 +1379,11 @@ def test_Dataset():
     ts += v.toString(1)
     ts += 'level 2,\n'
     ts += v.toString(2)
-    if mk_output:
+    ts += '\ntablefmt = html\n'
+    ts += v.toString(tablefmt1='html')
+    if mk_outputs:
         print(ts)
-        with open('/tmp/output.py', 'a') as f:
+        with open(output_write, 'a') as f:
             clsn = 'out_Dataset'
             f.write('%s = """%s"""\n' % (clsn, ts))
     else:
@@ -1365,7 +1402,7 @@ def do_ArrayDataset_init(atype):
     assert v.data is None
     assert v.unit is None
     assert v.description == 'UNKNOWN'
-    assert v.shape == ()
+    assert v.shape == None
     assert v.typecode == 'UNKNOWN'
     assert v.meta['description'].value == 'UNKNOWN'
     # from DRM
@@ -1375,24 +1412,20 @@ def do_ArrayDataset_init(atype):
     a4 = 'float'              # type
     a6 = 'f'                  # typecode
     a7 = (8, 9)
+
     v = ArrayDataset(data=a1, unit=a2, description=a3,
                      typ_=a4, typecode=a6)
     assert v.data == a1
     assert v.unit == a2
     assert v.description == a3
     assert v.typecode == a6
-    v.updateShape()
-    assert v.shape == (len(a1),)
-    v = ArrayDataset(data=a1)
-    assert v.data == a1
-    assert v.unit is None
-    assert v.description == 'UNKNOWN'
-    assert v.typecode == 'UNKNOWN'
-    assert v.updateShape()
     assert v.shape == (len(a1),)
 
     ashape = [[[1], [2]], [[3], [4]], [[5], [6]]]
-    v2 = ArrayDataset(data=ashape)
+    v2 = ArrayDataset()
+    # setting data also sets shape
+    v2.data = ashape
+    # setting dataa also v.updateShape()
     assert v2.shape == (3, 2, 1)
     v2.pop()
     assert v2.shape == (2, 2, 1)
@@ -1469,12 +1502,16 @@ def do_ArrayDataset_func(atype):
     assert v1 == v
     # not equal
     # change data
-    v1.data += atype([6])
+    if issubclass(v1.data.__class__, memoryview):
+        v1.data[0] += 1
+    else:
+        v1.data += atype([6])
     assert v != v1
     assert v1 != v
     # restore v1 so that v==v1
-    v1.data = copy.deepcopy(a1)
-    assert v == v1
+    if not issubclass(v1.data.__class__, memoryview):
+        v1.data = copy.deepcopy(a1)
+        assert v == v1
     # change description
     v1.description = 'changed'
     assert v != v1
@@ -1563,10 +1600,10 @@ def do_ArrayDataset_func(atype):
 
     ts += x.toString()
     i = ts.index('0  0  0')
-    if mk_output:
+    if mk_outputs:
         print(ts[i:])
     else:
-        assert ts[i:-126] == nds2
+        assert ts[i:-132] == nds2
     ts += '\n\nlevel 1\n'
     ts += x.toString(1)
     ts += '\n\nlevel 2, repr\n'
@@ -1574,17 +1611,16 @@ def do_ArrayDataset_func(atype):
     ts += '\n\n'
     ts += 'an empty meta and long data level 2: \n'
     ts += ArrayDataset(data=[8]*8).toString(level=2)
-    ts += '\n\n'
-    if mk_output:
+    ts += '\ntablefmt = html\n'
+    ts += x.toString(tablefmt1='html')
+    if mk_outputs:
         print(ts)
-        with open('/tmp/output.py', 'a') as f:
+        with open(output_write, 'a') as f:
             clsn = 'out_ArrayDataset'
             f.write('%s = """%s"""\n' % (clsn, ts))
     else:
         assert ts == out_ArrayDataset
 
-    h = hash(v)
-
     checkjson(x, dbg=0)
     checkgeneral(x)
 
@@ -1610,12 +1646,21 @@ def test_ArrayDataset_array_func():
     do_ArrayDataset_func(atype)
 
 
+def XXXtest_ArrayDataset_memoryview_init():
+    def atype(x): return array.array('f', memoryview(x))
+    do_ArrayDataset_init(atype)
+
+
+def XXXtest_ArrayDataset_memoreyvew_func():
+    def atype(x): return array.array('f', memoryview(x))
+    do_ArrayDataset_func(atype)
+
+
 def test_Column():
     v = Column()
     assert v.type == 'Column'
     v = Column(data=[4, 9], unit='m')
     assert v.data == [4, 9]
-    s = v.hash()
 
     checkjson(v)
 
@@ -1636,20 +1681,29 @@ def test_TableDataset_init():
             t = TableDataset(data=[{'name': 'a', 'column': Column(data=[])}])
 
         # setData format 1: data is a  mapping. Needs pytnon 3.6 to guarantee order
-        a1 = {'col1': Column(data=[1, 4.4, 5.4E3], unit='eV'),
-              'col2': Column(data=[0, 43.2, 2E3], unit='cnt')}
+        d1 = [1, 4.4, 5.4E3]
+        d2 = [0, 43.2, 2E3]
+        a1 = {'col1': Column(data=d1, unit='eV'),
+              'col2': Column(data=d2, unit='cnt')}
 
         v = TableDataset(data=a1)  # inherited from DataContaier
         assert v.getColumnCount() == len(a1)
         assert v.getColumnName(0) == 'col1'
         t = a1['col2'].data[1]  # 43.2
         assert v.getValueAt(rowIndex=1, columnIndex=1) == t
+        assert v.shape == (2, 3)
 
     # 2: add columns one by one
     v2 = TableDataset()
-    v2['col1'] = Column(data=[1, 4.4, 5.4E3], unit='eV')
-    v2['col2'] = Column(data=[0, 43.2, 2E3], unit='cnt')
+    v2['col1'] = Column(data=copy.deepcopy(d1), unit='eV')
+    v2['col2'] = Column(data=copy.deepcopy(d2), unit='cnt')
+    assert v2.shape == (2, 3)
 
+    # print('DDD ', deepcmp(v, v2, verbose=0))
+
+    # fdi.dataset.eq.XHASH_VERBOSE = True
+    # print(v.hash())
+    # print(v2.hash())
     assert v2 == v
 
     # 3: another syntax, list of tuples that does not need to use Column
@@ -1901,6 +1955,13 @@ def test_TableDataset_func():
                  'group2.wgt': Column(data=['', 0.32, -9876543210], unit='g'),
                  }.items():
         v.addColumn(n, c)
+    from fdi.utils.tree import tree
+    print('\n'.join(tree(v, style='ascii')))
+    from fdi.utils.jsonpath import jsonPath, flatten_compact
+    print(list(str(x.full_path)
+               for x in jsonPath(v, '$..*', val='context')))
+    print('\n'.join(flatten_compact([v]).keys()))
+
     ts += 'grouped column names'
     ts += v.string(0, 'grid', 'rst', 'simple')
     ts += v.string(0, 'grid', 'rst', 'rst')
@@ -1909,10 +1970,11 @@ def test_TableDataset_func():
     ts += v.string(0, 'grid', 'rst', 'plain')
     ts += v.string(0, 'grid', 'rst', 'orgtbl')
     ts += v.string(0, 'grid', 'rst', 'psql')
+    ts += v.string(0, 'unsafehtml', 'unsafehtml', 'unsafehtml')
     ts += '\n\n'
-    if mk_output:
+    if mk_outputs:
         print(ts)
-        with open('/tmp/output.py', 'a') as f:
+        with open(output_write, 'a') as f:
             clsn = 'out_TableDataset'
             f.write('%s = """%s"""\n' % (clsn, ts))
     else:
@@ -2094,10 +2156,6 @@ def test_CompositeDataset_init():
                            value=2.3, unit='sec')
     v1.meta[b11] = b12
 
-    d = deepcmp(v, v1)
-    q = v.__eq__(v1)
-    h = v.hash()
-
     assert v == v1
     assert v1 == v
 
@@ -2154,12 +2212,14 @@ def test_CompositeDataset_init():
     ts += s3
     ts += 'level 1, repr\n'
     ts += v3.toString(1)
-    ts += 'level 2,\n'
+    ts += '\nlevel 2,\n'
     ts += v3.toString(2)
     assert v3.string() == s3
-    if mk_output:
+    ts += '\nlevel 0, html\n'
+    ts += v3.toString(tablefmt1='html')
+    if mk_outputs:
         print(ts)
-        with open('/tmp/output.py', 'a') as f:
+        with open(output_write, 'a') as f:
             clsn = 'out_CompositeDataset'
             f.write('%s = """%s"""\n' % (clsn, ts))
     else:
@@ -2171,7 +2231,7 @@ def test_CompositeDataset_init():
 
 def test_UnstrcturedDataset():
 
-    u = UnstrcturedDataset({3: 6}, 'foo')
+    u = UnstructuredDataset({3: 6}, 'foo')
     assert u.data == {3: 6}
     assert u.description == 'foo'
     assert u.meta['description'].value == 'foo'
@@ -2180,27 +2240,27 @@ def test_UnstrcturedDataset():
     assert u.doctype == 'json'
 
     p = get_demo_product()
-    u.input(p.serialized(), 'json')
+    u.put(p.serialized(), 'json')
     assert issubclass(u.data.__class__, dict)
 
     v, s = u.fetch(["data", "_ATTR_meta", "speed"])
     # v is a dictionary
     assert json.dumps(v) == serialize(p.meta['speed'])
     assert s == '.data["_ATTR_meta"]["speed"]'
-    # ["results"]["calibration"]["meta"]["unit"]["value"]
-    v, s = u.fetch(["data", "results", "calibration",
+    # ["measurements"]["calibration"]["meta"]["unit"]["value"]
+    v, s = u.fetch(["data", "measurements", "calibration",
                     "_ATTR_meta", "unit", "value"])
     assert v == 'count'
-    assert s == '.data["results"]["calibration"]["_ATTR_meta"]["unit"]["value"]'
+    assert s == '.data["measurements"]["calibration"]["_ATTR_meta"]["unit"]["value"]'
     # data of a column in tabledataset within compositedataset
     v, s = u.fetch(
-        ["data", "results", "Time_Energy_Pos", "Energy", "_ATTR_data"])
+        ["data", "measurements", "Time_Energy_Pos", "Energy", "_ATTR_data"])
     t = [x * 1.0 for x in range(len(v))]
     assert v == [2 * x + 100 for x in t]
-    assert v == u.data["results"]["Time_Energy_Pos"]["Energy"]["_ATTR_data"]
-    assert s == '.data["results"]["Time_Energy_Pos"]["Energy"]["_ATTR_data"]'
+    assert v == u.data["measurements"]["Time_Energy_Pos"]["Energy"]["_ATTR_data"]
+    assert s == '.data["measurements"]["Time_Energy_Pos"]["Energy"]["_ATTR_data"]'
 
-    print('cache:', fdi.dataset.unstructureddataset.getCacheInfo())
+    print('cache:', fdi.utils.jsonpath.getCacheInfo())
     checkjson(u)
     checkgeneral(u)
 
@@ -2259,21 +2319,27 @@ complex_ex = """
 
 
 def test_jsonPath():
-    js = '[{"idAssetType":6,"name":"CCAA","enterprise":{"id":1,"name":"APV"}}]'
-    u = UnstrcturedDataset(data=js, doctype='json')
-    m = u.jsonPath("$[?(@.name == 'CCAA')].idAssetType")
-    assert m[0][1] == 6
 
     # xmltodict docs
-    u = UnstrcturedDataset(data=simple_ex, doctype='xml', attr_prefix='@')
+    u = UnstructuredDataset(data=simple_ex, doctype='xml', attr_prefix='@')
     assert u.data['a']['@prop'] == 'x'
     assert u.data['a']['b'] == ['1', '2']
     u.attr_prefix = '%'
-    u.input(simple_ex)
+    u.put(simple_ex)
     assert u.data['a']['%prop'] == 'x'
 
+    do_jsonPath(UnstructuredDataset)
+
+
+def do_jsonPath(UDSET):
+
+    js = '[{"idAssetType":6,"name":"CCAA","enterprise":{"id":1,"name":"APV"}}]'
+    u = UDSET(data=js, doctype='json')
+    m = u.jsonPath("$[?(@.name == 'CCAA')].idAssetType")
+    assert m[0][1] == 6
+
     ### BOOK STORE ###
-    u = UnstrcturedDataset(data=bookstore, doctype='json')
+    u = UDSET(data=bookstore, doctype='json')
     # print(u.toString())
     # the authors of all books in the store
     n = u.jsonPath("$.store.book[*].author", val='context')
@@ -2395,7 +2461,7 @@ def test_jsonPath():
                  ('store/bicycle/color', 'red'),
                  ('store/bicycle/price', 19.95)
                  ]
-    print('cache:', fdi.dataset.unstructureddataset.getCacheInfo())
+    print('cache:', fdi.utils.jsonpath.getCacheInfo())
 
 
 def test_Indexed():
@@ -2560,9 +2626,9 @@ def demo_CompositeDataset():
 def test_FineTime():
     # default
     v = FineTime()
-    assert v.tai == 0
+    assert v.tai is None
     assert v.format == v.DEFAULT_FORMAT
-    assert v.toDatetime().year == 1958
+    #assert v.toDatetime().year == 1958
     # at Epoch, TAI=0
     v = FineTime(v.EPOCH)
     assert v.tai == 0
@@ -2570,6 +2636,15 @@ def test_FineTime():
     v = FineTime(1)
     assert v.toDatetime().microsecond == 1
     # from string
+    print(FineTime('1970', format='%Y'))
+    print(FineTime(
+        datetime.datetime(1970, 1, 1, 0, 0, 0)))
+    assert FineTime('1970', format='%Y') == FineTime(
+        datetime.datetime(1970, 1, 1, 0, 0, 0))
+    assert FineTime('2000', format='%Y') == FineTime(
+        datetime.datetime(2000, 1, 1, 0, 0, 0))
+    assert FineTime(b'3000', format='%Y') == FineTime(
+        datetime.datetime(3000, 1, 1, 0, 0, 0))
     v = FineTime('1990-09-09T12:34:56.789098 UTC')
 
     # comparison
@@ -2637,9 +2712,9 @@ def test_FineTime():
 def test_FineTime1():
     # default
     v = FineTime1()
-    assert v.tai == 0
+    assert v.tai is None
     assert v.format == v.DEFAULT_FORMAT
-    assert v.toDatetime().year == 2017
+    #assert v.toDatetime().year == 2017
     # at Epoch, TAI=0
     v = FineTime1(v.EPOCH)
     assert v.tai == 0
@@ -2684,9 +2759,9 @@ def test_FineTimes_toString():
                 for level in [0, 1, 2]:
                     s = v.toString(level=level, width=width)
                     ts += f'level={level} width={width}: {s}\n'
-    if mk_output:
+    if mk_outputs:
         print(ts)
-        with open('/tmp/output.py', 'a') as f:
+        with open(output_write, 'a') as f:
             clsn = 'out_FineTime'
             f.write('%s = """%s"""\n' % (clsn, ts))
     else:
@@ -2695,6 +2770,30 @@ def test_FineTimes_toString():
 
 def test_History():
     v = History()
+    with pytest.raises(TypeError):
+        v.add_input(args=[(2, 3)])
+
+    v = History()
+    v.add_input(args=[1, 2.33, 'asd', True, [4, 5], array.array('f', [6, 7])],
+                keywords=dict(a='b', c=[11]),
+                info=dict(c='d')
+                )
+    args, kwds = v.get_args()
+    assert args[-1] == array.array('f', [6, 7])
+    assert kwds.rowCount == 2
+    assert v.meta['c'].value == 'd'
+    assert 'e' not in v.meta
+    # add more
+    v.add_input(args=[{9: 10}],
+                keywords=dict(d=FineTime(123456)),
+                info=dict(e='f')
+                )
+    assert args[-1][9] == 10
+    assert v['kw_args'].rowCount == 3
+    assert v.meta['e'].value == 'f'
+    # print(v.string())
+    # print(kwds.string())
+    # print(v.tree())
     checkjson(v)
     checkgeneral(v)
 
@@ -2980,6 +3079,19 @@ def est_yaml2python():
 #     p = json.loads(js, cls=JSONObjectDecoder)
 #     print(p['h'].b)
 
+def speed(v, v2=None):
+    """ see who was faster? xhash or deepcmp """
+
+    if v2 is None:
+        v2 = p, copy.copy(p)
+    import timeit
+    gl = copy.copy(globals())
+    gl.update(locals())
+
+    print(timeit.timeit('deepcmp(v, v2, verbose=0)', number=100, globals=gl))
+    print(timeit.timeit('v==v2', number=100, globals=gl))
+
+
 def running(t):
     print('running ' + str(t))
     t()
