@@ -1,9 +1,5 @@
 
 from fdi.dataset.arraydataset import ArrayDataset
-import itertools
-import random
-import timeit
-import pytest
 
 from fdi.pal.mempool import MemPool
 from fdi.pal.poolmanager import PoolManager, DEFAULT_MEM_POOL
@@ -29,6 +25,11 @@ from fdi.utils.checkjson import checkjson
 from fdi.pns.fdi_requests import save_to_server, read_from_server, delete_from_server
 
 from requests.auth import HTTPBasicAuth
+from requests.exceptions import ConnectionError
+import itertools
+import random
+import timeit
+import pytest
 import copy
 import traceback
 from pprint import pprint
@@ -52,6 +53,7 @@ else:
 
 
 Test_Pool_Name = __name__.replace('.', '_')
+defaultpoolPath = '/tmp/fditest'
 
 Classes.updateMapping()
 
@@ -131,7 +133,7 @@ def test_UrnUtils():
     assert scheme == a1
     assert place == ''
     assert poolname == b2
-    #assert (un, pw) == ('foo', 'bar')
+    # assert (un, pw) == ('foo', 'bar')
     # implicit pool is the last segment
     poolpath, scheme, place, poolname, un, pw = parse_poolurl(
         'file:///c:/tmp/mypool/v3/')
@@ -157,18 +159,18 @@ def test_UrnUtils():
 
 
 def test_Urn():
-    prd = Product(description='pal test')
+    prd = BaseProduct(description='pal test')
     a1 = 'file'      # scheme
     a2 = '/e:'            # place
     b1, b2 = '/tmp/foo', 'name'
     a3 = b1 + '/' + b2               # /tmp/foo/name
-    a4 = fullname(prd)           # fdi.dataset.Product
+    a4 = fullname(prd)           # fdi.dataset.BaseProduct
     a5 = 43
     s = a1 + '://' + a2          # file:///e:
     poolurl = s + a3                   # file:///e:/tmp/foo/name
-    r = a4 + ':' + str(a5)       # fdi.dataset.Product:43
-    rp = a4 + '_' + str(a5)      # fdi.dataset.Product_43
-    urn = 'urn:' + b2 + ':' + r  # urn:name:fdi.dataset.Product:43
+    r = a4 + ':' + str(a5)       # fdi.dataset.BaseProduct:43
+    rp = a4 + '_' + str(a5)      # fdi.dataset.BaseProduct_43
+    urn = 'urn:' + b2 + ':' + r  # urn:name:fdi.dataset.BaseProduct:43
     urn1 = 'urn:' + b2 + ':' + a4+':'+str(a5-1)
     # constructor
     # urn only
@@ -181,7 +183,6 @@ def test_Urn():
     assert v.getPlace() is None
     assert v.getPoolpath() is None
 
-    h = v.hash()
     # urn with poolurl
     v = Urn(urn=urn, poolurl=poolurl)
     assert v.getPoolId() == b2  #
@@ -234,7 +235,7 @@ def test_Urn():
 
 
 def transpath(direc, poolpath):
-    """ direc must have a leading / if base_poolpath is defined in config """
+    """ direc must have a leading / if base_local_poolpath is defined in config """
 
     return poolpath+'/'+direc
 
@@ -254,12 +255,12 @@ def cleanup(poolurl=None, poolname=None):
     """ remove pool from disk and memory"""
 
     if poolurl or poolname:
-        nu = [(poolname, poolurl)]
+        name_url = [(poolname, poolurl)]
     else:
-        nu = []
+        name_url = []
         for pn, pool in PoolManager.getMap().items():
-            nu.append((pn, pool._poolurl))
-    for pname, purl in nu:
+            name_url.append((pn, pool._poolurl))
+    for pname, purl in name_url:
         direc, schm, place, pn, un, pw = parse_poolurl(purl, pname)
         if schm in ['file', 'server']:
             d = direc + '/' + pn
@@ -269,6 +270,8 @@ def cleanup(poolurl=None, poolname=None):
                 PoolManager.getPool(DEFAULT_MEM_POOL).removeAll()
         elif schm in ['http', 'https']:
             pass
+        elif schm in ['csdb']:
+            pass
         else:
             assert False
         # correct way
@@ -288,7 +291,6 @@ def cleanup(poolurl=None, poolname=None):
 
 def test_PoolManager():
     defaultpoolName = Test_Pool_Name
-    defaultpoolPath = '/tmp/fditest'
     defaultpoolUrl = 'file://' + defaultpoolPath + '/' + defaultpoolName
     cleanup(defaultpoolUrl, defaultpoolName)
     # class methods
@@ -302,14 +304,14 @@ def test_PoolManager():
     PoolManager.removeAll()
     assert PoolManager.size() == 0
     assert weakref.getweakrefcount(pool) == 0
-    #print(weakref.getweakrefs(pool), id(pool), 'mmmm pool')
+    # print(weakref.getweakrefs(pool), id(pool), 'mmmm pool')
     del pool
 
     # initiate
     pm = PoolManager()
     assert len(pm) == 0
     p1 = pm.getPool(defaultpoolName, defaultpoolUrl)
-    #print(weakref.getweakrefs(p1), id(p1), 'mmmm p1')
+    # print(weakref.getweakrefs(p1), id(p1), 'mmmm p1')
     for k, v in pm.items():
         assert isinstance(v, ProductPool)
     assert defaultpoolName in pm
@@ -340,14 +342,14 @@ def test_PoolManager():
     assert pm.remove(defaultpoolName) == 0
 
     # http pool gets registered
-    with pytest.raises(RuntimeError):
+    with pytest.raises(ConnectionError):
         ph = pm.getPool(poolurl='http://h.edu/foo')
     assert not PoolManager.isLoaded('foo')
     assert PoolManager.remove('foo') == 1
 
 
-def checkdbcount(expected_cnt, poolurl, prodname, currentSN, usrpsw, *args):
-    """ count files in pool and entries in class db.
+def checkdbcount(expected_cnt, poolurl, prodname, currentSN, usrpsw, *args, csdb=None, **kwds):
+    """ init_count files in pool and entries in class db.
 
     expected_cnt, currentSN: expected number of prods and currentSN in pool for products named prodname
     """
@@ -357,12 +359,12 @@ def checkdbcount(expected_cnt, poolurl, prodname, currentSN, usrpsw, *args):
         path = op.join(poolpath, poolname)
         assert sum(1 for x in glob.glob(
             op.join(path, prodname + '*[0-9]'))) == expected_cnt
-        cp = op.join(path, 'classes.jsn')
+        cp = op.join(path, 'dTypes.jsn')
         if op.exists(cp) or expected_cnt != 0:
             with open(cp, 'r') as fp:
                 js = fp.read()
-            cread = deserialize(js)
-            if currentSN == -1:
+            cread = deserialize(js, int_key=True)
+            if currentSN is None:
                 assert cread[prodname]['currentSN'] == currentSN
                 # number of items is expected_cnt
             assert len(cread[prodname]['sn']) == expected_cnt
@@ -371,29 +373,82 @@ def checkdbcount(expected_cnt, poolurl, prodname, currentSN, usrpsw, *args):
         if mpool is None or len(mpool) == 0:
             # wiped
             assert expected_cnt == 0
-            assert currentSN == -1
+            assert currentSN is None
             return
         ns = [n for n in mpool if prodname in n]
         assert len(ns) == expected_cnt, len(ns)
-        if currentSN == -1:
-            assert mpool['classes'][prodname]['currentSN'] == currentSN
+        if currentSN is None:
+            assert mpool['dTypes'][prodname]['currentSN'] == currentSN
         # for this class there are  how many prods
-        assert len(mpool['classes'][prodname]['sn']) == expected_cnt
+        assert len(mpool['dTypes'][prodname]['sn']) == expected_cnt
     elif scheme in ['http', 'https']:
         auth = HTTPBasicAuth(*usrpsw)
+        # count
         cpath = poolname + '/' + 'count/' + prodname
         api_baseurl = scheme + '://' + place + poolpath + '/'
         url = api_baseurl + cpath
         x = requests.get(url, auth=auth)
+        assert 'Recorded' in x.text
         count = int(x.json()['result'])
         assert count == expected_cnt
+        # counting files. Append a '/'
+        cpath = poolname + '/' + 'count/' + prodname + '/'
+        api_baseurl = scheme + '://' + place + poolpath + '/'
+        url = api_baseurl + cpath
+        x = requests.get(url, auth=auth)
+        assert 'Counted' in x.text
+        count = int(x.json()['result'])
+        assert count == expected_cnt
+
+        # count all
+        cpath = poolname + '/' + 'count'
+        api_baseurl = scheme + '://' + place + poolpath + '/'
+        url = api_baseurl + cpath
+        x = requests.get(url, auth=auth)
+        assert 'Recorded' in x.text
+        count = int(x.json()['result'])
+        assert count >= expected_cnt
+
+        # count all with couting
+        cpath = poolname + '/' + 'count' + '/'
+        api_baseurl = scheme + '://' + place + poolpath + '/'
+        url = api_baseurl + cpath
+        x = requests.get(url, auth=auth)
+        assert 'Counted' in x.text
+        count1 = int(x.json()['result'])
+        assert count1 >= expected_cnt
+
+        assert count == count1
+
+        # sn
+        if currentSN is None:
+            return
+        spath = poolname + '/' + 'hk/dTypes'
+        url = api_baseurl + spath
+        x = requests.get(url, auth=auth)
+        csn = x.json()['result'][prodname]['currentSN']
+        assert csn == currentSN
+
+    elif scheme in ['csdb']:
+        test_pool = PoolManager.getPool(poolname)
+        #a, b = csdb
+        #assert test_pool == csdb
+        pinfo = test_pool.getPoolInfo()
+        count = test_pool.getCount(prodname)
+        assert count == expected_cnt
+        if currentSN is None:
+            return
+        for cl in pinfo[poolname]['_classes']:
+            if cl['productTypeName'] == prodname:
+                sn = cl['sn']
+                assert cl['currentSn'] == currentSN
+                break
     else:
         assert False, 'bad pool scheme'
 
 
 def test_ProductRef():
     defaultpoolName = Test_Pool_Name
-    defaultpoolPath = '/tmp/fditest'
     defaultpoolUrl = 'file://' + defaultpoolPath + '/' + defaultpoolName
     cleanup(defaultpoolUrl, defaultpoolName)
     prd = Product()
@@ -514,34 +569,82 @@ def test_ProductStorage_init():
         pass  # assert False, 'exception expected'
 
 
+def getCurrSnCount(csdb, prodname):
+    pinfo = csdb if issubclass(csdb.__class__, dict) else \
+        csdb.getPoolInfo()
+    for pool in pinfo.values():
+        for cl in pool['_classes']:
+            if cl['productTypeName'] == prodname:
+                init_sn = cl['currentSn']
+                init_count = len(cl['sn']) - 1
+                break
+    return init_sn, init_count, pinfo
+
+
 def check_prodStorage_func_for_pool(thepoolname, thepoolurl, *args):
     ps = ProductStorage(poolurl=thepoolurl)
     p1 = ps.getPools()[0]
     # get the pool object
     pspool = ps.getPool(p1)
-
     x = Product(description="This is my product example",
                 instrument="MyFavourite", modelName="Flight")
+#    y = MapContext(description='Keep it all un context.')
     pcq = fullname(x)
+    mcq = fullname(MapContext)
+
+    is_csdb = thepoolurl.startswith('csdb://')
+    if is_csdb:
+        csdb = PoolManager.getPool(thepoolname)
+        init_sn, init_count, pinfo = getCurrSnCount(csdb, pcq)
+        init_sn_m, init_count_m, _ = getCurrSnCount(pinfo, mcq)
+        print('1...', init_count, init_sn, init_count_m, init_sn_m)
+
     # save
     ref = ps.save(x)
+    s0, s1 = tuple(ref.urn.rsplit(':', 1))
+    assert s0 == 'urn:' + thepoolname + ':' + pcq
+    #ref_m = ps.save(y)
+    init_count, init_count_m = 0, 0
+    is_csdb = thepoolurl.startswith('csdb://')
+    if is_csdb:
+        currentsn2, count2, pinfo2 = getCurrSnCount(csdb, pcq)
+        currentsn_m2, count_m2, _ = getCurrSnCount(pinfo2, mcq)
+        print('2...', count2, currentsn2, count_m2, currentsn_m2)
+
+        assert count2 == init_count + 1
+        assert currentsn2 == init_sn + 1
+        assert init_count_m == init_count_m
+        assert currentsn_m2 == init_sn_m
+
+        # XXX this 1 is due to a bug in csdb implementation
+        init_sn += 1
+        init_sn_m += 1
+    else:
+        init_sn = 0
+        init_count = 0
+        init_sn_m = 0
+        init_count_m = 0
+
     # ps has 1 prod
-    assert ref.urn == 'urn:' + thepoolname + ':' + pcq + ':0'
-    checkdbcount(1, thepoolurl, pcq, 0, *args)
+    assert init_sn == int(s1)
+    checkdbcount(init_count+1, thepoolurl, pcq, init_sn + 0, *args)
+    #checkdbcount(init_count_m+0, thepoolurl, mcq, init_sn_m + 0, *args)
 
     # save more
     # one by one
     q = 3
     x2, ref2 = [], []
 
-    for d in range(q):  # The 1st one is a mapcontext, rest product
+    for d in range(q):
+        # The 1st one is a mapcontext, rest product
         tmp = Product(description='x' + str(d)
                       ) if d > 0 else MapContext(description='x0')
         x2.append(tmp)
         ref2.append(ps.save(tmp, tag='t' + str(d)))
 
-    checkdbcount(q, thepoolurl, pcq, q - 1, *args)
-    checkdbcount(1, thepoolurl, fullname(MapContext), 0, *args)
+    checkdbcount(init_count + q, thepoolurl, pcq,
+                 init_sn + q - 1, *args)
+    checkdbcount(init_count_m+1, thepoolurl, mcq, init_sn_m+0, *args)
     # save many in one go
     m, x3 = 2, []
     n = q + m
@@ -552,22 +655,24 @@ def check_prodStorage_func_for_pool(thepoolname, thepoolurl, *args):
     x2 += x3  # there are n prods in x2
     # check refs
     assert len(ref2) == n
-    checkdbcount(n, thepoolurl, pcq, n, *args)
-    checkdbcount(1, thepoolurl, fullname(MapContext), 0, *args)
+    checkdbcount(init_count+n, thepoolurl, pcq, init_sn + n - 1, *args)
+    checkdbcount(init_count_m+1, thepoolurl, mcq, init_sn_m+0, *args)
 
     # tags
-    ts = ps.getAllTags()
-    assert len(ts) == q + 1
-    ts = ps.getTags(ref2[0].urn)
-    assert len(ts) == 1
-    assert ts[0] == 't0'
-    u = ps.getUrnFromTag('all-tm')
-    assert len(u) == m
-    assert u[0] == ref2[q].urn
+    # XXX fix getUrn() and getTag() when no arg   is given
+    if not is_csdb:
+        ts = ps.getAllTags()
+        assert len(ts) == q + 1
+        ts = ps.getTags(ref2[0].urn)
+        assert len(ts) == 1
+        assert ts[0] == 't0'
+        u = ps.getUrnFromTag('all-tm')
+        assert len(u) == m
+        assert u[0] == ref2[q].urn
 
     # access resource
-    checkdbcount(n, thepoolurl, pcq, n, *args)
-    checkdbcount(1, thepoolurl, fullname(MapContext), 0, *args)
+    checkdbcount(init_count+n, thepoolurl, pcq, init_sn+n-1, *args)
+    checkdbcount(init_count_m+1, thepoolurl, mcq, init_sn_m+0, *args)
     # get ref from urn
     pref = ps.load(ref2[n - 2].urn)
     assert pref == ref2[n - 2]
@@ -583,15 +688,16 @@ def check_prodStorage_func_for_pool(thepoolname, thepoolurl, *args):
     # DB shows less in record
     # current serial number not changed
     # number of items decreased by 1
-    checkdbcount(n - 1, thepoolurl, pcq, n, *args)
-    checkdbcount(1, thepoolurl, fullname(MapContext), 0, *args)
+    checkdbcount(init_count + n - 1, thepoolurl,
+                 pcq, init_sn+n-1, *args)
+    checkdbcount(init_count_m+1, thepoolurl, mcq, init_sn_m+0, *args)
 
     # report lru_cache info
     print('****** %s ' % str(type(pspool)), pspool.getCacheInfo())
 
     # clean up a pool
     ps.wipePool()
-    checkdbcount(0, thepoolurl, pcq, -1, *args)
+    checkdbcount(0, thepoolurl, pcq, None, *args)
     assert ps.getPool(thepoolname).isEmpty()
 
 
@@ -644,15 +750,25 @@ def test_ProdStorage_func_server(local_pools_dir):
     check_prodStorage_func_for_pool(thepoolname, thepoolurl, None)
 
 
+def test_ProdStorage_func_csdb(csdb):
+    test_pool, url = csdb  # csdb:///csdb_test_pool
+    thepoolname = 'csdb_test_pool'
+    thepoolurl = 'csdb:///' + thepoolname
+
+    cleanup(thepoolurl, thepoolname)
+    check_prodStorage_func_for_pool(thepoolname, thepoolurl, None)
+
+
 def test_LocalPool():
     thepoolname = 'localpool_' + Test_Pool_Name
     thepoolpath = '/tmp/fditest'
     thepoolurl = 'file://' + thepoolpath + '/' + thepoolname
 
+    cleanup(thepoolurl, thepoolname)
     ps = ProductStorage(thepoolname, thepoolurl)
-    p1 = ps.getPools()[0]
+    pname = ps.getPools()[0]
     # get the pool object
-    pspool = ps.getPool(p1)
+    pspool = ps.getPool(pname)
 
     x = Product(description="This is my product example",
                 instrument="MyFavourite", modelName="Flight")
@@ -662,7 +778,7 @@ def test_LocalPool():
 
     # read HK
     # copy default pool data in memory
-    ps1 = copy.deepcopy(pspool)
+    p1 = pspool
     # rename the pool
     cpn = thepoolname + '_copy'
     cpu = thepoolurl + '_copy'
@@ -674,12 +790,51 @@ def test_LocalPool():
     ps2 = ProductStorage(pool=cpn, poolurl=cpu)
     # two ProdStorage instances have the same DB
     p2 = ps2.getPool(ps2.getPools()[0])
-    assert deepcmp(ps1._urns, p2._urns) is None
-    assert deepcmp(ps1._tags, p2._tags) is None
-    assert deepcmp(ps1._classes, p2._classes) is None
+    #assert deepcmp(p1._urns, p2._urns) is None
+    #assert deepcmp(p1._tags, p2._tags) is None
+    #assert deepcmp(p1._classes, p2._classes) is None
+    assert deepcmp(p1._dTypes, p2._dTypes) is None
+    assert deepcmp(p1._dTags, p2._dTags) is None
+
+    backup_restore(ps)
 
 
-def mkStorage(thepoolname, thepoolurl):
+def backup_restore(ps):
+    p1 = ps.getPool(ps.getPools()[0])
+    hk1 = p1.readHK()
+    cpn = p1._poolname + '_2'
+    cpu = p1._poolurl + '_2'
+    pstore = ProductStorage(pool=cpn, poolurl=cpu)
+    # backup
+    # the new pool is made empty
+    pstore.wipePool()
+    # register
+    pstore = ProductStorage(pool=cpn, poolurl=cpu)
+    # save something to the new pool
+    x = Product(description="This is my product 2")
+    ref = pstore.save(x, tag='i think')
+    ref2 = pstore.save(x, tag='i think')
+    assert ref != ref2
+    # two pools are different
+    p2 = pstore.getPool(pstore.getPools()[0])
+    assert deepcmp(hk1, p2.readHK()) is not None
+
+    # make a backup tarfile
+    tar = p1.backup()
+    os.makedirs('/tmp/fditest', exist_ok=True)
+    with open('/tmp/fditest/bk.tar', 'wb') as f:
+        f.write(tar)
+    with open('/tmp/fditest/bk.tar', 'rb') as f:
+        tar2 = f.read()
+    assert tar == tar2
+    # restore
+    lst = p2.restore(tar2)
+    # print(lst)
+    # two pools are the same
+    assert deepcmp(hk1, p2.readHK()) is None
+
+
+def mkStorage(thepoolname, thepoolurl, db=None):
     """ returns pool object and productStorage """
 
     cleanup(thepoolurl, thepoolname)
@@ -751,6 +906,9 @@ def doquery(poolpath, newpoolpath):
     # [T T T C] [C M M]
     #  0 1 2 3   4 5 6
 
+    assert thepool.count == 4
+    assert newpool.count == 3
+
     # query with a specific parameter in all products' metadata, which is the variable 'm' in the query expression, i.e. ``m = product.meta; ...``
     m = 2
     q = MetaQuery(TP, 'm["description"].value == "%s"' % rec1[m]['a0'])
@@ -841,6 +999,10 @@ def doquery(poolpath, newpoolpath):
     with pytest.raises(KeyError):
         res = pstore.select(q)
 
+    q = MetaQuery(Context, 'm["this_and_the_last_errors_are_expected"] < 5006')
+    with pytest.raises(KeyError):
+        res = pstore.select(q)
+
     # all 'time' < 5006 mapcontext. all in newpool
     q = MetaQuery(MapContext, 'm["time"] < 5006')
     res = pstore.select(q)
@@ -885,6 +1047,21 @@ def doquery(poolpath, newpoolpath):
     chk(res[0], rec1[3])
     chk(res[1], rec1[4])
 
+    q = '"n 1" in p.creator'
+    resu = thepool.where(q)
+    res = [ProductRef(u) for u in resu]
+    # [3]
+    assert len(res) == 1, str(res)
+    chk(res[0], rec1[3])
+
+    resu = thepool.where('p.meta["extra"] > 5000 and p.meta["extra"] < 5004')
+    res = [ProductRef(u) for u in resu]
+    # [1,2,3]
+    assert len(res) == 3, str(res)
+    chk(res[0], rec1[1])
+    chk(res[1], rec1[2])
+    chk(res[2], rec1[3])
+
     # report lru_cache info
     print('****** %s ' % str(type(thepool)), thepool.getCacheInfo())
 
@@ -904,6 +1081,7 @@ def test_query_http(server):
     aburl = aburl.rstrip('/')
     cleanup()
     lpath = '/tmp'
+    ### TODO: http
     doquery(aburl, aburl)
     doquery('file://'+lpath, aburl)
     doquery('mem://'+lpath, aburl)
@@ -1115,6 +1293,17 @@ def test_realistic_http(server, demo_product):
     # remove existing pools in memory
     # clean up possible garbage of previous runs. use class method to avoid reading pool hk info during ProdStorage initialization.
     thepool, pstore = mkStorage(poolname, poolurl)
+    pass
+
+
+def test_realistic_csdb(csdb, demo_product):
+    test_pool, url = csdb  # csdb:///csdb_test_pool
+    poolname = 'test_csdb'
+    poolurl = 'csdb:///' + poolname
+    cleanup(poolurl, poolname)
+    # remove existing pools in memory
+    # clean up possible garbage of previous runs. use class method to avoid reading pool hk info during ProdStorage initialization.
+    thepool, pstore = mkStorage(poolname, poolurl, test_pool)
 
     p1 = Product(description='p1')
     p2 = Product(description='p2')
