@@ -1,6 +1,5 @@
 # -*- coding: utf-8 -*-
 
-from fdi.utils.fits_kw import FITS_KEYWORDS, getFitsKw
 from fdi.utils.leapseconds import utc_to_tai, tai_to_utc, dTAI_UTC_from_utc, _fallback
 from fdi.dataset.eq import deepcmp
 from fdi.dataset.metadata import make_jsonable
@@ -12,28 +11,42 @@ from fdi.pal.productref import ProductRef
 from fdi.utils.checkjson import checkjson
 from fdi.utils.loadfiles import loadcsv
 from fdi.utils import moduleloader
-from fdi.utils.common import fullname, wls
+from fdi.utils.common import fullname, wls, find_all_files
 from fdi.utils.options import opt
 from fdi.utils.fetch import fetch
+from fdi.utils.tree import tree
 from fdi.utils.loadfiles import loadMedia
+from fdi.utils.validator import getValidator, validateJson
 
 import traceback
 import copy
 from datetime import timezone, timedelta, datetime
 import sys
 import os
+import json
 import hashlib
-import os.path
+import os.path as op
 import pytest
+import numpy as np
+from astropy.io import fits
 
 if sys.version_info[0] >= 3:  # + 0.1 * sys.version_info[1] >= 3.3:
     PY3 = True
 else:
     PY3 = False
 
+# make format output in /tmp/outputs.py
+mk_outputs = 0
+output_write = 'tests/outputs_utils.py'
+
+if mk_outputs:
+    with open(output_write, 'wt', encoding='utf-8') as f:
+        f.write('# -*- coding: utf-8 -*-\n')
+
 if __name__ == '__main__' and __package__ == 'tests':
     # run by python -m tests.test_dataset
-    pass
+    if not mk_outputs:
+        from outputs_utils import out_tree
 else:
     # run by pytest
 
@@ -42,6 +55,9 @@ else:
 
     from pycontext import fdi
 
+    if not mk_outputs:
+        from outputs_utils import out_tree
+
     from logdict import logdict
     import logging
     import logging.config
@@ -62,9 +78,10 @@ def test_get_demo_product(demo_product):
     assert v['Browse'].data[1:4] == b'PNG'
     # print(v.yaml())
     p = v.getDefault()
-    assert p == v['results']
+    assert p == v['measurements']
     aref = ProductRef(related)
-    v.refs['a'] = aref
+    v.refs['a_reference'] = aref
+    v.refs['a_different_name'] = aref
     r0 = v.refs
     p['dset'] = 'foo'
     # refs is always the last
@@ -74,6 +91,8 @@ def test_get_demo_product(demo_product):
     p['dset'] = 'foo'
     assert list(v.keys())[-1] == 'refs'
     assert r0 == v.refs
+    s = r0.string(level=0)
+    # print(s)
     checkjson(v, dbg=0)
     checkgeneral(v)
 
@@ -147,20 +166,20 @@ def chk_sample_pd(p):
     v, s = fetch(["Environment Temperature", "T0", "tai"], p)
     assert v == FineTime('2020-02-02T20:20:20.0202').tai
 
-    # a 2D array dataset in compositedataset 'results'
-    v, s = fetch(["results", 'calibration', "unit"], p)
+    # a 2D array dataset in compositedataset 'measurements'
+    v, s = fetch(["measurements", 'calibration', "unit"], p)
     assert v == 'count'
-    assert v == p['results']['calibration'].unit
-    assert s == '["results"]["calibration"].unit'
+    assert v == p['measurements']['calibration'].unit
+    assert s == '["measurements"]["calibration"].unit'
 
     # data of a column in tabledataset within compositedataset
-    v, s = fetch(["results", "Time_Energy_Pos", "Energy", "data"], p)
+    v, s = fetch(["measurements", "Time_Energy_Pos", "Energy", "data"], p)
     t = [x * 1.0 for x in range(len(v))]
     assert v == [2 * x + 100 for x in t]
-    assert v == p['results']['Time_Energy_Pos']['Energy'].data
-    assert s == '["results"]["Time_Energy_Pos"]["Energy"].data'
-    ys, s = fetch(["results", "Time_Energy_Pos", "y"], p)
-    zs, s = fetch(["results", "Time_Energy_Pos", "z"], p)
+    assert v == p['measurements']['Time_Energy_Pos']['Energy'].data
+    assert s == '["measurements"]["Time_Energy_Pos"]["Energy"].data'
+    ys, s = fetch(["measurements", "Time_Energy_Pos", "y"], p)
+    zs, s = fetch(["measurements", "Time_Energy_Pos", "z"], p)
     # y^2 + z^2 = 100 for all t
     assert all((y*y + z*z - 100) < 1e-5 for y, z in zip(ys.data, zs.data))
 
@@ -203,7 +222,7 @@ def test_fetch(demo_product):
 
     class al(list):
         ala = 0
-        @staticmethod
+        @ staticmethod
         def alb(): pass
 
         def alf(self, a, b=9):
@@ -277,21 +296,53 @@ def test_fetch(demo_product):
     chk_sample_pd(p)
 
 
-def test_Fits_Kw():
-    # Fits to Parameter name
-    assert FITS_KEYWORDS['DATASUM'] == 'checksumData'
-    assert FITS_KEYWORDS['DESC'] == 'description'
-    assert FITS_KEYWORDS['CUNIT'] == 'cunit'
-    assert getFitsKw('checksumData') == 'DATASUM'
-    assert getFitsKw('description') == 'DESC'
-    assert getFitsKw('cunit01') == 'CUNIT01'
-    assert getFitsKw('description01234') == 'DESC34'
-    # with pytest.raises(ValueError):
-    assert getFitsKw('checksumData0123', 3) == 'DATAS123'
-    assert getFitsKw('checksumData0123', 2) == 'DATASU23'
-    with pytest.raises(TypeError):
-        assert getFitsKw('foo0123', 5, (('foo', 'BAR'))) == 'BAR0123'
-    assert getFitsKw('foo0123', 5, (('foo', 'BAR'),)) == 'BAR0123'
+def test_tree(demo_product):
+    p, r = demo_product
+    # test output
+    ts = 'tree out_tree'
+    v = tree(p)
+    ts += "\n" + '\n'.join(v)
+    v = tree(p, level=1)
+    ts += "\n" + '\n'.join(v)
+    v = tree(p, level=1, style='ascii')
+    ts += "\n" + '\n'.join(v)
+    if mk_outputs:
+        print(ts)
+        with open(output_write, 'a') as f:
+            clsn = 'out_tree'
+            f.write('%s = """%s"""\n' % (clsn, ts))
+    else:
+        assert ts == out_tree
+
+
+def test_find_all_files():
+    tdir = '/tmp/test_find_all_files'
+    if os.path.exists(tdir) and tdir != '/':
+        os.system('rm -rf '+tdir)
+    os.makedirs(tdir+'/sub', exist_ok=True)
+    os.system('cd %s; touch a.jsn b.json sub/c.jsn' % tdir)
+    # exclude nothing
+    fs = find_all_files(tdir, not_if=lambda x: False)
+    assert set(fs) == {tdir+'/a.jsn', tdir+'/b.json', tdir+'/sub'}
+    # exclude directories
+    fs = find_all_files(tdir)
+    assert set(fs) == {tdir+'/a.jsn', tdir+'/b.json'}
+    # only with extension
+    fs = find_all_files(tdir, include='*.jsn')
+    assert set(fs) == {tdir+'/a.jsn'}
+    # recursive, no directories
+    fs = find_all_files(tdir, include='**/*')
+    assert set(fs) == {tdir+'/a.jsn', tdir+'/b.json',
+                       tdir+'/sub/c.jsn'}
+    # recursive, no directories
+    fs = find_all_files(tdir, include='**/*', not_if=lambda x: False)
+    assert set(fs) == {tdir+'/a.jsn', tdir+'/b.json',
+                       tdir+'/sub', tdir+'/sub/c.jsn'}
+    # recursive, no directories, limit names
+    fs = find_all_files(tdir, include='**/*.jsn')
+    assert set(fs) == {tdir+'/a.jsn', tdir+'/sub/c.jsn'}
+    fs = find_all_files(tdir, include='**/*.js*n')
+    assert set(fs) == {tdir+'/a.jsn', tdir+'/b.json', tdir+'/sub/c.jsn'}
 
 
 def test_loadcsv():
@@ -349,8 +400,8 @@ def test_loadcsv():
 
 def test_loadMedia():
     fname = 'bug.gif'
-    fname = os.path.join(os.path.join(os.path.abspath(os.path.dirname(__file__)),
-                                      'resources'), fname)
+    fname = op.join(op.join(op.abspath(op.dirname(__file__)),
+                            'resources'), fname)
     image = loadMedia(fname, 'image/gif')
     ho = hashlib.md5()
     ho.update(image.data)
@@ -360,7 +411,7 @@ def test_loadMedia():
 
 def test_moduleloader():
 
-    moduleloader.main(ipath=os.path.abspath('tests'))
+    moduleloader.main(ipath=op.abspath('tests'))
 
 
 def test_fullname():
@@ -392,8 +443,11 @@ def test_wls():
     assert wls('格a式版本', 4) == '格a\n式版\n本'
     assert wls('格式a版本', 4) == '格式\na版\n本'
     assert wls('格式版a本', 4) == '格式\n版a\n本'
-    assert wls('\ta', 1) == '#\na'
-    assert wls('') == '#'
+    assert wls('1\ta', 4) == '1   \n    \na'
+    # unprintable activated by wide chars
+    assert wls('格\ta', 4) == '格#a'
+    assert wls('') == ''
+    assert wls('格') == '#格'
     # \r and \r\n etc are taken out as line-breaks.
     # ref https://docs.python.org/3.6/library/stdtypes.html#str.splitlines
     # the last \n is removed
@@ -458,14 +512,14 @@ def test_opt(caplog):
 
 def check_conf(cfp, typ, getConfig):
     cfn = typ + 'local.py'
-    cfp = os.path.expanduser(cfp)
-    filec = os.path.join(cfp, cfn)
+    cfp = op.expanduser(cfp)
+    filec = op.join(cfp, cfn)
     os.system('rm -f ' + filec)
     conf = 'import os; %sconfig={"jk":98, "m":os.path.abspath(__file__)}' % typ
     with open(filec, 'w') as f:
         f.write(conf)
     # check conf file directory
-    w = getConfig(conf=typ)
+    w = getConfig(conf=typ, force=True)
     assert w['jk'] == 98
     pfile = w['m']
     assert pfile.startswith(cfp)
@@ -528,3 +582,17 @@ def test_leapseconds():
     assert utc_to_tai(t4) - utc_to_tai(t4 - timedelta(seconds=1)) == \
         timedelta(seconds=2)
     print(_fallback.cache_info())
+
+
+def test_validator():
+    fdi_sch_dir = op.join(op.abspath(op.dirname(__file__)), '../fdi/schemas')
+    sch_dir = op.join(op.abspath(op.dirname(__file__)), 'resources/schemas')
+    sch_path = op.join(sch_dir, 'prd_schema.jsn')
+    with open(sch_path, 'r') as file:
+        sch = json.load(file)
+
+    from fdi.dataset.baseproduct import BaseProduct
+    jsn = BaseProduct().zInfo
+
+    assert getValidator(sch, schema_dir=fdi_sch_dir,
+                        verbose=1).validate(jsn) is None
